@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,65 +22,61 @@ This contains tools to compare, classify and test constants.
 
 import math
 
-# pylint: disable=W0622
-from .__past__ import long, unicode, iterItems
-# pylint: enable=W0622
+from nuitka.utils.Utils import python_version
 
+from .__past__ import iterItems, long, unicode  # pylint: disable=W0622
 from .Builtins import builtin_anon_names
-from .Utils import python_version
 
-NoneType = type( None )
+NoneType = type(None)
 
 def compareConstants(a, b):
     # Many many cases to deal with, pylint: disable=R0911,R0912
 
     # Supposed fast path for comparison.
-    if type( a ) is not type( b ):
+    if type(a) is not type(b):
         return False
 
     # Now it's either not the same, or it is a container that contains NaN or it
     # is a complex or float that is NaN, the other cases can use == at the end.
-    if type( a ) is complex:
-        return compareConstants( a.imag, b.imag ) and \
-               compareConstants( a.real, b.real )
+    if type(a) is complex:
+        return compareConstants(a.imag, b.imag) and \
+               compareConstants(a.real, b.real)
 
-    if type( a ) is float:
+    if type(a) is float:
         # Check sign first, -0.0 is not 0.0, or -nan is not nan, it has a
         # different sign for a start.
-        if math.copysign( 1.0, a ) != math.copysign( 1.0, b ):
+        if math.copysign(1.0, a) != math.copysign(1.0, b):
             return False
 
-        if math.isnan( a ) and math.isnan( b ):
+        if math.isnan(a) and math.isnan(b):
             return True
 
         return a == b
 
-    if type( a ) in ( tuple, list ):
-        if len( a ) != len( b ):
+    if type(a) in (tuple, list):
+        if len(a) != len(b):
             return False
 
-        for ea, eb in zip( a, b ):
-            if not compareConstants( ea, eb ):
+        for ea, eb in zip(a, b):
+            if not compareConstants(ea, eb):
                 return False
-        else:
-            return True
+        return True
 
-    if type( a ) is dict:
-        if len( a ) != len( b ):
+    if type(a) is dict:
+        if len(a) != len(b):
             return False
 
-        for ea1, ea2 in iterItems( a ):
-            for eb1, eb2 in iterItems( b ):
-                if compareConstants( ea1, eb1 ) and \
-                   compareConstants( ea2, eb2 ):
+        for ea1, ea2 in iterItems(a):
+            for eb1, eb2 in iterItems(b):
+                if compareConstants(ea1, eb1) and \
+                   compareConstants(ea2, eb2):
                     break
             else:
                 return False
-        else:
-            return True
+        return True
 
-    if type( a ) in ( frozenset, set ):
-        if len( a ) != len( b ):
+    if type(a) in (frozenset, set):
+        if len(a) != len(b):
             return False
 
         for ea in a:
@@ -88,23 +84,32 @@ def compareConstants(a, b):
                 # Due to NaN values, we need to compare each set element with
                 # all the other set to be really sure.
                 for eb in b:
-                    if compareConstants( ea, eb ):
+                    if compareConstants(ea, eb):
                         break
                 else:
                     return False
-        else:
-            return True
+        return True
 
-    if type( a ) is range:
-        return str( a ) == str( b )
+    if type(a) is range:
+        return str(a) == str(b)
 
     # The NaN values of float and complex may let this fail, even if the
     # constants are built in the same way.
     return a == b
 
-# These builtin type references are kind of constant too. TODO: The list is
+# These built-in type references are kind of constant too. TODO: The list is
 # totally not complete.
-constant_builtin_types = int, set, str, float, list, tuple, dict, complex
+constant_builtin_types = (
+    int,
+    str,
+    float,
+    list,
+    tuple,
+    set,
+    dict,
+    slice,
+    complex
+)
 
 if python_version >= 300:
     constant_builtin_types += (
@@ -116,39 +121,40 @@ else:
         unicode,
         long,
         # This has no name in Python, but the natural one in C-API.
-        builtin_anon_names[ "instance" ]
+        builtin_anon_names["instance"]
     )
 
 def isConstant(constant):
     # Too many cases and all return, that is how we do it here,
-    # pylint: disable=R0911,R0912
+    # pylint: disable=R0911
 
-    constant_type = type( constant )
+    constant_type = type(constant)
 
     if constant_type is dict:
-        for key, value in iterItems( constant ):
-            if not isConstant( key ):
+        for key, value in iterItems(constant):
+            if not isConstant(key):
                 return False
-            if not isConstant( value ):
+            if not isConstant(value):
                 return False
-        else:
-            return True
-    elif constant_type in ( tuple, list ):
-        for element_value in constant:
-            if not isConstant( element_value ):
-                return False
-        else:
-            return True
-    elif constant_type in ( str, unicode, complex, int, long, bool, float,
-                            NoneType, range, bytes, set ):
         return True
-    elif constant in ( Ellipsis, NoneType ):
+    elif constant_type in (tuple, list):
+        for element_value in constant:
+            if not isConstant(element_value):
+                return False
+        return True
+    elif constant_type in (str, unicode, complex, int, long, bool, float,
+                           NoneType, range, bytes, set):
+        return True
+    elif constant in (Ellipsis, NoneType):
         return True
     elif constant_type is type:
         return constant in constant_builtin_types
+    elif constant_type is slice:
+        return True
     else:
         return False
 
+
 def isMutable(constant):
     """ Is a constant mutable
 
@@ -156,66 +162,76 @@ def isMutable(constant):
         a prime example of mutable, dictionaries are mutable.
     """
 
-    constant_type = type( constant )
+    constant_type = type(constant)
 
-    if constant_type in ( str, unicode, complex, int, long, bool, float,
-                          NoneType, range, bytes ):
+    if constant_type in (str, unicode, complex, int, long, bool, float,
+                         NoneType, range, bytes, slice):
         return False
-    elif constant_type in ( dict, list, set ):
+    elif constant_type in (dict, list, set):
         return True
     elif constant_type is tuple:
         for value in constant:
-            if isMutable( value ):
+            if isMutable(value):
                 return True
-        else:
-            return False
+        return False
     elif constant is Ellipsis:
         return False
     elif constant in constant_builtin_types:
         return True
+    else:
+        assert False, repr(constant)
+
+
+def isHashable(constant):
+    """ Is a constant hashable
+
+        That means a user of a reference to it, can use it for dicts and set
+        keys. This is distinct from mutable, there is one types that is not
+        mutable, and still not hashable: slices.
+    """
+    # Too many cases and all return, that is how we do it here,
+    # pylint: disable=R0911
+
+    constant_type = type(constant)
+
+    if constant_type in (str, unicode, complex, int, long, bool, float,
+                         NoneType, range, bytes):
+        return True
+    elif constant_type in (dict, list, set):
+        return False
+    elif constant_type is tuple:
+        for value in constant:
+            if not isHashable(value):
+                return False
+        return True
+    elif constant is Ellipsis:
+        return True
+    elif constant in constant_builtin_types:
+        return True
+    elif constant_type is slice:
+        return False
     else:
         assert False, constant_type
 
+
 def isIterableConstant(constant):
-    return type( constant ) in (
+    return type(constant) in (
         str, unicode, list, tuple, set, frozenset, dict, range, bytes
     )
 
-def getConstantIterationLength(constant):
-    assert isIterableConstant( constant )
 
-    return len( constant )
+def getConstantIterationLength(constant):
+    assert isIterableConstant(constant)
+
+    return len(constant)
+
 
 def isNumberConstant(constant):
-    return type( constant ) in ( int, long, float, bool )
+    return type(constant) in (int, long, float, bool)
+
 
 def isIndexConstant(constant):
-    return type( constant ) in ( int, long, bool )
-
-class HashableConstant:
-    def __init__(self, constant):
-        self.constant = constant
-
-        try:
-            # For Python3: range objects with same ranges give different hash
-            # values. It's not even funny, is it.
-            if type( constant ) is range:
-                raise TypeError
-
-            self.hash = hash( constant )
-        except TypeError:
-            self.hash = 55
-
-    def getConstant(self):
-        return self.constant
-
-    def __hash__(self):
-        return self.hash
-
-    def __eq__(self, other):
-        assert isinstance( other, self.__class__ )
-
-        return compareConstants( self.constant, other.constant )
+    return type(constant) in (int, long, bool)
 
 
 def createConstantDict(keys, values, lazy_order):
@@ -233,7 +249,29 @@ def createConstantDict(keys, values, lazy_order):
             None
         )
 
-    for key, value in zip( keys, values ):
+    for key, value in zip(keys, values):
         constant_value[ key ] = value
 
     return constant_value
+
+
+def getConstantWeight(constant):
+    constant_type = type(constant)
+
+    if constant_type is dict:
+        result = 0
+
+        for key, value in iterItems(constant):
+            result += getConstantWeight(key)
+            result += getConstantWeight(value)
+
+        return result
+    elif constant_type in (tuple, list, set, frozenset):
+        result = 0
+
+        for element_value in constant:
+            result += getConstantWeight(element_value)
+
+        return result
+    else:
+        return 1
