@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -28,16 +28,12 @@ import subprocess
 import sys
 from logging import info, warning
 
-from . import (
-    Importing,
-    ModuleRegistry,
-    Options,
-    PythonVersions,
-    SyntaxErrors,
-    Tracing,
-    TreeXML,
-    Utils
-)
+from nuitka.importing import Importing, Recursion
+from nuitka.plugins.PluginBase import Plugins
+from nuitka.tree import SyntaxErrors
+from nuitka.utils import InstanceCounters, Utils
+
+from . import ModuleRegistry, Options, Tracing, TreeXML
 from .build import SconsInterface
 from .codegen import CodeGeneration, ConstantCodes
 from .finalizations import Finalization
@@ -52,7 +48,7 @@ from .freezer.Standalone import (
     detectLateImports
 )
 from .optimizations import Optimization
-from .tree import Building, Recursion
+from .tree import Building
 
 
 def createNodeTree(filename):
@@ -97,6 +93,12 @@ def createNodeTree(filename):
             module_package  = None
         )
 
+    for pattern in Options.getShallFollowExtraFilePatterns():
+        Recursion.checkPluginFilenamePattern(
+            pattern = pattern
+        )
+
+
     # Then optimize the tree and potentially recursed modules.
     Optimization.optimize()
 
@@ -109,7 +111,7 @@ def dumpTreeXML(tree):
 
 def displayTree(tree):
     # Import only locally so the Qt4 dependency doesn't normally come into play
-    # when it's not strictly needed, pylint: disable=W0404
+    # when it's not strictly needed.
     from .gui import TreeDisplay
 
     TreeDisplay.displayTreeInspector(tree)
@@ -142,7 +144,7 @@ def getResultBasepath(main_module):
 
     if Options.isStandaloneMode():
         return Utils.joinpath(
-            getStandaloneDirectoryPath( main_module ),
+            getStandaloneDirectoryPath(main_module),
             Utils.basename(
                 getTreeFilenameWithSuffix(main_module, "")
             )
@@ -205,38 +207,50 @@ def pickSourceFilenames(source_dir, modules):
     collision_filenames = set()
     seen_filenames = set()
 
-    for module in sorted(modules, key = lambda x : x.getFullName()):
-        base_filename = Utils.joinpath(source_dir, module.getFullName())
-
-        # Note: Could detect if the filesystem is cases sensitive in source_dir
-        # or not, but that's probably not worth the effort.
-        collision_filename = Utils.normcase( base_filename )
-
-        if collision_filename in seen_filenames:
-            collision_filenames.add(collision_filename)
-
-        seen_filenames.add(collision_filename)
-
-    collision_counts = {}
-
+    # Our output.
     module_filenames = {}
 
-    for module in sorted(modules, key = lambda x : x.getFullName()):
-        if module.isPythonShlibModule():
-            continue
-
-        base_filename = Utils.joinpath(
+    def getFilenames(module):
+        base_filename =  Utils.joinpath(
             source_dir,
             "module." + module.getFullName()
               if not module.isInternalModule() else
             module.getFullName()
         )
 
-        collision_filename = Utils.normcase( base_filename )
+        # Note: Could detect if the file system is cases sensitive in source_dir
+        # or not, but that's probably not worth the effort. False positives do
+        # no harm at all.
+        collision_filename = Utils.normcase(base_filename)
+
+        return base_filename, collision_filename
+
+    # First pass, check for collisions.
+    for module in modules:
+        if module.isPythonShlibModule():
+            continue
+
+        base_filename = base_filename, collision_filename = getFilenames(module)
+
+        if collision_filename in seen_filenames:
+            collision_filenames.add(collision_filename)
+
+        seen_filenames.add(collision_filename)
+
+    # Count up for colliding filenames.
+    collision_counts = {}
+
+    # Second pass, this time sorted, so we get deterministic results. We will
+    # apply an @1/@2 to disambiguate the filenames.
+    for module in sorted(modules, key = lambda x : x.getFullName()):
+        if module.isPythonShlibModule():
+            continue
+
+        base_filename = base_filename, collision_filename = getFilenames(module)
 
         if collision_filename in collision_filenames:
             collision_counts[ collision_filename ] = \
-              collision_counts.get( collision_filename, 0 ) + 1
+              collision_counts.get(collision_filename, 0) + 1
             hash_suffix = "@%d" % collision_counts[ collision_filename ]
         else:
             hash_suffix = ""
@@ -334,17 +348,17 @@ def makeSourceDirectory(main_module):
                 )
 
             writeSourceCode(
-                filename     = cpp_filename,
-                source_code  = source_code
+                filename    = cpp_filename,
+                source_code = source_code
             )
 
             if Options.isShowInclusion():
-                info("Included compiled module '%s'.", module.getFullName())
+                info("Included compiled module '%s'." % module.getFullName())
 
         elif module.isPythonShlibModule():
             target_filename = Utils.joinpath(
                 getStandaloneDirectoryPath(main_module),
-                *module.getFullName().split(".")
+                *module.getFullName().split('.')
             )
 
             if Utils.getOS() == "Windows":
@@ -369,8 +383,8 @@ def makeSourceDirectory(main_module):
             assert False, module
 
     writeSourceCode(
-        filename    = Utils.joinpath( source_dir, "__constants.cpp" ),
-        source_code = CodeGeneration.generateConstantsDefinitionCode(
+        filename    = Utils.joinpath(source_dir, "__constants.cpp"),
+        source_code = ConstantCodes.getConstantsDefinitionCode(
             context = global_context
         )
     )
@@ -378,12 +392,12 @@ def makeSourceDirectory(main_module):
     helper_decl_code, helper_impl_code = CodeGeneration.generateHelpersCode()
 
     writeSourceCode(
-        filename    = Utils.joinpath( source_dir, "__helpers.hpp" ),
+        filename    = Utils.joinpath(source_dir, "__helpers.hpp"),
         source_code = helper_decl_code
     )
 
     writeSourceCode(
-        filename    = Utils.joinpath( source_dir, "__helpers.cpp" ),
+        filename    = Utils.joinpath(source_dir, "__helpers.cpp"),
         source_code = helper_impl_code
     )
 
@@ -392,16 +406,15 @@ def runScons(main_module, quiet):
     # Scons gets transported many details, that we express as variables, and
     # have checks for them, leading to many branches, pylint: disable=R0912
 
-    python_version = "%d.%d" % ( sys.version_info[0], sys.version_info[1] )
+    python_version = "%d.%d" % (sys.version_info[0], sys.version_info[1])
 
     if hasattr(sys, "abiflags"):
-        # The Python3 for some platforms has sys.abiflags pylint: disable=E1101
         if Options.isPythonDebug() or \
            hasattr(sys, "getobjects"):
-            if sys.abiflags.startswith("d"):
+            if sys.abiflags.startswith('d'):
                 python_version += sys.abiflags
             else:
-                python_version += "d" + sys.abiflags
+                python_version += 'd' + sys.abiflags
         else:
             python_version += sys.abiflags
 
@@ -409,23 +422,24 @@ def runScons(main_module, quiet):
         return "true" if value else "false"
 
     options = {
-        "name"           : Utils.basename(
+        "name"            : Utils.basename(
             getTreeFilenameWithSuffix(main_module, "")
         ),
-        "result_name"    : getResultBasepath(main_module),
-        "source_dir"     : getSourceDirectoryPath(main_module),
-        "debug_mode"     : asBoolStr(Options.isDebug()),
-        "python_debug"   : asBoolStr(Options.isPythonDebug()),
-        "unstriped_mode" : asBoolStr(Options.isUnstriped()),
-        "module_mode"    : asBoolStr(Options.shallMakeModule()),
-        "optimize_mode"  : asBoolStr(Options.isOptimize()),
-        "full_compat"    : asBoolStr(Options.isFullCompat()),
-        "experimental"   : asBoolStr(Options.isExperimental()),
-        "python_version" : python_version,
-        "target_arch"    : Utils.getArchitecture(),
-        "python_prefix"  : sys.prefix,
-        "nuitka_src"     : SconsInterface.getSconsDataPath(),
-        "module_count"   : "%d" % (
+        "result_name"     : getResultBasepath(main_module),
+        "source_dir"      : getSourceDirectoryPath(main_module),
+        "debug_mode"      : asBoolStr(Options.isDebug()),
+        "python_debug"    : asBoolStr(Options.isPythonDebug()),
+        "unstripped_mode" : asBoolStr(Options.isUnstripped()),
+        "module_mode"     : asBoolStr(Options.shallMakeModule()),
+        "optimize_mode"   : asBoolStr(Options.isOptimize()),
+        "full_compat"     : asBoolStr(Options.isFullCompat()),
+        "experimental"    : asBoolStr(Options.isExperimental()),
+        "trace_mode"      : asBoolStr(Options.shallTraceExecution()),
+        "python_version"  : python_version,
+        "target_arch"     : Utils.getArchitecture(),
+        "python_prefix"   : sys.prefix,
+        "nuitka_src"      : SconsInterface.getSconsDataPath(),
+        "module_count"    : "%d" % (
             len(ModuleRegistry.getDoneUserModules()) + 1
         )
     }
@@ -459,7 +473,7 @@ def runScons(main_module, quiet):
         msvc_version = Options.getMsvcVersion()
 
         msvc_version = msvc_version.replace("exp", "Exp")
-        if "." not in msvc_version:
+        if '.' not in msvc_version:
             msvc_version += ".0"
 
         options["msvc_version"] = msvc_version
@@ -482,7 +496,7 @@ def writeSourceCode(filename, source_code):
         with open(filename, "wb") as output_file:
             output_file.write(source_code.encode("latin1"))
     else:
-        with open(filename, "w") as output_file:
+        with open(filename, 'w') as output_file:
             output_file.write(source_code)
 
 
@@ -505,7 +519,7 @@ def callExec(args, clean_path, add_path):
 
     if add_path:
         if "PYTHONPATH" in os.environ:
-            os.environ["PYTHONPATH"] += ":" + Options.getOutputDir()
+            os.environ["PYTHONPATH"] += ':' + Options.getOutputDir()
         else:
             os.environ["PYTHONPATH"] = Options.getOutputDir()
 
@@ -513,9 +527,9 @@ def callExec(args, clean_path, add_path):
     sys.stdout.flush()
     sys.stderr.flush()
 
+    # Add the main arguments, previous separated.
     args += Options.getMainArgs()
 
-    # That's the API of execl, pylint: disable=W0142
     Utils.callExec(args)
 
 
@@ -524,7 +538,7 @@ def executeMain(binary_filename, tree, clean_path):
 
     if Options.isStandaloneMode():
         name = binary_filename
-    elif main_filename.endswith( ".py" ):
+    elif main_filename.endswith(".py"):
         name = main_filename[:-3]
     else:
         name = main_filename
@@ -543,9 +557,6 @@ def executeMain(binary_filename, tree, clean_path):
 def executeModule(tree, clean_path):
     python_command = "__import__('%s')" % tree.getName()
 
-    if Utils.getOS() == "Windows":
-        python_command = '"%s"' % python_command
-
     args = (
         sys.executable,
         "python",
@@ -577,7 +588,7 @@ def compileTree(main_module):
             frozen_code = generateBytecodeFrozenCode()
 
             writeSourceCode(
-                filename = Utils.joinpath(
+                filename    = Utils.joinpath(
                     source_dir,
                     "__frozen.cpp"
                 ),
@@ -591,20 +602,26 @@ def compileTree(main_module):
     else:
         source_dir = getSourceDirectoryPath(main_module)
 
-        if not Utils.isFile( Utils.joinpath(source_dir, "__helpers.hpp")):
+        if not Utils.isFile(Utils.joinpath(source_dir, "__helpers.hpp")):
             sys.exit("Error, no previous build directory exists.")
 
-    if Options.isShowProgress():
+    if Options.isShowProgress() or Options.isShowMemory():
         Tracing.printLine(
-            """Total memory usage before running scons: {memory}:""".format(
-                memory      = Utils.getHumanReadableProcessMemoryUsage()
+            "Total memory usage before running scons: {memory}:".format(
+                memory = Utils.getHumanReadableProcessMemoryUsage()
             )
         )
 
+    if Options.isShowMemory():
+        InstanceCounters.printStats()
+
+    if Options.shallNotDoExecCppCall():
+        return True, {}
+
     # Run the Scons to build things.
     result, options = runScons(
-        main_module  = main_module,
-        quiet        = not Options.isShowScons()
+        main_module = main_module,
+        quiet       = not Options.isShowScons()
     )
 
     return result, options
@@ -624,7 +641,7 @@ def main():
     """
 
     # Main has to fullfil many options, leading to many branches and statements
-    # to deal with them.  pylint: disable=R0912,R0915
+    # to deal with them.  pylint: disable=R0912
 
     positional_args = Options.getPositionalArgs()
     assert len(positional_args) > 0
@@ -660,22 +677,7 @@ def main():
             filename = filename
         )
     except (SyntaxError, IndentationError) as e:
-        if Options.isFullCompat() and \
-           (e.args[0].startswith("unknown encoding:") or \
-            e.args[0].startswith("encoding problem:")):
-            if PythonVersions.doShowUnknownEncodingName():
-                complaint = e.args[0].split(":",2)[1]
-            else:
-                complaint = " with BOM"
-
-            e.args = (
-                "encoding problem:%s" % complaint,
-                (e.args[1][0], 1, None, None)
-            )
-
-            if hasattr(e, "msg"):
-                e.msg = e.args[0]
-
+        # Syntax or indentation errors, output them to the user and abort.
         sys.exit(
             SyntaxErrors.formatOutput(e)
         )
@@ -694,6 +696,9 @@ def main():
         if not result:
             sys.exit(1)
 
+        if Options.shallNotDoExecCppCall():
+            sys.exit(0)
+
         # Remove the source directory (now build directory too) if asked to.
         if Options.isRemoveBuildDir():
             shutil.rmtree(
@@ -708,14 +713,18 @@ def main():
                 (binary_filename, None)
             )
 
+            dist_dir = getStandaloneDirectoryPath(main_module)
+
+            for module in ModuleRegistry.getDoneUserModules():
+                standalone_entry_points.extend(
+                    Plugins.considerExtraDlls(dist_dir, module)
+                )
+
             if Utils.getOS() == "NetBSD":
                 warning("Standalone mode on NetBSD is not functional, due to $ORIGIN linkage not being supported.")
 
             copyUsedDLLs(
-                dist_dir                = getStandaloneDirectoryPath(
-                    main_module
-                ),
-                binary_filename         = binary_filename,
+                dist_dir                = dist_dir,
                 standalone_entry_points = standalone_entry_points
             )
 
