@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -18,15 +18,19 @@
 """ Options module """
 
 version_string = """\
-Nuitka V0.5.5.3
-Copyright (C) 2014 Kay Hayen."""
+Nuitka V0.5.13
+Copyright (C) 2015 Kay Hayen."""
 
 import logging
-import re
 import sys
 from optparse import SUPPRESS_HELP, OptionGroup, OptionParser
 
-from . import Utils
+from nuitka.PythonVersions import (
+    getSupportedPythonVersions,
+    getSupportedPythonVersionStr,
+    python_version_str
+)
+from nuitka.utils import Utils
 
 # Indicator if we were called as "nuitka-run" in which case we assume some
 # other defaults and work a bit different with parameters.
@@ -35,6 +39,9 @@ is_nuitka_run = Utils.basename(sys.argv[0]).lower().startswith("nuitka-run")
 def getVersion():
     return version_string.split()[1][1:]
 
+def getYear():
+    return int(version_string.split()[4])
+
 if not is_nuitka_run:
     usage = "usage: %prog [--module] [--execute] [options] main_module.py"
 else:
@@ -71,7 +78,9 @@ parser.add_option(
     help    = """\
 Enable standalone mode in build. This allows you to transfer the created binary
 to other machines without it relying on an existing Python installation. It
-implies these options: "--recurse-all --recurse-stdlib". Defaults to off.""",
+implies these options: "--recurse-all --recurse-stdlib". You may also want
+to use "--python-flag=no_site" to avoid the "site.py" module. Defaults to
+off.""",
 )
 
 parser.add_option(
@@ -86,21 +95,6 @@ as bytecode. As a result compilation time will increase very much.
     )
 
 
-def getSupportedPythonVersions():
-    return ("2.6", "2.7", "3.2", "3.3", "3.4")
-
-def getSupportedPythonVersionStr():
-    supported_python_versions = getSupportedPythonVersions()
-
-    supported_python_versions_str = repr(supported_python_versions)[1:-1]
-    supported_python_versions_str = re.sub(
-        r"(.*),(.*)$",
-        r"\1, or\2",
-        supported_python_versions_str
-    )
-
-    return supported_python_versions_str
-
 parser.add_option(
     "--python-version",
     action  = "store",
@@ -109,8 +103,9 @@ parser.add_option(
     default = None,
     help    = """\
 Major version of Python to be used, one of %s.
-Defaults to what you run Nuitka with.""" % (
-       getSupportedPythonVersionStr()
+Defaults to what you run Nuitka with (currently %s)""" % (
+       getSupportedPythonVersionStr(),
+       python_version_str
     )
 )
 
@@ -214,11 +209,23 @@ recurse_group.add_option(
     default = [],
     help    = """\
 Recurse into that directory, no matter if it's used by the given main program
-in a visible form. Overrides all other options. Can be given multiple times.
-Default empty."""
+in a visible form. Overrides all other recursion options. Can be given multiple
+times. Default empty."""
 )
 
-parser.add_option_group( recurse_group )
+recurse_group.add_option(
+    "--recurse-files", "--recurse-pattern",
+    action  = "append",
+    dest    = "recurse_extra_files",
+    metavar = "PATTERN",
+    default = [],
+    help    = """\
+Recurse into files matching the PATTERN. Overrides all recursion other options.
+Can be given multiple times. Default empty."""
+)
+
+
+parser.add_option_group(recurse_group)
 
 execute_group = OptionGroup(
     parser,
@@ -247,7 +254,7 @@ PYTHONPATH. When all modules are successfully included, you ought to not need
 PYTHONPATH anymore."""
 )
 
-parser.add_option_group( execute_group )
+parser.add_option_group(execute_group)
 
 dump_group = OptionGroup(
     parser,
@@ -259,7 +266,7 @@ dump_group.add_option(
     action  = "store_true",
     dest    = "dump_xml",
     default = False,
-    help    = """Dump the final result of optimization as XML, then exit."""
+    help    = "Dump the final result of optimization as XML, then exit."
 )
 
 dump_group.add_option(
@@ -271,7 +278,7 @@ dump_group.add_option(
 Display the final result of optimization in a GUI, then exit."""
 )
 
-parser.add_option_group( dump_group )
+parser.add_option_group(dump_group)
 
 
 codegen_group = OptionGroup(
@@ -285,19 +292,37 @@ codegen_group.add_option(
     dest    = "improved",
     default = False,
     help    = """\
-Allow minor devitations from CPython behaviour, e.g. better tracebacks, which
+Allow minor deviations from CPython behavior, e.g. better tracebacks, which
 are not really incompatible, but different.""",
 )
 
-
 codegen_group.add_option(
     "--code-gen-no-statement-lines",
-    action  ="store_false",
+    action  = "store_false",
     dest    = "statement_lines",
     default = True,
+    help    = SUPPRESS_HELP,
+#     help    = """\
+# Statements shall have their line numbers set. Disable this for less precise
+# exceptions and slightly faster code. Not recommended. Defaults to off."""
+)
+
+codegen_group.add_option(
+    "--file-reference-choice",
+    action  = "store",
+    dest    = "file_reference_mode",
+    choices = ("original", "runtime", "frozen"),
+    default = None,
     help    = """\
-Statements shall have their line numbers set. Disable this for less precise
-exceptions and slightly faster code. Not recommended. Defaults to off."""
+Select what value "__file__" is going to be. With "runtime" (default for
+standalone binary mode and module mode), the created binaries and modules,
+use the location of themselves to deduct the value of "__file__". Included
+packages pretend to be in directories below that location. This allows you
+to include data files in deployments. If you merely seek acceleration, it's
+better for you to use the "original" value, where the source files location
+will be used. With "frozen" a notation "<frozen module_name>" is used. For
+compatibility reasons, the "__file__" value will always have ".py" suffix
+independent of what it really is."""
 )
 
 codegen_group.add_option(
@@ -309,7 +334,7 @@ codegen_group.add_option(
 # """Disable all unnecessary optimizations on Python level. Defaults to off."""
 )
 
-parser.add_option_group( codegen_group )
+parser.add_option_group(codegen_group)
 
 outputdir_group = OptionGroup(
     parser,
@@ -318,7 +343,7 @@ outputdir_group = OptionGroup(
 
 outputdir_group.add_option(
     "--output-dir",
-    action  ="store",
+    action  = "store",
     dest    = "output_dir",
     metavar = "DIRECTORY",
     default = "",
@@ -365,10 +390,10 @@ production. Defaults to off."""
 debug_group.add_option(
     "--unstripped", "--no-strip", "--unstriped",
     action  = "store_true",
-    dest    = "unstriped",
+    dest    = "unstripped",
     default = False,
     help    = """\
-Keep debug info in the resulting object file for better gdb interaction.
+Keep debug info in the resulting object file for better debugger interaction.
 Defaults to off."""
 )
 
@@ -383,14 +408,26 @@ Defaults to off."""
 )
 
 debug_group.add_option(
-    "--c++-only",
+    "--recompile-c++-only",
     action  = "store_true",
-    dest    = "cpp_only",
+    dest    = "recompile_cpp_only",
     default = False,
     help    = """\
-Compile the would-be regenerated source file. Allows compiling edited C++ files
+Take existing files and compile them again.Allows compiling edited C++ files
 with the C++ compiler for quick debugging changes to the generated source.
-Defaults to off."""
+Defaults to off. Depends on compiling Python source to determine which files it
+should look at."""
+)
+
+debug_group.add_option(
+    "--generate-c++-only",
+    action  = "store_true",
+    dest    = "generate_cpp_only",
+    default = False,
+    help    = """\
+Generate only C++ source code, and do not compile it to binary or module. This
+is for debugging and code coverage analysis that doesn't waste CPU. Defaults to
+off."""
 )
 
 debug_group.add_option(
@@ -403,6 +440,26 @@ Use features declared as 'experimental'. May have no effect if no experimental
 features are present in the code. Defaults to off."""
 )
 
+debug_group.add_option(
+    "--explain-imports",
+    action  = "store_true",
+    dest    = "explain_imports",
+    default = False,
+    help    = SUPPRESS_HELP
+)
+
+
+# This is for testing framework, "coverage.py" hates to loose the process. And
+# we can use it to make sure it's not done unknowingly.
+parser.add_option(
+    "--must-not-re-execute",
+    action  = "store_false",
+    dest    = "allow_reexecute",
+    default = True,
+    help    = SUPPRESS_HELP
+)
+
+
 parser.add_option_group(debug_group)
 
 cpp_compiler_group = OptionGroup(
@@ -444,9 +501,9 @@ compilers. Defaults to the most recent version."""
 
 cpp_compiler_group.add_option(
     "-j", "--jobs",
-    action  ="store",
+    action  = "store",
     dest    = "jobs",
-    metavar = "N",
+    metavar = 'N',
     default = Utils.getCoreCount(),
     help    = """\
 Specify the allowed number of parallel C++ compiler jobs. Defaults to the
@@ -489,6 +546,16 @@ tracing_group.add_option(
 Defaults to off."""
 )
 
+tracing_group.add_option(
+    "--show-memory",
+    action  = "store_true",
+    dest    = "show_memory",
+    default = False,
+    help    = """Provide memory information and statistics.
+Defaults to off."""
+)
+
+
 tracing_group.add_option(
     "--show-modules",
     action  = "store_true",
@@ -504,7 +571,7 @@ tracing_group.add_option(
     dest    = "verbose",
     default = False,
     help    = """\
-Output details of actions take, esp. in optimizations. Can become a lot.
+Output details of actions taken, esp. in optimizations. Can become a lot.
 Defaults to off."""
 )
 
@@ -516,7 +583,7 @@ windows_group.add_option(
     dest    = "win_disable_console",
     default = False,
     help    = """\
-When compiling for windows, disable the console window. Defaults to off."""
+When compiling for Windows, disable the console window. Defaults to off."""
 )
 
 windows_group.add_option(
@@ -525,22 +592,26 @@ windows_group.add_option(
     dest    = "icon_path",
     metavar = "ICON_PATH",
     default = None,
-    help    = """Add executable icon (windows only).""",
+    help    = "Add executable icon (Windows only).",
 )
 
 parser.add_option_group(windows_group)
 
 
-# First, isolate the first non-option arguments. TODO: Should repect "--"
-# as a terminator to options.
+# First, isolate the first non-option arguments.
 if is_nuitka_run:
     count = 0
 
-    for count, arg in enumerate( sys.argv ):
+    for count, arg in enumerate(sys.argv):
         if count == 0:
             continue
 
-        if arg[0] != "-":
+        if arg[0] != '-':
+            break
+
+        # Treat "--" as a terminator.
+        if arg == "--":
+            count += 1
             break
 
     if count > 0:
@@ -554,13 +625,13 @@ options, positional_args = parser.parse_args()
 if not positional_args:
     parser.print_help()
 
-    sys.exit( """
-Error, need positional argument with python module or main program.""" )
+    sys.exit("""
+Error, need positional argument with python module or main program.""")
 
 if options.verbose:
-    logging.getLogger().setLevel( logging.DEBUG )
+    logging.getLogger().setLevel(logging.DEBUG)
 else:
-    logging.getLogger().setLevel( logging.INFO )
+    logging.getLogger().setLevel(logging.INFO)
 
 # Standalone mode implies an executable, not importing "site" module, which is
 # only for this machine, recursing to all modules, and even including the
@@ -583,11 +654,24 @@ def shallDisplayBuiltTree():
     return options.display_tree
 
 def shallOnlyExecCppCall():
-    return options.cpp_only
+    return options.recompile_cpp_only
+
+def shallNotDoExecCppCall():
+    return options.generate_cpp_only
 
 def shallHaveStatementLines():
     return options.statement_lines
 
+def getFileReferenceMode():
+    if options.file_reference_mode is None:
+        value = ("runtime"
+                   if shallMakeModule() or isStandaloneMode() else
+                 "original")
+    else:
+        value = options.file_reference_mode
+
+    return value
+
 def shallMakeModule():
     return not options.executable
 
@@ -601,10 +685,13 @@ def shallFollowAllImports():
     return options.recurse_all
 
 def getShallFollowModules():
-    return sum( [ x.split( "," ) for x in options.recurse_modules ], [] )
+    return sum([ x.split(',') for x in options.recurse_modules ], [])
+
+def isAllowedToReexecute():
+    return options.allow_reexecute
 
 for any_case_module in getShallFollowModules():
-    if any_case_module.startswith( "." ):
+    if any_case_module.startswith('.'):
         bad = True
     else:
         for char in "/\\:":
@@ -615,15 +702,15 @@ for any_case_module in getShallFollowModules():
             bad = False
 
     if bad:
-        sys.exit( """
+        sys.exit("""
 Error, '--recurse-to' takes only module names, not directory path '%s'.""" % \
-any_case_module )
+any_case_module)
 
 def getShallFollowInNoCase():
-    return sum( [ x.split( "," ) for x in options.recurse_not_modules ], [] )
+    return sum([ x.split(',') for x in options.recurse_not_modules ], [])
 
 for no_case_module in getShallFollowInNoCase():
-    if no_case_module.startswith( "." ):
+    if no_case_module.startswith('.'):
         bad = True
     else:
         for char in "/\\:":
@@ -634,13 +721,16 @@ for no_case_module in getShallFollowInNoCase():
             bad = False
 
     if bad:
-        sys.exit( """
+        sys.exit("""
 Error, '--recurse-not-to' takes only module names, not directory path '%s'.""" % \
-no_case_module )
+no_case_module)
 
 
 def getShallFollowExtra():
-    return sum( [ x.split( "," ) for x in options.recurse_extra ], [] )
+    return sum([ x.split(',') for x in options.recurse_extra ], [])
+
+def getShallFollowExtraFilePatterns():
+    return sum([ x.split(',') for x in options.recurse_extra_files ], [])
 
 def shallWarnImplicitRaises():
     return options.warn_implicit_exceptions
@@ -654,17 +744,17 @@ def isPythonDebug():
 def isOptimize():
     return not options.no_optimize
 
-def isUnstriped():
-    return options.unstriped
+def isUnstripped():
+    return options.unstripped
 
 def getOutputPath(path):
     if options.output_dir:
-        return Utils.normpath( Utils.joinpath( options.output_dir, path ) )
+        return Utils.normpath(Utils.joinpath(options.output_dir, path))
     else:
         return path
 
 def getOutputDir():
-    return options.output_dir if options.output_dir else "."
+    return options.output_dir if options.output_dir else '.'
 
 def getPositionalArgs():
     return tuple(positional_args)
@@ -682,7 +772,7 @@ def isShowScons():
     return options.show_scons
 
 def getJobLimit():
-    return int( options.jobs )
+    return int(options.jobs)
 
 def isLto():
     return options.lto
@@ -705,17 +795,23 @@ def isFullCompat():
 def isShowProgress():
     return options.show_progress
 
+def isShowMemory():
+    return options.show_memory
+
 def isShowInclusion():
     return options.show_inclusion
 
 def isRemoveBuildDir():
-    return options.remove_build
+    return options.remove_build and not options.generate_cpp_only
 
 def getIntendedPythonVersion():
     return options.python_version
 
 def isExperimental():
-    return hasattr( options, "experimental" ) and options.experimental
+    return hasattr(options, "experimental") and options.experimental
+
+def shallExplainImports():
+    return options.explain_imports
 
 def isStandaloneMode():
     return options.is_standalone
@@ -730,7 +826,7 @@ def getPythonFlags():
         if part in ("-S", "nosite", "no_site"):
             result.append("no_site")
         elif part in ("static_hashes", "norandomization", "no_randomization"):
-            result.append( "no_randomization" )
+            result.append("no_randomization")
         elif part in ("-v", "trace_imports", "trace_import"):
             result.append("trace_imports")
         elif part in ("no_warnings", "nowarnings"):
