@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -16,36 +16,29 @@
 #     limitations under the License.
 #
 """
-Module like __future__ for things that are no more in CPython3,
+Module like __future__ for things that are changed between Python2 and Python3.
 
-but provide compatible fallbacks.
+These are here to provide compatible fall-backs. This is required to run the
+same code easily with both CPython2 and CPython3. Sometimes, we do not care
+about the actual types, or API, but would rather just check for something to
+be a "in (str, unicode)" rather than making useless version checks.
 
-This is required to run the same code easily with both CPython2 and CPython3.
 """
 
 
 # pylint: disable=W0622,C0103
 
-# Work around for CPython 3.x renaming long to int.
+# Work around for CPython 3.x renaming "long" to "int".
 try:
-    long = long  # lint:ok
+    long = long  # @ReservedAssignment
 except NameError:
-    long = int  # lint:ok
+    long = int   # @ReservedAssignment
 
-# Work around for CPython 3.x renaming unicode to str.
+# Work around for CPython 3.x renaming "unicode" to "str".
 try:
-    unicode = unicode  # lint:ok
+    unicode = unicode  # @ReservedAssignment
 except NameError:
-    unicode = str  # lint:ok
-
-# Work around for CPython 3.x removal of commands
-try:
-    import commands
-except ImportError:
-    # false alarm, no re-import, just another try if above fails, which it will
-    # on Python3 pylint: disable=W0404
-
-    import subprocess as commands  # lint:ok
+    unicode = str      # @ReservedAssignment
 
 def iterItems(d):
     try:
@@ -54,19 +47,18 @@ def iterItems(d):
         return d.items()
 
 if unicode is str:
-    raw_input = input
+    raw_input = input      # @ReservedAssignment
 else:
-    raw_input = raw_input
+    raw_input = raw_input  # @ReservedAssignment
 
-# pylint: disable=E0611,F0401
+# pylint: disable=E0611
 try:
     from urllib.request import urlretrieve
 except ImportError:
     from urllib import urlretrieve
-# pylint: enable=E0611,F0401
+# pylint: enable=E0611
 
 # For PyLint to be happy.
 assert long
 assert unicode
-assert commands
 assert urlretrieve
