@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -28,6 +28,23 @@ from .ErrorCodes import (
     getReleaseCode,
     getReleaseCodes
 )
+from .Helpers import generateChildExpressionsCode
+
+
+def generateAttributeLookupCode(to_name, expression, emit, context):
+    source_name, = generateChildExpressionsCode(
+        expression = expression,
+        emit       = emit,
+        context    = context
+    )
+
+    getAttributeLookupCode(
+        to_name        = to_name,
+        source_name    = source_name,
+        attribute_name = expression.getAttributeName(),
+        emit           = emit,
+        context        = context
+    )
 
 
 def getSpecialAttributeLookupCode(to_name, source_name, attr_name, emit,
