@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -52,7 +52,7 @@ def enableDebug():
             return self.value
 
         def __mod__(self, other):
-            assert type( other ) is dict, self.name
+            assert type(other) is dict, self.name
 
             for key in other.keys():
                 if "%%(%s)" % key not in self.value:
@@ -67,14 +67,14 @@ def enableDebug():
             return self.value % other
 
         def split(self, sep):
-            return self.value.split( sep )
+            return self.value.split(sep)
 
     from nuitka.__past__ import iterItems
 
     for template_name, template_value in iterItems(templates):
         # Ignore internal attribute like "__name__" that the module will also
         # have of course.
-        if template_name.startswith("_"):
+        if template_name.startswith('_'):
             continue
 
         if type(template_value) is str:
