@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,7 +22,7 @@ of frames for different uses.
 """
 
 
-from nuitka.Utils import python_version
+from nuitka.utils.Utils import python_version
 
 from . import CodeTemplates, Emission
 from .ExceptionCodes import getTracebackMakingIdentifier
@@ -39,6 +39,17 @@ def getFrameGuardHeavyCode(frame_identifier, code_identifier, codes,
                            frame_return_exit, provider, context):
     no_exception_exit = context.allocateLabel("frame_no_exception")
 
+    context.addFrameDeclaration(
+        CodeTemplates.template_frame_guard_cache_decl % {
+            "frame_identifier" : frame_identifier,
+        }
+    )
+    context.addFrameDeclaration(
+        CodeTemplates.template_frame_guard_frame_decl % {
+            "frame_identifier" : frame_identifier,
+        }
+    )
+
     result = CodeTemplates.template_frame_guard_full_block % {
         "frame_identifier"  : frame_identifier,
         "code_identifier"   : code_identifier,
@@ -97,6 +108,12 @@ def getFrameGuardOnceCode(frame_identifier, code_identifier,
         # TODO: Not using locals, which is only OK for modules
         assert False, locals_code
 
+    context.addFrameDeclaration(
+        CodeTemplates.template_frame_guard_frame_decl % {
+            "frame_identifier" : frame_identifier,
+        }
+    )
+
     return CodeTemplates.template_frame_guard_once % {
         "frame_identifier"      : frame_identifier,
         "code_identifier"       : code_identifier,
@@ -120,6 +137,18 @@ def getFrameGuardLightCode(frame_identifier, code_identifier, codes,
 
     assert frame_exception_exit is not None
 
+    context.addFrameDeclaration(
+        CodeTemplates.template_frame_guard_cache_decl % {
+            "frame_identifier" : frame_identifier,
+        }
+    )
+
+    context.addFrameDeclaration(
+        CodeTemplates.template_frame_guard_frame_decl % {
+            "frame_identifier" : frame_identifier,
+        }
+    )
+
     no_exception_exit = context.allocateLabel("frame_no_exception")
 
     result = CodeTemplates.template_frame_guard_generator % {
@@ -143,9 +172,12 @@ def getFrameGuardLightCode(frame_identifier, code_identifier, codes,
         context  = context
     )
 
+
+    # TODO: Don't create locals for StopIteration or GeneratorExit, that is just
+    # wasteful.
     result += CodeTemplates.template_frame_guard_generator_exception_handler % {
         "frame_identifier"      : frame_identifier,
-        "store_frame_locals"    : indented(locals_code, 0, vert_block = True),
+        "store_frame_locals"    : indented(locals_code, vert_block = True),
         "tb_making"             : getTracebackMakingIdentifier(context),
         "frame_exception_exit"  : frame_exception_exit,
         "parent_exception_exit" : parent_exception_exit,
@@ -158,6 +190,10 @@ def getFrameGuardLightCode(frame_identifier, code_identifier, codes,
 def getFrameLocalsUpdateCode(provider, context):
     locals_codes = Emission.SourceCodeCollector()
 
+    context.setCurrentSourceCodeReference(
+        provider.getSourceReference()
+    )
+
     frame_locals_name = context.allocateTempName(
         "frame_locals",
         unique = True
@@ -206,9 +242,12 @@ def getFramePreserveExceptionCode(emit, context):
 
             emit(
                 """\
-%(keeper_type)s = INCREASE_REFCOUNT( PyThreadState_GET()->exc_type );
-%(keeper_value)s = INCREASE_REFCOUNT_X( PyThreadState_GET()->exc_value );
-%(keeper_tb)s = (PyTracebackObject *)INCREASE_REFCOUNT_X( PyThreadState_GET()->exc_traceback );
+%(keeper_type)s = PyThreadState_GET()->exc_type;
+Py_XINCREF( %(keeper_type)s );
+%(keeper_value)s = PyThreadState_GET()->exc_value;
+Py_XINCREF( %(keeper_value)s );
+%(keeper_tb)s = (PyTracebackObject *)PyThreadState_GET()->exc_traceback;
+Py_XINCREF( %(keeper_tb)s );
 """ % {
                     "keeper_type"  : keeper_type,
                     "keeper_value" : keeper_value,
@@ -253,11 +292,14 @@ def getFrameReraiseExceptionCode(emit, context):
 
     emit(
         """\
-exception_type = INCREASE_REFCOUNT( PyThreadState_GET()->exc_type );
-exception_value = INCREASE_REFCOUNT( PyThreadState_GET()->exc_value );
-exception_tb = (PyTracebackObject *)INCREASE_REFCOUNT( PyThreadState_GET()->exc_traceback );
+exception_type = PyThreadState_GET()->exc_type;
+Py_INCREF( exception_type );
+exception_value = PyThreadState_GET()->exc_value;
+Py_INCREF( exception_value );
+exception_tb = (PyTracebackObject *)PyThreadState_GET()->exc_traceback;
+Py_INCREF( exception_tb );
 """ )
     getSetLineNumberCodeRaw("exception_tb->tb_lineno", emit, context)
     getFrameRestoreExceptionCode(emit, context)
 
-    getGotoCode(context.getExceptionEscape(), emit )
+    getGotoCode(context.getExceptionEscape(), emit)
