@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -27,7 +27,7 @@ def getLineNumberUpdateCode(context):
     else:
         source_ref = context.getCurrentSourceCodeReference()
 
-        if source_ref.shallSetCurrentLine():
+        if source_ref.isInternal():
             return "%s->f_lineno = %d;" % (
                 frame_handle,
                 source_ref.getLineNumber()
@@ -35,12 +35,14 @@ def getLineNumberUpdateCode(context):
         else:
             return ""
 
+
 def emitLineNumberUpdateCode(context, emit):
     code = getLineNumberUpdateCode(context)
 
     if code:
         emit(code)
 
+
 def getSetLineNumberCodeRaw(to_name, emit, context):
     assert context.getFrameHandle() is not None
 
@@ -52,14 +54,6 @@ def getSetLineNumberCodeRaw(to_name, emit, context):
     )
 
 
-# TODO: The below should probably become unused.
-def getSetLineNumberCode(source_ref, emit, context):
-    if source_ref.shallSetCurrentLine():
-        line_number = source_ref.getLineNumber()
-
-        getSetLineNumberCodeRaw(line_number, emit, context)
-
-
 def getLineNumberCode(to_name, emit, context):
     assert context.getFrameHandle() is not None
 
