@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,9 +25,16 @@ that should be unified at some point.
 
 """
 
-from .NodeBases import StatementChildrenHavingBase
+from nuitka import VariableRegistry
+from nuitka.utils import Utils
+
+from .NodeBases import NodeBase, StatementChildrenHavingBase
+from .NodeMakingHelpers import (
+    makeStatementExpressionOnlyReplacementNode,
+    makeStatementOnlyNodesFromExpressions,
+    makeStatementsSequenceReplacementNode
+)
 
-# Delayed import into multiple branches is not an issue, pylint: disable=W0404
 
 class StatementAssignmentVariable(StatementChildrenHavingBase):
     kind = "STATEMENT_ASSIGNMENT_VARIABLE"
@@ -54,6 +61,8 @@ class StatementAssignmentVariable(StatementChildrenHavingBase):
             source_ref = source_ref
         )
 
+        self.variable_trace = None
+
     def getDetail(self):
         variable_ref = self.getTargetVariableRef()
         variable = variable_ref.getVariable()
@@ -69,6 +78,9 @@ class StatementAssignmentVariable(StatementChildrenHavingBase):
     getAssignSource = StatementChildrenHavingBase.childGetter(
         "source"
     )
+    setAssignSource = StatementChildrenHavingBase.childSetter(
+        "source"
+    )
 
     def markAsInplaceSuspect(self):
         self.inplace_suspect = True
@@ -81,21 +93,106 @@ class StatementAssignmentVariable(StatementChildrenHavingBase):
 
     def computeStatement(self, constraint_collection):
         # Assignment source may re-compute here:
-        constraint_collection.onExpression(
-            expression = self.getAssignSource()
-        )
+        constraint_collection.onExpression(self.getAssignSource())
+        source = self.getAssignSource()
 
+        # No assignment will occur, if the assignment source raises, so strip it
+        # away.
+        if source.willRaiseException(BaseException):
+
+            result = makeStatementExpressionOnlyReplacementNode(
+                expression = source,
+                node       = self
+            )
+
+            return result, "new_raise", """\
+Assignment raises exception in assigned value, removed assignment."""
+
+        variable_ref = self.getTargetVariableRef()
+        variable = variable_ref.getVariable()
+
+        # Not allowed anymore at this point.
+        assert variable is not None
+
+        # Assigning from and to the same variable, can be optimized away
+        # immediately, there is no point in doing it. Exceptions are of course
+        # module variables that collide with built-in names.
+        if not variable.isModuleVariable() and \
+             source.isExpressionVariableRef() and \
+             source.getVariable() == variable:
+
+            # A variable access that has a side effect, must be preserved,
+            # so it can e.g. raise an exception, otherwise we can be fully
+            # removed.
+            if source.mayHaveSideEffects():
+                result = makeStatementExpressionOnlyReplacementNode(
+                    expression = source,
+                    node       = self
+                )
+
+                return result, "new_statements", """\
+Reduced assignment of variable from itself to access of it."""
+            else:
+                return None, "new_statements", """\
+Removed assignment of variable from itself which is known to be defined."""
+
+
+        # If the assignment source has side effects, we can simply evaluate them
+        # beforehand, we have already visited and evaluated them before.
+        if source.isExpressionSideEffects():
+            statements = [
+                makeStatementExpressionOnlyReplacementNode(
+                    side_effect,
+                    self
+                )
+                for side_effect in
+                source.getSideEffects()
+            ]
+
+            statements.append(self)
+
+            parent = self.parent
+            result = makeStatementsSequenceReplacementNode(
+                statements = statements,
+                node       = self,
+            )
+            result.parent = parent
+
+            # Need to update it.
+            self.setAssignSource(source.getExpression())
+            source = self.getAssignSource()
+
+            result = result, "new_statements", """\
+Side effects of assignments promoted to statements."""
+        else:
+            result = self, None, None
+
+        # Set-up the trace to the trace collection, so future references will
+        # find this assignment.
         self.variable_trace = constraint_collection.onVariableSet(
             assign_node = self
         )
 
-        # TODO: Remove this, it's old.
-        return constraint_collection._onStatementAssignmentVariable(self)
+        global_trace = VariableRegistry.getGlobalVariableTrace(variable)
+
+        if global_trace is not None:
+            if variable.isTempVariable():
+                if source.isCompileTimeConstant() and not source.isMutable():
+                    self.variable_trace.setReplacementNode(source)
+            elif variable.isLocalVariable():
+                if source.isCompileTimeConstant() and not source.isMutable():
+                    provider = self.getParentVariableProvider()
+
+                    if provider.isPythonModule() or \
+                       (not provider.isUnoptimized() and not provider.isClassDictCreation()):
+                        self.variable_trace.setReplacementNode(source)
+
+        return result
 
     def needsReleaseValue(self):
         previous = self.variable_trace.getPrevious()
 
-        if previous.mustBeUninit():
+        if previous.mustNotHaveValue():
             return False
         elif previous.mustHaveValue():
             return True
@@ -147,9 +244,6 @@ class StatementAssignmentAttribute(StatementChildrenHavingBase):
         # No assignment will occur, if the assignment source raises, so strip it
         # away.
         if source.willRaiseException(BaseException):
-            from .NodeMakingHelpers import \
-                makeStatementExpressionOnlyReplacementNode
-
             result = makeStatementExpressionOnlyReplacementNode(
                 expression = source,
                 node       = self
@@ -162,9 +256,6 @@ Attribute assignment raises exception in assigned value, removed assignment."""
         lookup_source = self.getLookupSource()
 
         if lookup_source.willRaiseException(BaseException):
-            from .NodeMakingHelpers import \
-                makeStatementOnlyNodesFromExpressions
-
             result = makeStatementOnlyNodesFromExpressions(
                 expressions = (
                     source,
@@ -175,6 +266,9 @@ Attribute assignment raises exception in assigned value, removed assignment."""
             return result, "new_raise", """\
 Attribute assignment raises exception in source, removed assignment."""
 
+        # Any code could be run, note that.
+        constraint_collection.onControlFlowEscape(self)
+
         return self, None, None
 
 
@@ -211,8 +305,6 @@ class StatementAssignmentSubscript(StatementChildrenHavingBase):
         # No assignment will occur, if the assignment source raises, so strip it
         # away.
         if source.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementExpressionOnlyReplacementNode
-
             result = makeStatementExpressionOnlyReplacementNode(
                 expression = source,
                 node       = self
@@ -225,8 +317,6 @@ Subscript assignment raises exception in assigned value, removed assignment."""
         subscribed = self.getSubscribed()
 
         if subscribed.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
-
             result = makeStatementOnlyNodesFromExpressions(
                 expressions = (
                     source,
@@ -242,9 +332,7 @@ Subscript assignment raises exception in subscribed, removed assignment."""
         )
         subscript = self.getSubscript()
 
-        if subscript.willRaiseException( BaseException ):
-            from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
-
+        if subscript.willRaiseException(BaseException):
             result = makeStatementOnlyNodesFromExpressions(
                 expressions = (
                     source,
@@ -257,6 +345,9 @@ Subscript assignment raises exception in subscribed, removed assignment."""
 Subscript assignment raises exception in subscript value, removed \
 assignment."""
 
+        # Any code could be run, note that.
+        constraint_collection.onControlFlowEscape(self)
+
         return self, None, None
 
 
@@ -271,6 +362,8 @@ class StatementAssignmentSlice(StatementChildrenHavingBase):
     )
 
     def __init__(self, expression, lower, upper, source, source_ref):
+        assert Utils.python_version < 300
+
         StatementChildrenHavingBase.__init__(
             self,
             values     = {
@@ -294,8 +387,6 @@ class StatementAssignmentSlice(StatementChildrenHavingBase):
         # No assignment will occur, if the assignment source raises, so strip it
         # away.
         if source.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementExpressionOnlyReplacementNode
-
             result = makeStatementExpressionOnlyReplacementNode(
                 expression = source,
                 node       = self
@@ -308,8 +399,6 @@ Slice assignment raises exception in assigned value, removed assignment."""
         lookup_source = self.getLookupSource()
 
         if lookup_source.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
-
             result = makeStatementOnlyNodesFromExpressions(
                 expressions = (
                     source,
@@ -320,12 +409,10 @@ Slice assignment raises exception in assigned value, removed assignment."""
             return result, "new_raise", """\
 Slice assignment raises exception in sliced value, removed assignment."""
 
-        constraint_collection.onExpression( self.getLower(), allow_none = True )
+        constraint_collection.onExpression(self.getLower(), allow_none = True)
         lower = self.getLower()
 
         if lower is not None and lower.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
-
             result = makeStatementOnlyNodesFromExpressions(
                 expressions = (
                     source,
@@ -338,12 +425,10 @@ Slice assignment raises exception in sliced value, removed assignment."""
 Slice assignment raises exception in lower slice boundary value, removed \
 assignment."""
 
-        constraint_collection.onExpression( self.getUpper(), allow_none = True )
+        constraint_collection.onExpression(self.getUpper(), allow_none = True)
         upper = self.getUpper()
 
         if upper is not None and upper.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
-
             result = makeStatementOnlyNodesFromExpressions(
                 expressions = (
                     source,
@@ -357,6 +442,9 @@ assignment."""
 Slice assignment raises exception in upper slice boundary value, removed \
 assignment."""
 
+        # Any code could be run, note that.
+        constraint_collection.onControlFlowEscape(self)
+
         return self, None, None
 
 
@@ -380,6 +468,7 @@ class StatementDelVariable(StatementChildrenHavingBase):
             source_ref = source_ref
         )
 
+        self.variable_trace = None
         self.tolerant = tolerant
 
     def getDetail(self):
@@ -405,20 +494,28 @@ class StatementDelVariable(StatementChildrenHavingBase):
     )
 
     def computeStatement(self, constraint_collection):
-        self.variable_trace = constraint_collection.onVariableDel(
+        variable_trace = constraint_collection.onVariableDel(
             del_node = self
         )
 
         if self.isTolerant():
-            previous = self.variable_trace.getPrevious()
-
-            if previous is None or previous.isUninitTrace():
+            if variable_trace.isUninitTrace():
                 return (
                     None,
                     "new_statements",
-                    "Removed tolerant del without effect."
+                    "Removed tolerant 'del' statement without effect."
                 )
 
+
+        # Any code could be run, note that.
+        constraint_collection.onControlFlowEscape(self)
+
+        # Need to fetch the potentially invalidated variable. A "del" on a
+        # or shared value, may easily assign the global variable in "__del__".
+        self.variable_trace = constraint_collection.getVariableCurrentTrace(
+            variable = self.getTargetVariableRef().getVariable()
+        )
+
         return self, None, None
 
     def mayHaveSideEffects(self):
@@ -430,13 +527,70 @@ class StatementDelVariable(StatementChildrenHavingBase):
         else:
             variable = self.getTargetVariableRef().getVariable()
 
-            # Temp variables won't raise.
-            if variable.isTempVariable():
-                return False
+            # TODO: This condition must become unnecessary, but enhancing
+            # SSA to notice potential escapes.
+            if not variable.isSharedTechnically():
+
+                # Temporary variables deletions won't raise, just because we don't
+                # create them that way. We can avoid going through SSA in these
+                # cases.
+                if variable.isTempVariable():
+                    return False
+
+                # If SSA knows, that's fine.
+                if self.variable_trace.mustHaveValue():
+                    return False
 
             return True
 
 
+class StatementReleaseVariable(NodeBase):
+    kind = "STATEMENT_RELEASE_VARIABLE"
+
+    def __init__(self, variable, tolerant, source_ref):
+        NodeBase.__init__(
+            self,
+            source_ref = source_ref
+        )
+
+        # TODO: Unsure if that makes sense, we only have those it seems.
+        self.tolerant = tolerant
+
+        self.variable = variable
+
+        self.variable_trace = None
+
+    def getDetail(self):
+        return "of variable %s" % self.variable
+
+    def getDetails(self):
+        return {
+            "variable" : self.variable
+        }
+
+    def getVariable(self):
+        return self.variable
+
+    def computeStatement(self, constraint_collection):
+        self.variable_trace = constraint_collection.onVariableRelease(
+            release_node = self
+        )
+
+        if self.variable_trace.isUninitTrace():
+            return None, "new_statements", None
+
+        # TODO: We might be able to remove ourselves based on the trace
+        # we belong to.
+
+        return self, None, None
+
+    def mayHaveSideEffects(self):
+        return True
+
+    def mayRaiseException(self, exception_type):
+        return False
+
+
 class StatementDelAttribute(StatementChildrenHavingBase):
     kind = "STATEMENT_DEL_ATTRIBUTE"
 
@@ -456,7 +610,9 @@ class StatementDelAttribute(StatementChildrenHavingBase):
         self.attribute_name = attribute_name
 
     def getDetails(self):
-        return { "attribute" : self.attribute_name }
+        return {
+            "attribute" : self.attribute_name
+        }
 
     def getDetail(self):
         return "to attribute %s" % self.attribute_name
@@ -474,13 +630,14 @@ class StatementDelAttribute(StatementChildrenHavingBase):
         lookup_source = self.getLookupSource()
 
         if lookup_source.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementExpressionOnlyReplacementNode
-
             return makeStatementExpressionOnlyReplacementNode(
                 expression = lookup_source,
                 node       = self
             )
 
+        # Any code could be run, note that.
+        constraint_collection.onControlFlowEscape(self)
+
         return self, None, None
 
 
@@ -510,22 +667,18 @@ class StatementDelSubscript(StatementChildrenHavingBase):
         subscribed = self.getSubscribed()
 
         if subscribed.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementExpressionOnlyReplacementNode
-
             result = makeStatementExpressionOnlyReplacementNode(
                 expression = subscribed,
                 node       = self
             )
 
             return result, "new_raise", """\
-Subscript del raises exception in subscribed value, removed del"""
+Subscript 'del' raises exception in subscribed value, removed del."""
 
         constraint_collection.onExpression(self.getSubscript())
         subscript = self.getSubscript()
 
         if subscript.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
-
             result = makeStatementOnlyNodesFromExpressions(
                 expressions = (
                     subscribed,
@@ -534,7 +687,10 @@ Subscript del raises exception in subscribed value, removed del"""
             )
 
             return result, "new_raise", """\
-Subscript del raises exception in subscribt value, removed del"""
+Subscript 'del' raises exception in subscript value, removed del."""
+
+        # Any code could be run, note that.
+        constraint_collection.onControlFlowEscape(self)
 
         return self, None, None
 
@@ -568,8 +724,6 @@ class StatementDelSlice(StatementChildrenHavingBase):
         lookup_source = self.getLookupSource()
 
         if lookup_source.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementExpressionOnlyReplacementNode
-
             result = makeStatementExpressionOnlyReplacementNode(
                 expression = lookup_source,
                 node       = self
@@ -583,8 +737,6 @@ Slice del raises exception in sliced value, removed del"""
         lower = self.getLower()
 
         if lower is not None and lower.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
-
             result = makeStatementOnlyNodesFromExpressions(
                 expressions = (
                     lookup_source,
@@ -599,8 +751,6 @@ Slice del raises exception in lower slice boundary value, removed del"""
         upper = self.getUpper()
 
         if upper is not None and upper.willRaiseException(BaseException):
-            from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
-
             result = makeStatementOnlyNodesFromExpressions(
                 expressions = (
                     lookup_source,
@@ -612,4 +762,7 @@ Slice del raises exception in lower slice boundary value, removed del"""
             return result, "new_raise", """
 Slice del raises exception in upper slice boundary value, removed del"""
 
+        # Any code could be run, note that.
+        constraint_collection.onControlFlowEscape(self)
+
         return self, None, None
