@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,21 +23,27 @@ and objects and classes.
 There will be a method "computeExpressionAttribute" to aid predicting them.
 """
 
+from nuitka.Builtins import calledWithBuiltinArgumentNamesDecorator
+
 from .NodeBases import ExpressionChildrenHavingBase
+from .NodeMakingHelpers import (
+    getComputationResult,
+    wrapExpressionWithNodeSideEffects
+)
 
 
 class ExpressionAttributeLookup(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_ATTRIBUTE_LOOKUP"
 
     named_children = (
-        "expression",
+        "source",
     )
 
-    def __init__(self, expression, attribute_name, source_ref):
+    def __init__(self, source, attribute_name, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
             values     = {
-                "expression" : expression
+                "source" : source
             },
             source_ref = source_ref
         )
@@ -51,7 +57,9 @@ class ExpressionAttributeLookup(ExpressionChildrenHavingBase):
         self.attribute_name = attribute_name
 
     def getDetails(self):
-        return { "attribute" : self.getAttributeName() }
+        return {
+            "attribute" : self.getAttributeName()
+        }
 
     def getDetail(self):
         return "attribute %s from %s" % (
@@ -60,12 +68,12 @@ class ExpressionAttributeLookup(ExpressionChildrenHavingBase):
         )
 
     getLookupSource = ExpressionChildrenHavingBase.childGetter(
-        "expression"
+        "source"
     )
 
     def makeCloneAt(self, source_ref):
         return ExpressionAttributeLookup(
-            expression     = self.getLookupSource().makeCloneAt( source_ref ),
+            source         = self.getLookupSource().makeCloneAt(source_ref),
             attribute_name = self.getAttributeName(),
             source_ref     = source_ref
         )
@@ -73,7 +81,7 @@ class ExpressionAttributeLookup(ExpressionChildrenHavingBase):
     def computeExpression(self, constraint_collection):
         lookup_source = self.getLookupSource()
 
-        if lookup_source.willRaiseException( BaseException ):
+        if lookup_source.willRaiseException(BaseException):
             return lookup_source, "new_raise", "Attribute lookup source raises exception."
 
         return lookup_source.computeExpressionAttribute(
@@ -94,7 +102,7 @@ class ExpressionSpecialAttributeLookup(ExpressionAttributeLookup):
     def computeExpression(self, constraint_collection):
         lookup_source = self.getLookupSource()
 
-        if lookup_source.willRaiseException( BaseException ):
+        if lookup_source.willRaiseException(BaseException):
             return lookup_source, "new_raise", "Special attribute lookup source raises exception."
 
         # TODO: Special lookups may reuse "computeExpressionAttribute"
@@ -106,14 +114,12 @@ class ExpressionBuiltinGetattr(ExpressionChildrenHavingBase):
 
     named_children = ("source", "attribute", "default")
 
-    # Need to accept 'object' keyword argument, that is just the API of getattr,
-    # pylint: disable=W0622
-
-    def __init__(self, object, name, default, source_ref):
+    @calledWithBuiltinArgumentNamesDecorator
+    def __init__(self, object_arg, name, default, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
             values     = {
-                "source"    : object,
+                "source"    : object_arg,
                 "attribute" : name,
                 "default"   : default
             },
@@ -125,44 +131,45 @@ class ExpressionBuiltinGetattr(ExpressionChildrenHavingBase):
     getDefault = ExpressionChildrenHavingBase.childGetter("default")
 
     def computeExpression(self, constraint_collection):
-        # Children can tell all we need to know, pylint: disable=W0613
+        attribute = self.getAttribute()
 
-        default = self.getDefault()
+        attribute_name = attribute.getStringValue()
 
-        if default is None:
-            attribute = self.getAttribute()
+        if attribute_name is not None:
+            source = self.getLookupSource()
+            # If source has side effects, they must be evaluated, before the
+            # lookup, meaning, a temporary variable should be assigned. For
+            # now, we give up in this case. TODO: Replace source with a
+            # temporary variable assignment as a side effect.
 
-            attribute_name = attribute.getStringValue()
+            side_effects = source.extractSideEffects()
 
-            if attribute_name is not None:
-                source = self.getLookupSource()
-                # If source has sideeffects, it must be evaluated, before the
-                # lookup, meaning, a temporary variable should be assigned. For
-                # now, we give up in this case. TODO: Replace source with a
-                # temporary variable assignment as a side effect.
+            if not side_effects:
+                result = ExpressionAttributeLookup(
+                    source         = source,
+                    attribute_name = attribute_name,
+                    source_ref     = self.source_ref
+                )
 
-                side_effects = source.extractSideEffects()
+                result = wrapExpressionWithNodeSideEffects(
+                    new_node = result,
+                    old_node = attribute
+                )
 
-                if not side_effects:
-                    result = ExpressionAttributeLookup(
-                        expression     = source,
-                        attribute_name = attribute_name,
-                        source_ref     = self.source_ref
-                    )
-
-                    from .NodeMakingHelpers import wrapExpressionWithNodeSideEffects
+                default = self.getDefault()
 
+                if default is not None:
                     result = wrapExpressionWithNodeSideEffects(
                         new_node = result,
-                        old_node = attribute
+                        old_node = default
                     )
 
-                    return (
-                        result,
-                        "new_expression",
-                        """Replaced call to built-in 'getattr' with constant \
+                return (
+                    result,
+                    "new_expression",
+                    """Replaced call to built-in 'getattr' with constant \
 attribute '%s' to mere attribute lookup""" % attribute_name
-                    )
+                )
 
         return self, None, None
 
@@ -170,29 +177,25 @@ attribute '%s' to mere attribute lookup""" % attribute_name
 class ExpressionBuiltinSetattr(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_SETATTR"
 
-    named_children = ( "source", "attribute", "value" )
+    named_children = ("source", "attribute", "value")
 
-    # Need to accept 'object' keyword argument, that is just the API of setattr,
-    # pylint: disable=W0622
-
-    def __init__(self, object, name, value, source_ref):
+    @calledWithBuiltinArgumentNamesDecorator
+    def __init__(self, object_arg, name, value, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
             values     = {
-                "source"    : object,
+                "source"    : object_arg,
                 "attribute" : name,
                 "value"     : value
             },
             source_ref = source_ref
         )
 
-    getLookupSource = ExpressionChildrenHavingBase.childGetter( "source" )
-    getAttribute = ExpressionChildrenHavingBase.childGetter( "attribute" )
-    getValue = ExpressionChildrenHavingBase.childGetter( "value" )
+    getLookupSource = ExpressionChildrenHavingBase.childGetter("source")
+    getAttribute = ExpressionChildrenHavingBase.childGetter("attribute")
+    getValue = ExpressionChildrenHavingBase.childGetter("value")
 
     def computeExpression(self, constraint_collection):
-        # Children can tell all we need to know, pylint: disable=W0613
-
         # Note: Might be possible to predict or downgrade to mere attribute set.
         return self, None, None
 
@@ -200,28 +203,57 @@ class ExpressionBuiltinSetattr(ExpressionChildrenHavingBase):
 class ExpressionBuiltinHasattr(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_HASATTR"
 
-    named_children = ( "source", "attribute" )
+    named_children = ("source", "attribute")
 
-    # Need to accept object keyword argument, that is just the API of hasattr,
-    # pylint: disable=W0622
-
-    def __init__(self, object, name, source_ref):
+    @calledWithBuiltinArgumentNamesDecorator
+    def __init__(self, object_arg, name, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
             values     = {
-                "source"    : object,
+                "source"    : object_arg,
                 "attribute" : name,
             },
             source_ref = source_ref
         )
 
-    getLookupSource = ExpressionChildrenHavingBase.childGetter( "source" )
-    getAttribute = ExpressionChildrenHavingBase.childGetter( "attribute" )
+    getLookupSource = ExpressionChildrenHavingBase.childGetter("source")
+    getAttribute = ExpressionChildrenHavingBase.childGetter("attribute")
 
     def computeExpression(self, constraint_collection):
-        # Children can tell all we need to know, pylint: disable=W0613
+        # We do at least for compile time constants optimization here, but more
+        # could be done, were we to know shapes.
+        source = self.getLookupSource()
 
-        # Note: Might be possible to predict or downgrade to mere attribute
-        # check.
+        if source.isCompileTimeConstant():
+            attribute = self.getAttribute()
+
+            attribute_name = attribute.getStringValue()
+
+            if attribute_name is not None:
+
+                # If source has side effects, they must be evaluated, before the
+                # lookup, meaning, a temporary variable should be assigned. For
+                # now, we give up in this case. TODO: Replace source with a
+                # temporary variable assignment as a side effect.
+
+                result, tags, change_desc = getComputationResult(
+                    node        = self,
+                    computation = lambda : hasattr(
+                        source.getCompileTimeConstant(),
+                        attribute_name
+                    ),
+                    description = "Call to 'hasattr' pre-computed."
+                )
+
+                result = wrapExpressionWithNodeSideEffects(
+                    new_node = result,
+                    old_node = attribute
+                )
+                result = wrapExpressionWithNodeSideEffects(
+                    new_node = result,
+                    old_node = source
+                )
+
+                return result, tags, change_desc
 
         return self, None, None
