@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -27,12 +27,15 @@ from .NodeBases import ExpressionChildrenHavingBase
 class ExpressionSelectMetaclass(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_SELECT_METACLASS"
 
-    named_children = ( "metaclass", "bases", )
+    named_children = (
+        "metaclass",
+        "bases"
+    )
 
     def __init__(self, metaclass, bases, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
-            values = {
+            values     = {
                 "metaclass" : metaclass,
                 "bases"     : bases
             },
@@ -44,13 +47,13 @@ class ExpressionSelectMetaclass(ExpressionChildrenHavingBase):
         return self, None, None
 
     getMetaclass = ExpressionChildrenHavingBase.childGetter("metaclass")
-    getBases = ExpressionChildrenHavingBase.childGetter( "bases" )
+    getBases = ExpressionChildrenHavingBase.childGetter("bases")
 
 
 class ExpressionBuiltinType3(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_TYPE3"
 
-    named_children = ( "type_name", "bases", "dict" )
+    named_children = ("type_name", "bases", "dict")
 
     def __init__(self, type_name, bases, type_dict, source_ref):
         ExpressionChildrenHavingBase.__init__(
@@ -63,9 +66,9 @@ class ExpressionBuiltinType3(ExpressionChildrenHavingBase):
             source_ref = source_ref
         )
 
-    getTypeName = ExpressionChildrenHavingBase.childGetter( "type_name" )
-    getBases = ExpressionChildrenHavingBase.childGetter( "bases" )
-    getDict = ExpressionChildrenHavingBase.childGetter( "dict" )
+    getTypeName = ExpressionChildrenHavingBase.childGetter("type_name")
+    getBases = ExpressionChildrenHavingBase.childGetter("bases")
+    getDict = ExpressionChildrenHavingBase.childGetter("dict")
 
     def computeExpression(self, constraint_collection):
         # TODO: Should be compile time computable if bases and dict are.
