@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,11 +26,14 @@ from .NodeBases import (
     ExpressionChildrenHavingBase,
     SideEffectsFromChildrenMixin
 )
+from .NodeMakingHelpers import getComputationResult
 
 
 class ExpressionMakeSequenceBase(SideEffectsFromChildrenMixin,
                                  ExpressionChildrenHavingBase):
-    named_children = ("elements",)
+    named_children = (
+        "elements",
+    )
 
     def __init__(self, sequence_kind, elements, source_ref):
         assert sequence_kind in ("TUPLE", "LIST", "SET"), sequence_kind
@@ -43,7 +46,7 @@ class ExpressionMakeSequenceBase(SideEffectsFromChildrenMixin,
         ExpressionChildrenHavingBase.__init__(
             self,
             values     = {
-                "elements" : tuple( elements ),
+                "elements" : tuple(elements),
             },
             source_ref = source_ref
 
@@ -58,20 +61,18 @@ class ExpressionMakeSequenceBase(SideEffectsFromChildrenMixin,
     getElements = ExpressionChildrenHavingBase.childGetter("elements")
 
     def getSimulator(self):
-        # Abstract method, pylint: disable=R0201,W0613
+        # Abstract method, pylint: disable=R0201
         return None
 
     def computeExpression(self, constraint_collection):
-        # Children can tell all we need to know, pylint: disable=W0613
-
         elements = self.getElements()
 
-        for count, element in enumerate( elements ):
-            if element.willRaiseException( BaseException ):
+        for count, element in enumerate(elements):
+            if element.willRaiseException(BaseException):
                 from .NodeMakingHelpers import wrapExpressionWithSideEffects
 
                 result = wrapExpressionWithSideEffects(
-                    side_effects = elements[ : count ],
+                    side_effects = elements[:count],
                     new_node     = element,
                     old_node     = self
                 )
@@ -86,8 +87,6 @@ class ExpressionMakeSequenceBase(SideEffectsFromChildrenMixin,
         simulator = self.getSimulator()
         assert simulator is not None
 
-        from .NodeMakingHelpers import getComputationResult
-
         # The simulator is in fact callable if not None, pylint: disable=E1102
         return getComputationResult(
             node        = self,
@@ -96,7 +95,7 @@ class ExpressionMakeSequenceBase(SideEffectsFromChildrenMixin,
                 for element in
                 self.getElements()
             ),
-            description = "%s with constant arguments." % simulator
+            description = "%s with constant arguments." % simulator.__name__.title()
         )
 
     def mayHaveSideEffectsBool(self):
@@ -111,7 +110,8 @@ class ExpressionMakeSequenceBase(SideEffectsFromChildrenMixin,
     def getIterationLength(self):
         return len(self.getElements())
 
-    def canPredictIterationValues(self):
+    @staticmethod
+    def canPredictIterationValues():
         return True
 
     def getIterationValues(self):
@@ -141,7 +141,6 @@ class ExpressionMakeSequenceBase(SideEffectsFromChildrenMixin,
 Removed sequence creation for unused sequence."""
 
 
-
 class ExpressionMakeTuple(ExpressionMakeSequenceBase):
     kind = "EXPRESSION_MAKE_TUPLE"
 
@@ -180,7 +179,7 @@ class ExpressionMakeList(ExpressionMakeSequenceBase):
         self.replaceWith(result)
 
         return iter_node, "new_expression", """\
-Iteration of list reduced to tuple."""
+Iteration over list reduced to tuple."""
 
 
 class ExpressionMakeSet(ExpressionMakeSequenceBase):
@@ -197,6 +196,16 @@ class ExpressionMakeSet(ExpressionMakeSequenceBase):
     def getSimulator(self):
         return set
 
+    def mayRaiseException(self, exception_type):
+        for element in self.getElements():
+            if not element.isKnownToBeHashable():
+                return True
+
+            if element.mayRaiseException(exception_type):
+                return True
+
+        return False
+
     def computeExpressionIter1(self, iter_node, constraint_collection):
         result = ExpressionMakeTuple(
             elements   = self.getElements(),
@@ -206,7 +215,7 @@ class ExpressionMakeSet(ExpressionMakeSequenceBase):
         self.replaceWith(result)
 
         return iter_node, "new_expression", """\
-Iteration of set reduced to tuple."""
+Iteration over set reduced to tuple."""
 
 
 class ExpressionKeyValuePair(SideEffectsFromChildrenMixin,
@@ -228,16 +237,17 @@ class ExpressionKeyValuePair(SideEffectsFromChildrenMixin,
             source_ref = source_ref
         )
 
-    getKey = ExpressionChildrenHavingBase.childGetter( "key" )
-    getValue = ExpressionChildrenHavingBase.childGetter( "value" )
+    getKey = ExpressionChildrenHavingBase.childGetter("key")
+    getValue = ExpressionChildrenHavingBase.childGetter("value")
 
     def computeExpression(self, constraint_collection):
-        # Children can tell all we need to know, pylint: disable=W0613
         key = self.getKey()
 
         if key.willRaiseException(BaseException):
             return key, "new_raise", "Dictionary key raises exception"
 
+        # TODO: Maybe consider isKnownToBeHashable
+
         value = self.getValue()
 
         if value.willRaiseException(BaseException):
@@ -252,6 +262,13 @@ class ExpressionKeyValuePair(SideEffectsFromChildrenMixin,
 
         return self, None, None
 
+    def mayRaiseException(self, exception_type):
+        key = self.getKey()
+
+        return key.mayRaiseException(exception_type) or \
+               key.isKnownToBeHashable() is not True or \
+               self.getValue().mayRaiseException(exception_type)
+
 
 class ExpressionMakeDict(SideEffectsFromChildrenMixin,
                          ExpressionChildrenHavingBase):
@@ -275,7 +292,6 @@ class ExpressionMakeDict(SideEffectsFromChildrenMixin,
     getPairs = ExpressionChildrenHavingBase.childGetter("pairs")
 
     def computeExpression(self, constraint_collection):
-        # Children can tell all we need to know, pylint: disable=W0613
         pairs = self.getPairs()
 
         for count, pair in enumerate(pairs):
@@ -295,8 +311,8 @@ class ExpressionMakeDict(SideEffectsFromChildrenMixin,
         for pair in pairs:
             key = pair.getKey()
 
-            # TODO: Mutable key should cause something problematic.
-            if not key.isExpressionConstantRef() or key.isMutable():
+            # TODO: Mutable key should cause an exception raise to be produced.
+            if not key.isExpressionConstantRef() or not key.isKnownToBeHashable():
                 return self, None, None
 
             value = pair.getValue()
@@ -328,14 +344,21 @@ class ExpressionMakeDict(SideEffectsFromChildrenMixin,
         return new_node, "new_constant", """\
 Created dictionary found to be constant."""
 
+    def mayRaiseException(self, exception_type):
+        for pair in self.getPairs():
+            if pair.mayRaiseException(exception_type):
+                return True
+
+        return False
+
     def mayHaveSideEffectsBool(self):
         return False
 
     def isKnownToBeIterable(self, count):
-        return count is None or count == len( self.getPairs() )
+        return count is None or count == len(self.getPairs())
 
     def getIterationLength(self):
-        return len( self.getPairs() )
+        return len(self.getPairs())
 
     def canPredictIterationValues(self):
         # Dictionaries are fully predictable, pylint: disable=R0201
@@ -361,8 +384,7 @@ Created dictionary found to be constant."""
 
             if not key.isExpressionConstantRef() or not key.isStringConstant():
                 return False
-        else:
-            return True
+        return True
 
     def getMappingStringKeyPairs(self):
         return [
@@ -400,3 +422,12 @@ Created dictionary found to be constant."""
 
         return result, "new_statements", """\
 Removed sequence creation for unused sequence."""
+
+    def computeExpressionIter1(self, iter_node, constraint_collection):
+        return self, None, None
+
+        # TODO: This ought to be possible. Only difficulty is to
+        # preserve order of evaluation, by making values a side
+        # effect of the keys.
+        # return iter_node, "new_expression", """\
+# Iteration over dict reduced to tuple."""
