@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -28,8 +28,8 @@ CPython reference, and may escape.
 
 """
 
-from nuitka import Utils, Variables
-from nuitka.__past__ import iterItems
+from nuitka import Variables
+from nuitka.utils import Utils
 
 from .IndicatorMixins import (
     MarkGeneratorIndicator,
@@ -38,36 +38,43 @@ from .IndicatorMixins import (
 )
 from .NodeBases import (
     ChildrenHavingMixin,
+    ClosureGiverNodeBase,
     ClosureTakerMixin,
     CompileTimeConstantExpressionMixin,
     ExpressionChildrenHavingBase,
     ExpressionMixin,
     NodeBase,
-    ParameterHavingNodeBase,
-    SideEffectsFromChildrenMixin
+    SideEffectsFromChildrenMixin,
+    checkStatementsSequenceOrNone
 )
 from .ParameterSpecs import TooManyArguments, matchCall
 
 
 class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
-                             ParameterHavingNodeBase, ExpressionMixin,
+                             ClosureGiverNodeBase, ExpressionMixin,
                              MarkGeneratorIndicator,
                              MarkLocalsDictIndicator,
                              MarkUnoptimizedFunctionIndicator):
     # We really want these many ancestors, as per design, we add properties via
-    # base class mix-ins a lot, pylint: disable=R0901
+    # base class mix-ins a lot, leading to many instance attributes, and
+    # methods, pylint: disable=R0901,R0902
 
     kind = "EXPRESSION_FUNCTION_BODY"
 
-    named_children = ("body",)
+    named_children = (
+        "body",
+    )
+
+    checkers = {
+        "body" : checkStatementsSequenceOrNone
+    }
 
     if Utils.python_version >= 340:
         qualname_setup = None
 
     def __init__(self, provider, name, doc, parameters, source_ref,
                  is_class = False):
-        # Register ourselves immediately with the module.
-        provider.getParentModule().addFunction(self)
+        # These got too many details to cover, pylint: disable=R0915
 
         if is_class:
             code_prefix = "class"
@@ -116,17 +123,18 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
             early_closure = is_class
         )
 
-        ParameterHavingNodeBase.__init__(
+        ClosureGiverNodeBase.__init__(
             self,
             name        = name,
             code_prefix = code_prefix,
-            parameters  = parameters,
             source_ref  = source_ref
         )
 
         ChildrenHavingMixin.__init__(
             self,
-            values = {}
+            values = {
+                "body" : None # delayed
+            }
         )
 
         MarkGeneratorIndicator.__init__(self)
@@ -158,6 +166,19 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
         if Utils.python_version >= 340:
             self.qualname_provider = provider
 
+        self.parameters = parameters
+        self.parameters.setOwner(self)
+
+        # Register ourselves immediately with the module.
+        provider.getParentModule().addFunction(self)
+
+        self.registerProvidedVariables(
+            *self.parameters.getVariables()
+        )
+
+        self.constraint_collection = None
+
+
     def getDetails(self):
         return {
             "name"       : self.getFunctionName(),
@@ -168,7 +189,7 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
         }
 
     def getDetail(self):
-        return "named %s with %s" % ( self.name, self.parameters )
+        return "named %s with %s" % (self.getFunctionName(), self.parameters)
 
     def getParent(self):
         assert False
@@ -212,13 +233,16 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
         if provider.isPythonModule():
             return function_name
         elif provider.isClassDictCreation():
-            return provider.getFunctionQualname() + "." + function_name
+            return provider.getFunctionQualname() + '.' + function_name
         else:
             return provider.getFunctionQualname() + ".<locals>." + function_name
 
     def getDoc(self):
         return self.doc
 
+    def getParameters(self):
+        return self.parameters
+
     def getLocalVariableNames(self):
         return Variables.getNames(self.getLocalVariables())
 
@@ -242,16 +266,25 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
     def getVariables(self):
         return self.providing.values()
 
-    def removeVariable(self, variable):
-        assert variable in self.providing.values(), ( self.providing, variable )
+    def removeClosureVariable(self, variable):
+        assert variable in self.providing.values(), (self.providing, variable)
 
         del self.providing[variable.getName()]
 
         assert not variable.isParameterVariable() or \
-               not variable.getOwner() is self
+               variable.getOwner() is not self
 
         self.taken.remove(variable)
 
+    def removeUserVariable(self, variable):
+        assert variable in self.providing.values(), (self.providing, variable)
+
+        del self.providing[variable.getName()]
+
+        assert not variable.isParameterVariable() or \
+               variable.getOwner() is not self
+
+
     def getVariableForAssignment(self, variable_name):
         # print ("ASS func", self, variable_name)
 
@@ -268,21 +301,24 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
         if self.hasProvidedVariable(variable_name):
             result = self.getProvidedVariable(variable_name)
         else:
-            # For exec containing/star import containing, get a closure variable
-            # and if it is a module variable, only then make it a maybe local
-            # variable.
             result = self.getClosureVariable(
                 variable_name = variable_name
             )
 
+            # Remember that we need that closure variable for something, so
+            # we don't create it again all the time.
+            if not result.isModuleVariable():
+                self.registerProvidedVariable(result)
+
+            # For "exec" containing/star import containing, we get a
+            # closure variable already, but if it is a module variable,
+            # only then make it a maybe local variable.
             if self.isUnoptimized() and result.isModuleVariable():
                 result = Variables.MaybeLocalVariable(
-                    owner         = self,
-                    variable_name = variable_name
+                    owner          = self,
+                    maybe_variable = result
                 )
 
-            # Remember that we need that closure for something.
-            if not result.isModuleVariable():
                 self.registerProvidedVariable(result)
 
         return result
@@ -296,7 +332,7 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
         if self.isClassDictCreation():
             if variable_name == "__class__":
                 if Utils.python_version < 300:
-                    return self.provider.getVariableForReference(
+                    return self.provider.getVariableForClosure(
                         variable_name
                     )
                 elif Utils.python_version >= 340 and False: # TODO: Temporarily reverted
@@ -307,7 +343,7 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
 
                     return result
             else:
-                return self.provider.getVariableForReference(
+                return self.provider.getVariableForClosure(
                     variable_name
                 )
 
@@ -320,19 +356,13 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
         # print("createProvidedVariable", self, variable_name)
 
         if self.local_locals:
-            if self.isClassDictCreation():
-                return Variables.ClassVariable(
-                    owner         = self,
-                    variable_name = variable_name
-                )
-            else:
-                return Variables.LocalVariable(
-                    owner         = self,
-                    variable_name = variable_name
-                )
+            return Variables.LocalVariable(
+                owner         = self,
+                variable_name = variable_name
+            )
         else:
             # Make sure the provider knows it has to provide a variable of this
-            # name for the assigment.
+            # name for the assignment.
             self.provider.getVariableForAssignment(
                 variable_name = variable_name
             )
@@ -375,7 +405,7 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
     def computeExpression(self, constraint_collection):
         assert False
 
-        # Function body is quite irreplacable.
+        # Function body is quite irreplaceable.
         return self, None, None
 
     def getLocalsMode(self):
@@ -390,64 +420,8 @@ class ExpressionFunctionBody(ClosureTakerMixin, ChildrenHavingMixin,
         # TODO: Until we have something to re-order the arguments, we need to
         # skip this. For the immediate need, we avoid this complexity, as a
         # re-ordering will be needed.
-        if call_node.getNamedArgumentPairs():
-            return call_node, None, None
-
-        call_spec = self.getParameters()
-
-        try:
-            args_dict = matchCall(
-                func_name     = self.getName(),
-                args          = call_spec.getArgumentNames(),
-                star_list_arg = call_spec.getStarListArgumentName(),
-                star_dict_arg = call_spec.getStarDictArgumentName(),
-                num_defaults  = call_spec.getDefaultCount(),
-                positional    = call_node.getPositionalArguments(),
-                pairs         = ()
-            )
-
-            values = []
-
-            for positional_arg in call_node.getPositionalArguments():
-                for _arg_name, arg_value in iterItems( args_dict ):
-                    if arg_value is positional_arg:
-                        values.append( arg_value )
-
-            result = ExpressionFunctionCall(
-                function_body = self,
-                values        = values,
-                source_ref    = call_node.getSourceReference()
-            )
-
-            return (
-                result,
-                "new_statements", # TODO: More appropriate tag maybe.
-                """Replaced call to created function body '%s' with direct \
-function call""" % self.getName()
-            )
-
-        except TooManyArguments as e:
-            from .NodeMakingHelpers import (
-                makeRaiseExceptionReplacementExpressionFromInstance,
-                wrapExpressionWithSideEffects
-            )
-
-            result = wrapExpressionWithSideEffects(
-                new_node = makeRaiseExceptionReplacementExpressionFromInstance(
-                    expression     = call_node,
-                    exception      = e.getRealException()
-                ),
-                old_node           = call_node,
-                side_effects = call_node.extractPreCallSideEffects()
-            )
-
-            return (
-                result,
-                "new_statements,new_raise", # TODO: More appropriate tag maybe.
-                """Replaced call to created function body '%s' to argument \
-error""" % self.getName()
-            )
 
+        assert False, self
 
     def isCompileTimeConstant(self):
         # TODO: It's actually pretty much compile time accessible maybe.
@@ -557,6 +531,94 @@ class ExpressionFunctionCreation(SideEffectsFromChildrenMixin,
     def mayRaiseException(self, exception_type):
         return True
 
+    def computeExpressionCall(self, call_node, constraint_collection):
+        # TODO: Until we have something to re-order the keyword arguments, we
+        # need to skip this. For the immediate need, we avoid this complexity,
+        # as a re-ordering will be needed.
+        call_kw = call_node.getCallKw()
+
+        # TODO: empty constant can happen too
+        if call_kw is not None:
+            return call_node, None, None
+
+        call_args = call_node.getCallArgs()
+
+        if call_args is None:
+            args_tuple = ()
+        elif call_args.isExpressionConstantRef() or \
+             call_args.isExpressionMakeTuple():
+            args_tuple = call_args.getIterationValues()
+        else:
+            # TODO: Can this even happen, i.e. does the above check make
+            # sense.
+            assert False, call_args
+
+            return call_node, None, None
+
+
+        # TODO: Actually the above disables it entirely, as it is at least
+        # the empty dictionary node in any case. We will need some enhanced
+        # interfaces for "matchCall" to work on.
+
+        call_spec = self.getFunctionRef().getFunctionBody().getParameters()
+
+        try:
+            args_dict = matchCall(
+                func_name     = self.getName(),
+                args          = call_spec.getArgumentNames(),
+                star_list_arg = call_spec.getStarListArgumentName(),
+                star_dict_arg = call_spec.getStarDictArgumentName(),
+                num_defaults  = call_spec.getDefaultCount(),
+                positional    = args_tuple,
+                pairs         = ()
+            )
+
+            values = []
+
+            for positional_arg in args_tuple:
+                for arg_value in args_dict.values():
+                    if arg_value is positional_arg:
+                        values.append(arg_value)
+                        break
+                else:
+                    assert False
+
+            result = ExpressionFunctionCall(
+                function   = self,
+                values     = values,
+                source_ref = call_node.getSourceReference()
+            )
+
+            return (
+                result,
+                "new_statements", # TODO: More appropriate tag maybe.
+                """\
+Replaced call to created function body '%s' with direct \
+function call.""" % self.getName()
+            )
+
+        except TooManyArguments as e:
+            from .NodeMakingHelpers import (
+                makeRaiseExceptionReplacementExpressionFromInstance,
+                wrapExpressionWithSideEffects
+            )
+
+            result = wrapExpressionWithSideEffects(
+                new_node     = makeRaiseExceptionReplacementExpressionFromInstance(
+                    expression = call_node,
+                    exception  = e.getRealException()
+                ),
+                old_node     = call_node,
+                side_effects = call_node.extractPreCallSideEffects()
+            )
+
+            return (
+                result,
+                "new_statements,new_raise", # TODO: More appropriate tag maybe.
+                """Replaced call to created function body '%s' to argument \
+error""" % self.getName()
+            )
+
 
 class ExpressionFunctionRef(NodeBase, ExpressionMixin):
     kind = "EXPRESSION_FUNCTION_REF"
@@ -571,9 +633,6 @@ class ExpressionFunctionRef(NodeBase, ExpressionMixin):
 
         self.function_body = function_body
 
-        # SSA trace based information about the function.
-        self.collection = None
-
     def getDetails(self):
         return {
             "function" : self.function_body.getCodeName()
@@ -598,14 +657,36 @@ class ExpressionFunctionRef(NodeBase, ExpressionMixin):
 
         owning_module.addUsedFunction(function_body)
 
-        from nuitka.optimizations.ConstraintCollections import \
+        from nuitka.optimizations.TraceCollections import \
             ConstraintCollectionFunction
 
-        collection = ConstraintCollectionFunction(
+        # TODO: Doesn't this mean, we can do this multiple times by doing it
+        # in the reference. We should do it in the body, and there we should
+        # limit us to only doing it once per module run, e.g. being based on
+        # presence in used functions of the module already.
+        old_collection = function_body.constraint_collection
+
+        function_body.constraint_collection = ConstraintCollectionFunction(
             parent        = constraint_collection,
             function_body = function_body
         )
-        function_body.collection = collection
+
+        statements_sequence = function_body.getBody()
+
+        if statements_sequence is not None and \
+           not statements_sequence.getStatements():
+            function_body.setStatements(None)
+            statements_sequence = None
+
+        if statements_sequence is not None:
+            result = statements_sequence.computeStatementsSequence(
+                constraint_collection = function_body.constraint_collection
+            )
+
+            if result is not statements_sequence:
+                function_body.setBody(result)
+
+        function_body.constraint_collection.updateFromCollection(old_collection)
 
         # TODO: Function collection may now know something.
         return self, None, None
