@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -18,13 +18,17 @@
 """ Loop nodes.
 
 There are for and loop nodes, but both are reduced to loops with break/continue
-statements for it. These reformulations require that optimization of loops has
+statements for it. These re-formulations require that optimization of loops has
 to be very general, yet the node type for loop, becomes very simple.
 """
 
-from nuitka.tree.Extractions import getVariablesWritten
+from nuitka.optimizations.TraceCollections import ConstraintCollectionLoop
 
-from .NodeBases import NodeBase, StatementChildrenHavingBase
+from .NodeBases import (
+    NodeBase,
+    StatementChildrenHavingBase,
+    checkStatementsSequenceOrNone
+)
 
 
 class StatementLoop(StatementChildrenHavingBase):
@@ -34,6 +38,10 @@ class StatementLoop(StatementChildrenHavingBase):
         "frame",
     )
 
+    checkers = {
+        "frame" : checkStatementsSequenceOrNone
+    }
+
     def __init__(self, body, source_ref):
         StatementChildrenHavingBase.__init__(
             self,
@@ -43,11 +51,11 @@ class StatementLoop(StatementChildrenHavingBase):
             source_ref = source_ref
         )
 
-        # For code generation, so it knows wether to produce the exit target or
-        # not.
+        # For code generation, so it knows if an exit target is needed.
         self.has_break = False
 
     getLoopBody = StatementChildrenHavingBase.childGetter("frame")
+    setLoopBody = StatementChildrenHavingBase.childSetter("frame")
 
     def mayReturn(self):
         loop_body = self.getLoopBody()
@@ -68,24 +76,20 @@ class StatementLoop(StatementChildrenHavingBase):
     def isStatementAborting(self):
         loop_body = self.getLoopBody()
 
-        if loop_body is not None:
-            return not loop_body.mayBreak()
-        else:
+        if loop_body is None:
             return True
+        else:
+            return not loop_body.mayBreak()
 
     def computeStatement(self, constraint_collection):
+        outer_constraint_collection = constraint_collection
+        constraint_collection = ConstraintCollectionLoop(constraint_collection)
+
         loop_body = self.getLoopBody()
 
         if loop_body is not None:
-            # Look ahead. what will be written.
-            variable_writes = getVariablesWritten(loop_body)
-
-            # Mark all variables as unknown that are written in the loop body,
-            # so it destroys the assumptions for loop turn around.
-            for variable, _variable_version in variable_writes:
-                constraint_collection.markActiveVariableAsUnknown(
-                    variable = variable
-                )
+            # Look ahead. what will be written and degrade about that.
+            constraint_collection.degradePartiallyFromCode(loop_body)
 
             result = loop_body.computeStatementsSequence(
                 constraint_collection = constraint_collection
@@ -93,7 +97,7 @@ class StatementLoop(StatementChildrenHavingBase):
 
             # Might be changed.
             if result is not loop_body:
-                loop_body.replaceWith(result)
+                self.setLoopBody(result)
                 loop_body = result
 
         # Consider trailing "continue" statements, these have no effect, so we
@@ -108,19 +112,28 @@ class StatementLoop(StatementChildrenHavingBase):
             # be discarded.
             last_statement = statements[-1]
             if last_statement.isStatementContinueLoop():
-                # TODO: This is unnecessarily ugly way of doing this, simply
-                # replacing the continue should be good enough.
-                loop_body.removeStatement(last_statement)
-                statements = loop_body.getStatements()
-
-                if not statements:
-                    loop_body.replaceWith(None)
+                if len(statements) == 1:
+                    self.setLoopBody(None)
                     loop_body = None
+                else:
+                    last_statement.replaceWith(None)
 
                 constraint_collection.signalChange(
                     "new_statements",
                     last_statement.getSourceReference(),
-                    "Removed continue as last statement of loop."
+                    """\
+Removed useless terminal 'continue' as last statement of loop."""
+                )
+
+        break_collections = constraint_collection.getLoopBreakCollections()
+
+        if break_collections:
+            outer_constraint_collection.replaceBranch(break_collections[0])
+
+            for break_collection in break_collections[1:]:
+                outer_constraint_collection.mergeBranches(
+                    collection_yes = break_collection,
+                    collection_no  = None
                 )
 
         # Consider leading "break" statements, they should be the only, and
@@ -136,7 +149,7 @@ class StatementLoop(StatementChildrenHavingBase):
 
             if len(statements) == 1 and statements[-1].isStatementBreakLoop():
                 return None, "new_statements", """\
-Removed loop immediately broken."""
+Removed useless loop with immediate 'break' statement."""
 
         return self, None, None
 
@@ -145,29 +158,29 @@ class StatementContinueLoop(NodeBase):
     kind = "STATEMENT_CONTINUE_LOOP"
 
     def __init__(self, source_ref):
-        NodeBase.__init__( self, source_ref = source_ref )
+        NodeBase.__init__(self, source_ref = source_ref)
 
     def isStatementAborting(self):
         return True
 
-    def computeStatement(self, constraint_collection):
-        # This statement being aborting, will already tell everything. TODO: The
-        # fine difference that this jumps to loop start for sure, should be
-        # represented somehow one day.
-        return self, None, None
-
     def mayRaiseException(self, exception_type):
         return False
 
     def mayContinue(self):
         return True
 
+    def computeStatement(self, constraint_collection):
+        # This statement being aborting, will already tell everything.
+        constraint_collection.onLoopContinue(constraint_collection)
+
+        return self, None, None
+
 
 class StatementBreakLoop(NodeBase):
     kind = "STATEMENT_BREAK_LOOP"
 
     def __init__(self, source_ref):
-        NodeBase.__init__( self, source_ref = source_ref )
+        NodeBase.__init__(self, source_ref = source_ref)
 
     def isStatementAborting(self):
         return True
@@ -179,7 +192,7 @@ class StatementBreakLoop(NodeBase):
         return True
 
     def computeStatement(self, constraint_collection):
-        # This statement being aborting, will already tell everything. TODO: The
-        # fine difference that this exits the loop for sure, should be
-        # represented somehow one day.
+        # This statement being aborting, will already tell everything.
+        constraint_collection.onLoopBreak(constraint_collection)
+
         return self, None, None
