@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,15 +24,17 @@ These classes provide the generic base classes available for nodes.
 
 from nuitka import Options, Tracing, TreeXML, Variables
 from nuitka.__past__ import iterItems
-from nuitka.odict import OrderedDict
-from nuitka.oset import OrderedSet
+from nuitka.containers.odict import OrderedDict
+from nuitka.containers.oset import OrderedSet
+from nuitka.utils.InstanceCounters import counted_del, counted_init
+from nuitka.VariableRegistry import addVariableUsage
 
-lxml = TreeXML.lxml
 
 class NodeCheckMetaClass(type):
     kinds = set()
 
     def __new__(cls, name, bases, dictionary):
+        # This is in conflict with either PyDev or Pylint, pylint: disable=C0204
         assert len(bases) == len(set(bases))
 
         return type.__new__(cls, name, bases, dictionary)
@@ -54,7 +56,7 @@ class NodeCheckMetaClass(type):
                     return value.title()
 
             kind_to_name_part = "".join(
-                [convert(x) for x in kind.split("_")]
+                [convert(x) for x in kind.split('_')]
             )
             assert name.endswith(kind_to_name_part), \
               (name, kind_to_name_part)
@@ -67,26 +69,26 @@ class NodeCheckMetaClass(type):
                 return self.kind == kind
 
             if not hasattr(NodeBase, checker_method):
-                setattr( NodeBase, checker_method, checkKind )
+                setattr(NodeBase, checker_method, checkKind)
 
         type.__init__(cls, name, bases, dictionary)
 
 # For every node type, there is a test, and then some more members,
-# pylint: disable=R0904
 
 # For Python2/3 compatible source, we create a base class that has the metaclass
 # used and doesn't require making a choice.
-NodeMetaClassBase = NodeCheckMetaClass("NodeMetaClassBase", (object, ), {})
+NodeMetaClassBase = NodeCheckMetaClass("NodeMetaClassBase", (object,), {})
+
 
 class NodeBase(NodeMetaClassBase):
     kind = None
 
-    # Must be overloaded by expressions.
-    value_friend_maker = None
-
+    @counted_init
     def __init__(self, source_ref):
-        # The base class has no __init__ worth calling, pylint: disable=W0231
+        # The base class has no __init__ worth calling.
 
+        # Check source reference to meet basic standards, so we note errors
+        # when they occur.
         assert source_ref is not None
         assert source_ref.line is not None
 
@@ -94,9 +96,7 @@ class NodeBase(NodeMetaClassBase):
 
         self.source_ref = source_ref
 
-    def isNode(self):
-        # Virtual method, pylint: disable=R0201
-        return True
+    __del__ = counted_del()
 
     def __repr__(self):
         # This is to avoid crashes, because of bugs in detail.
@@ -109,14 +109,14 @@ class NodeBase(NodeMetaClassBase):
         if not detail:
             return "<Node %s>" % self.getDescription()
         else:
-            return "<Node %s %s>" % ( self.getDescription(), detail )
+            return "<Node %s %s>" % (self.getDescription(), detail)
 
     def getDescription(self):
         """ Description of the node, intented for use in __repr__ and
             graphical display.
 
         """
-        return "%s at %s" % ( self.kind, self.source_ref.getAsString() )
+        return "%s at %s" % (self.kind, self.source_ref.getAsString())
 
     def getDetails(self):
         """ Details of the node, intended for use in __repr__ and dumps.
@@ -130,8 +130,7 @@ class NodeBase(NodeMetaClassBase):
             display.
 
         """
-        # Virtual method, pylint: disable=R0201
-        return str( self.getDetails() )[1:-1]
+        return str(self.getDetails())[1:-1]
 
     def getParent(self):
         """ Parent of the node. Every node except modules have to have a parent.
@@ -139,7 +138,7 @@ class NodeBase(NodeMetaClassBase):
         """
 
         if self.parent is None and not self.isPythonModule():
-            assert False, ( self,  self.source_ref )
+            assert False, (self,  self.source_ref)
 
         return self.parent
 
@@ -153,7 +152,7 @@ class NodeBase(NodeMetaClassBase):
         while True:
             current = current.getParent()
 
-            result.append( current )
+            result.append(current)
 
             if current.isPythonModule() or current.isExpressionFunctionBody():
                 break
@@ -163,6 +162,19 @@ class NodeBase(NodeMetaClassBase):
         result.reverse()
         return result
 
+    def getChildName(self):
+        """ Return the role in the current parent, subject to changes.
+
+        """
+        parent = self.getParent()
+
+        for key, value in parent.child_values.items():
+            if self is value:
+                return key
+
+        # TODO: Not checking tuples yet
+        return None
+
     def getParentFunction(self):
         """ Return the parent that is a function.
 
@@ -182,9 +194,9 @@ class NodeBase(NodeMetaClassBase):
         parent = self
 
         while not parent.isPythonModule():
-            if hasattr( parent, "provider" ):
-                # After we checked, we can use it, will be much faster,
-                # pylint: disable=E1101
+            if hasattr(parent, "provider"):
+                # After we checked, we can use it, will be much faster route
+                # to take.
                 parent = parent.provider
             else:
                 parent = parent.getParent()
@@ -193,8 +205,7 @@ class NodeBase(NodeMetaClassBase):
 
     def isParentVariableProvider(self):
         # Check if it's a closure giver, in which cases it can provide variables,
-        # pylint: disable=E1101
-        return isinstance( self, ClosureGiverNodeBase )
+        return isinstance(self, ClosureGiverNodeBase)
 
     def getParentVariableProvider(self):
         parent = self.getParent()
@@ -216,35 +227,70 @@ class NodeBase(NodeMetaClassBase):
 
             current = current.getParent()
 
-
     def getSourceReference(self):
         return self.source_ref
 
+    def setCompatibleSourceReference(self, source_ref):
+        """ Bug compatible line numbers information.
+
+            As CPython outputs the last bit of bytecode executed, and not the
+            line of the operation. For example calls, output the line of the
+            last argument, as opposed to the line of the operation start.
+
+            For tests, we wants to be compatible. In improved more, we are
+            not being fully compatible, and just drop it altogether.
+        """
+
+        # Getting the same source reference can be dealt with quickly, so do
+        # this first.
+        if self.source_ref is not source_ref and \
+           Options.isFullCompat() and \
+           self.source_ref != source_ref:
+            # An attribute outside of "__init__", so we save one memory for the
+            # most cases. Very few cases involve splitting across lines.
+            # pylint: disable=W0201
+            self.effective_source_ref = source_ref
+
+
+    def getCompatibleSourceReference(self):
+        """ Bug compatible line numbers information.
+
+            See above.
+        """
+        return getattr(self, "effective_source_ref", self.source_ref)
+
     def asXml(self):
-        result = lxml.etree.Element(
+        line = self.getSourceReference().getLineNumber()
+
+        result = TreeXML.Element(
             "node",
             kind = self.__class__.__name__,
-            line = "%s" % self.getSourceReference().getLineNumber()
+            line = "%s" % line
         )
 
-        for key, value in iterItems( self.getDetails() ):
-            value = str( value )
+        compat_line = self.getCompatibleSourceReference().getLineNumber()
 
-            if value.startswith( "<" ) and value.endswith( ">" ):
+        if compat_line != line:
+            result.attrib["compat_line"] = str(compat_line)
+
+        for key, value in iterItems(self.getDetails()):
+            value = str(value)
+
+            if value.startswith('<') and value.endswith('>'):
                 value = value[1:-1]
 
-            result.set( key, str( value ) )
+            result.set(key, str(value))
 
         for name, children in self.getVisitableNodesNamed():
-            if type( children ) not in ( list, tuple ):
-                children = ( children, )
+            if type(children) not in (list, tuple):
+                children = (children,)
 
-            role = lxml.etree.Element(
+            role = TreeXML.Element(
                 "role",
                 name = name
             )
 
-            result.append( role )
+            result.append(role)
 
             for child in children:
                 if child is not None:
@@ -255,57 +301,56 @@ class NodeBase(NodeMetaClassBase):
         return result
 
     def dump(self, level = 0):
-        Tracing.printIndented( level, self )
-        Tracing.printSeparator( level )
+        Tracing.printIndented(level, self)
+        Tracing.printSeparator(level)
 
         for visitable in self.getVisitableNodes():
-            visitable.dump( level + 1 )
+            visitable.dump(level + 1)
 
-        Tracing.printSeparator( level )
+        Tracing.printSeparator(level)
+
+    @staticmethod
+    def isPythonModule():
+        # For overload by module nodes
+        return False
 
     def isExpression(self):
-        return self.kind.startswith( "EXPRESSION_" )
+        return self.kind.startswith("EXPRESSION_")
 
     def isStatement(self):
-        return self.kind.startswith( "STATEMENT_" )
+        return self.kind.startswith("STATEMENT_")
 
     def isExpressionBuiltin(self):
-        return self.kind.startswith( "EXPRESSION_BUILTIN_" )
-
-    def isOperation(self):
-        return self.kind.startswith( "EXPRESSION_OPERATION_" )
-
-    def isExpressionOperationBool2(self):
-        return self.kind.startswith( "EXPRESSION_BOOL_" )
+        return self.kind.startswith("EXPRESSION_BUILTIN_")
 
     def isStatementReraiseException(self):
-        # Virtual method, pylint: disable=R0201,W0613
+        # Virtual method, pylint: disable=R0201
         return False
 
     def isExpressionMakeSequence(self):
-        # Virtual method, pylint: disable=R0201,W0613
+        # Virtual method, pylint: disable=R0201
         return False
 
     def isIteratorMaking(self):
-        # Virtual method, pylint: disable=R0201,W0613
+        # Virtual method, pylint: disable=R0201
         return False
 
     def isNumberConstant(self):
-        # Virtual method, pylint: disable=R0201,W0613
+        # Virtual method, pylint: disable=R0201
         return False
 
     def isExpressionCall(self):
-        # Virtual method, pylint: disable=R0201,W0613
+        # Virtual method, pylint: disable=R0201
         return False
 
     def visit(self, context, visitor):
-        visitor( self )
+        visitor(self)
 
         for visitable in self.getVisitableNodes():
-            visitable.visit( context, visitor )
+            visitable.visit(context, visitor)
 
     def getVisitableNodes(self):
-        # Virtual method, pylint: disable=R0201,W0613
+        # Virtual method, pylint: disable=R0201
         return ()
 
     def getVisitableNodesNamed(self):
@@ -318,20 +363,13 @@ class NodeBase(NodeMetaClassBase):
             new_node = new_node
         )
 
-    def discard(self):
-        """ The node has become unused. """
-        # print "Discarding", self
-
-        if Options.isExperimental():
-            self.parent = None
-
     def getName(self):
-        # Virtual method, pylint: disable=R0201,W0613
+        # Virtual method, pylint: disable=R0201
         return None
 
     def mayHaveSideEffects(self):
         """ Unless we are told otherwise, everything may have a side effect. """
-        # Virtual method, pylint: disable=R0201,W0613
+        # Virtual method, pylint: disable=R0201
 
         return True
 
@@ -340,15 +378,14 @@ class NodeBase(NodeMetaClassBase):
 
     def mayHaveSideEffectsBool(self):
         """ Unless we are told otherwise, everything may have a side effect. """
-        # Virtual method, pylint: disable=R0201,W0613
+        # Virtual method, pylint: disable=R0201
 
         return True
 
     def extractSideEffects(self):
         """ Unless defined otherwise, the expression is the side effect. """
-        # Virtual method, pylint: disable=R0201,W0613
 
-        return ( self, )
+        return (self,)
 
     def mayRaiseException(self, exception_type):
         """ Unless we are told otherwise, everything may raise everything. """
@@ -356,13 +393,21 @@ class NodeBase(NodeMetaClassBase):
 
         return True
 
+    def mayRaiseExceptionBool(self, exception_type):
+        """ Unless we are told otherwise, everything may raise being checked. """
+        # Virtual method, pylint: disable=R0201,W0613
+
+        return True
+
     def mayReturn(self):
         return "_RETURN" in self.kind
 
     def mayBreak(self):
+        # For overload, pylint: disable=R0201
         return False
 
     def mayContinue(self):
+        # For overload, pylint: disable=R0201
         return False
 
     def needsFrame(self):
@@ -379,13 +424,12 @@ class NodeBase(NodeMetaClassBase):
 
     def isIndexable(self):
         """ Unless we are told otherwise, it's not indexable. """
-        # Virtual method, pylint: disable=R0201,W0613
+        # Virtual method, pylint: disable=R0201
 
         return False
 
     def isStatementAborting(self):
         """ Is the node aborting, control flow doesn't continue after this node.  """
-        # Virtual method, pylint: disable=R0201
         assert self.isStatement(), self.kind
 
         return False
@@ -393,12 +437,12 @@ class NodeBase(NodeMetaClassBase):
     def needsLocalsDict(self):
         """ Node requires a locals dictionary by provider. """
 
-        # Virtual method, pylint: disable=R0201,W0613
+        # Virtual method, pylint: disable=R0201
         return False
 
     def getIntegerValue(self):
         """ Node as integer value, if possible."""
-        # Virtual method, pylint: disable=R0201,W0613
+        # Virtual method, pylint: disable=R0201
         return None
 
 
@@ -406,7 +450,7 @@ class CodeNodeBase(NodeBase):
     def __init__(self, name, code_prefix, source_ref):
         assert name is not None
 
-        NodeBase.__init__( self, source_ref = source_ref )
+        NodeBase.__init__(self, source_ref = source_ref)
 
         self.name = name
         self.code_prefix = code_prefix
@@ -434,9 +478,9 @@ class CodeNodeBase(NodeBase):
             name = current.getName()
 
             if name is not None:
-                result = "%s__%s" % ( name, result )
+                result = "%s__%s" % (name, result)
 
-        assert "<" not in result, result
+        assert '<' not in result, result
 
         return result
 
@@ -445,16 +489,16 @@ class CodeNodeBase(NodeBase):
             provider = self.getParentVariableProvider()
             parent_name = provider.getCodeName()
 
-            uid = "_%d" % provider.getChildUID( self )
+            uid = "_%d" % provider.getChildUID(self)
 
-            assert isinstance( self, CodeNodeBase )
+            assert isinstance(self, CodeNodeBase)
 
             if self.name:
-                name = uid + "_" + self.name
+                name = uid + '_' + self.name
             else:
                 name = uid
 
-            self.code_name = "%s%s_of_%s" % ( self.code_prefix, name, parent_name )
+            self.code_name = "%s%s_of_%s" % (self.code_prefix, name, parent_name)
 
         return self.code_name
 
@@ -466,22 +510,20 @@ class CodeNodeBase(NodeBase):
 
         return self.uids[ node.kind ]
 
+
 class ChildrenHavingMixin:
     named_children = ()
 
     checkers = {}
 
     def __init__(self, values):
-        assert len(self.named_children)
-        assert type(self.named_children) is tuple
+        assert type(self.named_children) is tuple and len(self.named_children)
 
-        for key in values.keys():
-            assert key in self.named_children, key
+        # Check for completeness of given values, everything should be there
+        # but of course, might be put to None.
+        assert set(values.keys()) == set(self.named_children)
 
-        # Default non-given values to None. TODO: Good idea? Better check for
-        # completeness instead.
-        self.child_values = dict.fromkeys(self.named_children)
-        self.child_values.update(values)
+        self.child_values = dict(values)
 
         for key, value in self.child_values.items():
             if key in self.checkers:
@@ -489,13 +531,17 @@ class ChildrenHavingMixin:
 
             assert type(value) is not list, key
 
-            if type( value ) is tuple:
+            if type(value) is tuple:
                 assert None not in value, key
 
                 for val in value:
                     val.parent = self
             elif value is not None:
                 value.parent = self
+            elif value is None:
+                pass
+            else:
+                assert False, type(value)
 
     def setChild(self, name, value):
         """ Set a child value.
@@ -526,15 +572,6 @@ class ChildrenHavingMixin:
 
         self.child_values[name] = value
 
-        # TODO: Enable this
-        if old_value is not None:
-            if type(old_value) is tuple:
-                for val in old_value:
-                    if val not in value:
-                        val.discard()
-            else:
-                old_value.discard()
-
     def getChild(self, name):
         # Only accept legal child names
         assert name in self.child_values, name
@@ -566,17 +603,17 @@ class ChildrenHavingMixin:
 
             if value is None:
                 pass
-            elif type( value ) is tuple:
-                result += list( value )
-            elif isinstance( value, NodeBase ):
-                result.append( value )
+            elif type(value) is tuple:
+                result += list(value)
+            elif isinstance(value, NodeBase):
+                result.append(value)
             else:
                 raise AssertionError(
                     self,
                     "has illegal child", name, value, value.__class__
                 )
 
-        return tuple( result )
+        return tuple(result)
 
     def getVisitableNodesNamed(self):
         result = []
@@ -584,7 +621,7 @@ class ChildrenHavingMixin:
         for name in self.named_children:
             value = self.child_values[ name ]
 
-            result.append( ( name, value ) )
+            result.append((name, value))
 
         return result
 
@@ -629,7 +666,7 @@ class ChildrenHavingMixin:
 
                     return key
             else:
-                assert False, ( key, value, value.__class__ )
+                assert False, (key, value, value.__class__)
 
         raise AssertionError(
             "Didn't find child",
@@ -642,7 +679,7 @@ class ChildrenHavingMixin:
         values = {}
 
         for key, value in self.child_values.items():
-            assert type( value ) is not list, key
+            assert type(value) is not list, key
 
             if value is None:
                 values[key] = None
@@ -664,6 +701,8 @@ class ChildrenHavingMixin:
         )
 
         try:
+            # Using star dictionary arguments here for generic use,
+            # pylint: disable=E1123
             return self.__class__(
                 source_ref = source_ref,
                 **values
@@ -697,15 +736,15 @@ class ClosureGiverNodeBase(CodeNodeBase):
 
     def getProvidedVariable(self, variable_name):
         if variable_name not in self.providing:
-            self.providing[ variable_name ] = self.createProvidedVariable(
+            self.providing[variable_name] = self.createProvidedVariable(
                 variable_name = variable_name
             )
 
-        return self.providing[ variable_name ]
+        return self.providing[variable_name]
 
     def createProvidedVariable(self, variable_name):
-        # Virtual method, pylint: disable=R0201,W0613
-        assert type( variable_name ) is str
+        # Virtual method, pylint: disable=R0201
+        assert type(variable_name) is str
 
         return None
 
@@ -760,41 +799,23 @@ class ClosureGiverNodeBase(CodeNodeBase):
 
         self.temp_variables[full_name] = result
 
+        addVariableUsage(result, self)
+
         return result
 
     def getTempVariable(self, temp_scope, name):
         if temp_scope is not None:
-            full_name = "%s__%s" % ( temp_scope, name )
+            full_name = "%s__%s" % (temp_scope, name)
         else:
             full_name = name
 
-        return self.temp_variables[ full_name ]
+        return self.temp_variables[full_name]
 
     def getTempVariables(self):
-        return tuple( self.temp_variables.values() )
+        return tuple(self.temp_variables.values())
 
     def removeTempVariable(self, variable):
-        del self.temp_variables[ variable.getName() ]
-
-
-class ParameterHavingNodeBase(ClosureGiverNodeBase):
-    def __init__(self, name, code_prefix, parameters, source_ref):
-        ClosureGiverNodeBase.__init__(
-            self,
-            name        = name,
-            code_prefix = code_prefix,
-            source_ref  = source_ref
-        )
-
-        self.parameters = parameters
-        self.parameters.setOwner( self )
-
-        self.registerProvidedVariables(
-            *self.parameters.getVariables()
-        )
-
-    def getParameters(self):
-        return self.parameters
+        del self.temp_variables[variable.getName()]
 
 
 class ClosureTakerMixin:
@@ -822,21 +843,14 @@ class ClosureTakerMixin:
         # There is no maybe with closures. It means, it is closure variable in
         # this case.
         if result.isMaybeLocalVariable():
-            # This mixin is used with nodes only, but doesn't want to inherit
-            # from it, pylint: disable=E1101
-            result = self.getParentModule().getVariableForClosure(
-                variable_name = variable_name
-            )
+            result = result.getMaybeVariable()
 
-        return self.addClosureVariable(result)
+        if not result.isModuleVariable():
+            self.addClosureVariable(result)
 
-    def addClosureVariables(self, *variables):
-        for variable in variables:
-            self.addClosureVariable(variable)
+        return result
 
     def addClosureVariable(self, variable):
-        variable = variable.makeReference(self)
-
         self.taken.add(variable)
 
         return variable
@@ -848,7 +862,7 @@ class ClosureTakerMixin:
                     take
                     for take in
                     self.taken
-                    if take.isClosureReference()
+                    if not take.isModuleVariable()
                 ],
                 key = lambda x : x.getName()
             )
@@ -858,15 +872,13 @@ class ClosureTakerMixin:
         for variable in self.taken:
             if variable.getName() == variable_name:
                 return True
-        else:
-            return False
+        return False
 
     def getTakenVariable(self, variable_name):
         for variable in self.taken:
             if variable.getName() == variable_name:
                 return variable
-        else:
-            return None
+        return None
 
     def isEarlyClosure(self):
         """ Early closure taking means immediate binding of references.
@@ -901,10 +913,18 @@ class ExpressionMixin:
         """ Return known truth value. The "None" value indicates unknown. """
 
         if self.isCompileTimeConstant():
-            return bool( self.getCompileTimeConstant() )
+            return bool(self.getCompileTimeConstant())
         else:
             return None
 
+    def mayBeNone(self):
+        """ Could this evaluate to be "None".
+
+            Yes or no. Defaults to pessimistic yes."""
+        # For overload, pylint: disable=R0201
+
+        return True
+
     def isKnownToBeIterable(self, count):
         """ Can be iterated at all (count is None) or exactly count times.
 
@@ -923,15 +943,6 @@ class ExpressionMixin:
         # Virtual method, pylint: disable=R0201,W0613
         return False
 
-    def mayProvideReference(self):
-        """ May at run time produce a reference.
-
-        This then would have to be consumed or released in a reliable way.
-        """
-
-        # Virtual method, pylint: disable=R0201
-        return True
-
     def getIterationLength(self):
         """ Value that "len" or "PyObject_Size" would give, if known.
 
@@ -943,22 +954,21 @@ class ExpressionMixin:
 
     def getStringValue(self):
         """ Node as integer value, if possible."""
-        # Virtual method, pylint: disable=R0201,W0613
+        # Virtual method, pylint: disable=R0201
         return None
 
     def getStrValue(self):
         """ Value that "str" or "PyObject_Str" would give, if known.
 
-            Otherwise it is "None" to indicate unknown.
+            Otherwise it is "None" to indicate unknown. Users must not
+            forget to take side effects into account, when replacing a
+            node with its string value.
         """
         string_value = self.getStringValue()
 
         if string_value is not None:
             from .NodeMakingHelpers import makeConstantReplacementNode
 
-            # TODO: Side effects should be considered, getStringValue may be
-            # omitting effects.
-
             return makeConstantReplacementNode(
                 node     = self,
                 constant = string_value
@@ -966,6 +976,13 @@ class ExpressionMixin:
 
         return None
 
+    def isKnownToBeHashable(self):
+        """ Is the value hashable, i.e. suitable for dictionary/set keying."""
+
+        # Virtual method, pylint: disable=R0201
+        # Unknown by default.
+        return None
+
     def onRelease(self, constraint_collection):
         # print "onRelease", self
         pass
@@ -994,8 +1011,11 @@ class ExpressionMixin:
     def computeExpressionAttribute(self, lookup_node, attribute_name,
                                     constraint_collection):
         # By default, an attribute lookup may change everything about the lookup
-        # source. Virtual method, pylint: disable=R0201,W0613
-        constraint_collection.removeKnowledge( lookup_node )
+        # source. Virtual method, pylint: disable=W0613
+        constraint_collection.removeKnowledge(lookup_node)
+
+        # Any code could be run, note that.
+        constraint_collection.onControlFlowEscape(self)
 
         return lookup_node, None, None
 
@@ -1003,30 +1023,43 @@ class ExpressionMixin:
                                    constraint_collection):
         # By default, an subscript may change everything about the lookup
         # source.
-        constraint_collection.removeKnowledge( lookup_node )
+        constraint_collection.removeKnowledge(lookup_node)
+        constraint_collection.removeKnowledge(subscript)
+
+        # Any code could be run, note that.
+        constraint_collection.onControlFlowEscape(self)
 
         return lookup_node, None, None
 
     def computeExpressionSlice(self, lookup_node, lower, upper,
                                constraint_collection):
         # By default, a slicing may change everything about the lookup source.
-        constraint_collection.removeKnowledge( lookup_node )
+        # Virtual method, pylint: disable=R0201,W0613
+        constraint_collection.removeKnowledge(lookup_node)
 
         return lookup_node, None, None
 
     def computeExpressionCall(self, call_node, constraint_collection):
+        # Virtual method, pylint: disable=R0201
         call_node.getCalled().onContentEscapes(constraint_collection)
 
         return call_node, None, None
 
     def computeExpressionIter1(self, iter_node, constraint_collection):
+        # Virtual method, pylint: disable=R0201
         iter_node.getValue().onContentEscapes(constraint_collection)
 
         return iter_node, None, None
 
     def computeExpressionOperationNot(self, not_node, constraint_collection):
+        # Virtual method, pylint: disable=R0201
+
+        # The value of that node escapes and could change its contents.
         constraint_collection.removeKnowledge(not_node)
 
+        # Any code could be run, note that.
+        constraint_collection.onControlFlowEscape(not_node)
+
         return not_node, None, None
 
     def computeExpressionDrop(self, statement, constraint_collection):
@@ -1041,8 +1074,12 @@ class ExpressionMixin:
 
 
 class CompileTimeConstantExpressionMixin(ExpressionMixin):
+    # TODO: Do this for all computations, do this in the base class of all
+    # nodes.
+    computed_attribute = False
+
     def __init__(self):
-        self.computed_attribute = False
+        pass
 
     def isCompileTimeConstant(self):
         """ Has a value that we can use at compile time.
@@ -1051,16 +1088,23 @@ class CompileTimeConstantExpressionMixin(ExpressionMixin):
             compile time and e.g. operations or conditions, or even calls may
             be executed against it.
         """
-        # Virtual method, pylint: disable=R0201
-
         return True
 
+    def isMutable(self):
+        # Virtual method, pylint: disable=R0201
+        return False
+
     def mayHaveSideEffects(self):
+        # Virtual method, pylint: disable=R0201
         return False
 
     def mayHaveSideEffectsBool(self):
+        # Virtual method, pylint: disable=R0201
         return False
 
+    def mayBeNone(self):
+        return self.getCompileTimeConstant() is None
+
     def computeExpressionOperationNot(self, not_node, constraint_collection):
         from .NodeMakingHelpers import getComputationResult
 
@@ -1068,7 +1112,7 @@ class CompileTimeConstantExpressionMixin(ExpressionMixin):
             node        = not_node,
             computation = lambda : not self.getCompileTimeConstant(),
             description = """\
-Compile time constant negation truth value precomputed."""
+Compile time constant negation truth value pre-computed."""
         )
 
 
@@ -1080,12 +1124,15 @@ Compile time constant negation truth value precomputed."""
 
         from .NodeMakingHelpers import getComputationResult, isCompileTimeConstantValue
 
-        if not hasattr( value, attribute_name ) or isCompileTimeConstantValue( getattr( value, attribute_name ) ):
+        # If it raises, or the attribute itself is a compile time constant,
+        # then do execute it.
+        if not hasattr(value, attribute_name) or \
+           isCompileTimeConstantValue(getattr(value, attribute_name)):
 
             return getComputationResult(
                 node        = lookup_node,
-                computation = lambda : getattr( value, attribute_name ),
-                description = "Attribute lookup to %s precomputed." % (
+                computation = lambda : getattr(value, attribute_name),
+                description = "Attribute lookup to '%s' pre-computed." % (
                     attribute_name
                 )
             )
@@ -1094,7 +1141,6 @@ Compile time constant negation truth value precomputed."""
 
         return lookup_node, None, None
 
-
     def computeExpressionSubscript(self, lookup_node, subscript, constraint_collection):
         from .NodeMakingHelpers import getComputationResult
 
@@ -1105,6 +1151,8 @@ Compile time constant negation truth value precomputed."""
                 description = "Subscript of constant with constant value."
             )
 
+        # TODO: Look-up of subscript to index may happen.
+
         return lookup_node, None, None
 
     def computeExpressionSlice(self, lookup_node, lower, upper, constraint_collection):
@@ -1173,7 +1221,7 @@ class ExpressionSpecBasedComputationMixin(ExpressionMixin):
         return getComputationResult(
             node        = self,
             computation = lambda : self.builtin_spec.simulateCall(given_values),
-            description = "Builtin call to '%s' precomputed." % (
+            description = "Built-in call to '%s' pre-computed." % (
                 self.builtin_spec.getName()
             )
         )
@@ -1194,7 +1242,7 @@ class ExpressionChildrenHavingBase(ChildrenHavingMixin, NodeBase,
 
 class StatementChildrenHavingBase(ChildrenHavingMixin, NodeBase):
     def __init__(self, values, source_ref):
-        NodeBase.__init__( self, source_ref = source_ref )
+        NodeBase.__init__(self, source_ref = source_ref)
 
         ChildrenHavingMixin.__init__(
             self,
@@ -1214,12 +1262,12 @@ class ExpressionBuiltinNoArgBase(NodeBase, ExpressionMixin):
     def computeExpression(self, constraint_collection):
         from .NodeMakingHelpers import getComputationResult
 
-        # The lamba is there for make sure that no argument parsing will reach
-        # the builtin function at all, pylint: disable=W0108
+        # The lambda is there for make sure that no argument parsing will reach
+        # the built-in function at all, pylint: disable=W0108
         return getComputationResult(
             node        = self,
             computation = lambda : self.builtin_function(),
-            description = "No arg %s builtin" % self.builtin_function.__name__
+            description = "No arg %s built-in" % self.builtin_function.__name__
         )
 
 
@@ -1232,7 +1280,7 @@ class ExpressionBuiltinSingleArgBase(ExpressionChildrenHavingBase,
     def __init__(self, value, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
-            values = {
+            values     = {
                 "value" : value,
             },
             source_ref = source_ref
@@ -1254,7 +1302,7 @@ class ExpressionBuiltinSingleArgBase(ExpressionChildrenHavingBase,
         else:
             if value.willRaiseException(BaseException):
                 return value, "new_raise", """\
-Builtin call raises exception while building argument."""
+Built-in call raises exception while building argument."""
 
             return self.computeBuiltinSpec(
                 given_values = (value,)
@@ -1266,8 +1314,7 @@ class SideEffectsFromChildrenMixin:
         for child in self.getVisitableNodes():
             if child.mayHaveSideEffects():
                 return True
-        else:
-            return False
+        return False
 
     def extractSideEffects(self):
         # No side effects at all but from the children.
@@ -1280,3 +1327,10 @@ class SideEffectsFromChildrenMixin:
             )
 
         return tuple(result)
+
+
+# TODO: Maybe this should be in a "Checkers" module
+def checkStatementsSequenceOrNone(value):
+    assert value is None or value.kind == "STATEMENTS_SEQUENCE"
+
+    return value
