@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,6 +23,7 @@ does not matter at all.
 
 from .NodeBases import ExpressionChildrenHavingBase
 
+
 def checkSideEffects(value):
     real_value = []
 
@@ -33,7 +34,7 @@ def checkSideEffects(value):
         else:
             assert child.isExpression()
 
-            real_value.append( child )
+            real_value.append(child)
 
     return tuple(real_value)
 
@@ -41,7 +42,10 @@ def checkSideEffects(value):
 class ExpressionSideEffects(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_SIDE_EFFECTS"
 
-    named_children = ( "side_effects", "expression" )
+    named_children = (
+        "side_effects",
+        "expression"
+    )
 
     checkers = {
         "side_effects" : checkSideEffects
@@ -50,17 +54,17 @@ class ExpressionSideEffects(ExpressionChildrenHavingBase):
     def __init__(self, side_effects, expression, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
-            values = {
-                "side_effects" : tuple( side_effects ),
+            values     = {
+                "side_effects" : tuple(side_effects),
                 "expression"   : expression
             },
             source_ref = source_ref
         )
 
-    getSideEffects  = ExpressionChildrenHavingBase.childGetter( "side_effects" )
+    getSideEffects  = ExpressionChildrenHavingBase.childGetter("side_effects")
     setSideEffects  = ExpressionChildrenHavingBase.childSetter("side_effects")
 
-    getExpression = ExpressionChildrenHavingBase.childGetter( "expression" )
+    getExpression = ExpressionChildrenHavingBase.childGetter("expression")
 
     def computeExpression(self, constraint_collection):
         side_effects = self.getSideEffects()
@@ -68,19 +72,21 @@ class ExpressionSideEffects(ExpressionChildrenHavingBase):
 
         for side_effect in side_effects:
             if side_effect.mayHaveSideEffects():
-                new_side_effects.append( side_effect )
+                new_side_effects.append(side_effect)
 
         expression = self.getExpression()
 
         if expression.isExpressionSideEffects():
-            new_side_effects.extend( expression.getSideEffects() )
+            new_side_effects.extend(
+                expression.getSideEffects()
+            )
 
-            expression.setSideEffects( new_side_effects )
+            expression.setSideEffects(new_side_effects)
 
-            return expression, "new_expression", "Remove nested side effects"
+            return expression, "new_expression", "Remove nested side effects."
 
         if new_side_effects != side_effects:
-            self.setSideEffects( new_side_effects )
+            self.setSideEffects(new_side_effects)
 
         if not new_side_effects:
             return expression, "new_expression", "Removed empty side effects."
@@ -91,8 +97,7 @@ class ExpressionSideEffects(ExpressionChildrenHavingBase):
         for child in self.getVisitableNodes():
             if child.willRaiseException(exception_type):
                 return True
-        else:
-            return False
+        return False
 
     def getTruthValue(self):
         return self.getExpression().getTruthValue()
