@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -30,6 +30,7 @@ from .NodeBases import (
     ExpressionBuiltinSingleArgBase,
     ExpressionChildrenHavingBase
 )
+from .NodeMakingHelpers import getComputationResult
 
 
 class ExpressionBuiltinType1(ExpressionBuiltinSingleArgBase):
@@ -85,7 +86,10 @@ Removed type taking for unused result."""
 class ExpressionBuiltinSuper(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_SUPER"
 
-    named_children = ( "type", "object" )
+    named_children = (
+        "type",
+        "object"
+    )
 
     def __init__(self, super_type, super_object, source_ref):
         ExpressionChildrenHavingBase.__init__(
@@ -97,8 +101,8 @@ class ExpressionBuiltinSuper(ExpressionChildrenHavingBase):
             },
             source_ref = source_ref )
 
-    getType = ExpressionChildrenHavingBase.childGetter( "type" )
-    getObject = ExpressionChildrenHavingBase.childGetter( "object" )
+    getType = ExpressionChildrenHavingBase.childGetter("type")
+    getObject = ExpressionChildrenHavingBase.childGetter("object")
 
     def computeExpression(self, constraint_collection):
         # TODO: Quite some cases should be possible to predict.
@@ -110,27 +114,44 @@ class ExpressionBuiltinIsinstance(ExpressionChildrenHavingBase):
 
     named_children = (
         "instance",
-        "cls"
+        "classes"
     )
 
-    def __init__(self, instance, cls, source_ref):
+    def __init__(self, instance, classes, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
             values     = {
                 "instance" : instance,
-                "cls"      : cls
+                "classes"  : classes
 
             },
-            source_ref = source_ref )
+            source_ref = source_ref
+        )
 
-    getInstance = ExpressionChildrenHavingBase.childGetter( "instance" )
-    getCls = ExpressionChildrenHavingBase.childGetter( "cls" )
+    getInstance = ExpressionChildrenHavingBase.childGetter("instance")
+    getCls = ExpressionChildrenHavingBase.childGetter("classes")
 
     def computeExpression(self, constraint_collection):
         # TODO: Quite some cases should be possible to predict.
-        return self, None, None
 
-    def mayProvideReference(self):
-        # Dedicated code returns "True" or "False" only, which requires no reference,
-        # except for rich comparisons, which do.
-        return False
+        instance = self.getInstance()
+
+        # TODO: Should be possible to query run time type instead, but we don't
+        # have that method yet. Later this will be essential.
+        if not instance.isCompileTimeConstant():
+            return self, None, None
+
+        cls = self.getCls()
+
+        if not cls.isCompileTimeConstant():
+            return self, None, None
+
+        # So if both are compile time constant, we are able to compute it.
+        return getComputationResult(
+            node        = self,
+            computation = lambda : isinstance(
+                instance.getCompileTimeConstant(),
+                cls.getCompileTimeConstant()
+            ),
+            description = "Built-in call to 'isinstance' computed."
+        )
