@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,7 +22,7 @@ and its expressions, changing the meaning of course dramatically.
 
 """
 
-from nuitka import Builtins, Variables
+from nuitka import Builtins, VariableRegistry, Variables
 
 from .ConstantRefNodes import ExpressionConstantRef
 from .NodeBases import ExpressionMixin, NodeBase
@@ -43,7 +43,10 @@ class ExpressionVariableRef(NodeBase, ExpressionMixin):
     kind = "EXPRESSION_VARIABLE_REF"
 
     def __init__(self, variable_name, source_ref, variable = None):
-        NodeBase.__init__( self, source_ref = source_ref )
+        NodeBase.__init__(
+            self,
+            source_ref = source_ref
+        )
 
         self.variable_name = variable_name
         self.variable = variable
@@ -51,17 +54,24 @@ class ExpressionVariableRef(NodeBase, ExpressionMixin):
         if variable is not None:
             assert variable.getName() == variable_name
 
+        self.variable_trace = None
+
     def getDetails(self):
         if self.variable is None:
-            return { "name" : self.variable_name }
+            return {
+                "name" : self.variable_name
+            }
         else:
-            return { "name" : self.variable_name, "variable" : self.variable }
+            return {
+                "name"     : self.variable_name,
+                "variable" : self.variable
+            }
 
     def getDetail(self):
         if self.variable is None:
             return self.variable_name
         else:
-            return repr( self.variable )
+            return repr(self.variable)
 
     def makeCloneAt(self, source_ref):
         result = self.__class__(
@@ -73,7 +83,8 @@ class ExpressionVariableRef(NodeBase, ExpressionMixin):
 
         return result
 
-    def isTargetVariableRef(self):
+    @staticmethod
+    def isTargetVariableRef():
         return False
 
     def getVariableName(self):
@@ -83,18 +94,33 @@ class ExpressionVariableRef(NodeBase, ExpressionMixin):
         return self.variable
 
     def setVariable(self, variable):
-        assert isinstance( variable, Variables.Variable ), repr( variable )
+        assert isinstance(variable, Variables.Variable), repr(variable)
         assert self.variable is None
 
         self.variable = variable
 
     def computeExpression(self, constraint_collection):
-        assert self.variable is not None
+        variable = self.variable
 
-        if _isReadOnlyUnterdeterminedModuleVariable(self.variable):
-            constraint_collection.assumeUnclearLocals(self.source_ref)
+        assert variable is not None
 
-        if _isReadOnlyModuleVariable( self.variable ):
+        self.variable_trace = constraint_collection.getVariableCurrentTrace(
+            variable = variable
+        )
+
+        replacement = self.variable_trace.getReplacementNode(self)
+
+        if replacement is not None:
+            return replacement, "new_expression", "Value propagated for '%s'." % self.variable.getName()
+
+        global_trace = VariableRegistry.getGlobalVariableTrace(variable)
+
+
+        # TODO: Maybe local variables are factored into this strangely.
+        if global_trace is None and variable.isModuleVariable():
+            constraint_collection.assumeUnclearLocals()
+        elif (variable.isModuleVariable() and not global_trace.hasDefiniteWrites() ) or \
+             variable.isMaybeLocalVariable():
             if self.variable_name in Builtins.builtin_exception_names:
                 from .BuiltinRefNodes import ExpressionBuiltinExceptionRef
 
@@ -103,13 +129,13 @@ class ExpressionVariableRef(NodeBase, ExpressionMixin):
                     source_ref     = self.getSourceReference()
                 )
 
-                # TODO: More like "removed_variable and new_constant" probably
-                change_tags = "new_builtin"
+                change_tags = "new_builtin_ref"
                 change_desc = """\
-Module variable '%s' found to be builtin exception reference.""" % (
+Module variable '%s' found to be built-in exception reference.""" % (
                     self.variable_name
                 )
-            elif self.variable_name in Builtins.builtin_names:
+            elif self.variable_name in Builtins.builtin_names and \
+                 self.variable_name != "pow":
                 from .BuiltinRefNodes import ExpressionBuiltinRef
 
                 new_node = ExpressionBuiltinRef(
@@ -117,16 +143,15 @@ Module variable '%s' found to be builtin exception reference.""" % (
                     source_ref   = self.getSourceReference()
                 )
 
-                # TODO: More like "removed_variable and new_constant" probably
-                change_tags = "new_builtin"
+                change_tags = "new_builtin_ref"
                 change_desc = """\
-Module variable '%s' found to be builtin reference.""" % (
+Module variable '%s' found to be built-in reference.""" % (
                     self.variable_name
                 )
             elif self.variable_name == "__name__":
                 new_node = ExpressionConstantRef(
-                    constant   = self.variable.getReferenced().getOwner().\
-                        getFullName(),
+                    constant   = variable.getOwner().getParentModule().\
+                                   getFullName(),
                     source_ref = self.getSourceReference()
                 )
 
@@ -135,8 +160,7 @@ Module variable '%s' found to be builtin reference.""" % (
 Replaced read-only module attribute '__name__' with constant value."""
             elif self.variable_name == "__package__":
                 new_node = ExpressionConstantRef(
-                    constant   = self.variable.getReferenced().getOwner().\
-                                   getPackage(),
+                    constant   = variable.getOwner().getPackage(),
                     source_ref = self.getSourceReference()
                 )
 
@@ -159,81 +183,28 @@ Replaced read-only module attribute '__package__' with constant value."""
     def isKnownToBeIterable(self, count):
         return None
 
-    def mayProvideReference(self):
-        # Variables are capable of "asObject0".
-        return False
-
     def mayHaveSideEffects(self):
-        # TODO: Remembered traced could tell better.
-        return True
+        return not self.variable_trace.mustHaveValue()
 
     def mayRaiseException(self, exception_type):
-        # TODO: Remembered traced could tell better.
-        return True
+        variable_trace = self.variable_trace
 
-
-class ExpressionTargetVariableRef(ExpressionVariableRef):
-    kind = "EXPRESSION_TARGET_VARIABLE_REF"
-
-    # TODO: Remove default and correct argument order later.
-    def __init__(self, variable_name, source_ref, variable = None):
-        ExpressionVariableRef.__init__(self, variable_name, source_ref)
-
-        self.variable_version = None
-
-        # TODO: Remove setVariable, once not needed anymore and inline to
-        # here.
-        if variable is not None:
-            self.setVariable(variable)
-            assert variable.getName() == variable_name
-
-    def getDetails(self):
-        if self.variable is None:
-            return { "name" : self.variable_name }
-        else:
-            return {
-                "name"     : self.variable_name,
-                "variable" : self.variable,
-                "version"  : self.variable_version
-            }
-
-    def makeCloneAt(self, source_ref):
-        result = self.__class__(
-            variable_name = self.variable_name,
-            source_ref    = source_ref,
-            variable      = self.variable
-        )
-
-        return result
-
-    def computeExpression(self, constraint_collection):
-        assert False
-
-    def isTargetVariableRef(self):
-        return True
-
-    def getVariableVersion(self):
-        assert self.variable_version is not None, self
-
-        return self.variable_version
-
-    def setVariable(self, variable):
-        ExpressionVariableRef.setVariable(self, variable)
-
-        self.variable_version = variable.allocateTargetNumber()
-        assert self.variable_version is not None
+        return variable_trace is None or not self.variable_trace.mustHaveValue()
 
 
 class ExpressionTempVariableRef(NodeBase, ExpressionMixin):
     kind = "EXPRESSION_TEMP_VARIABLE_REF"
 
     def __init__(self, variable, source_ref):
-        NodeBase.__init__( self, source_ref = source_ref )
+        NodeBase.__init__(self, source_ref = source_ref)
 
         self.variable = variable
+        self.variable_trace = None
 
     def getDetails(self):
-        return { "name" : self.variable.getName() }
+        return {
+            "name" : self.variable.getName()
+        }
 
     def getDetail(self):
         return self.variable.getName()
@@ -250,17 +221,25 @@ class ExpressionTempVariableRef(NodeBase, ExpressionMixin):
     def getVariable(self):
         return self.variable
 
-    def isTargetVariableRef(self):
+    @staticmethod
+    def isTargetVariableRef():
         return False
 
     def computeExpression(self, constraint_collection):
-        constraint_collection.onVariableUsage( self )
+        self.variable_trace = constraint_collection.getVariableCurrentTrace(
+            variable = self.variable
+        )
+
+        replacement = self.variable_trace.getReplacementNode(self)
+
+        if replacement is not None:
+            return replacement, "new_expression", "Value propagated for temp '%s'." % self.variable.getName()
 
         # Nothing to do here.
         return self, None, None
 
     def onContentEscapes(self, constraint_collection):
-        constraint_collection.onVariableContentEscapes( self.variable )
+        constraint_collection.onVariableContentEscapes(self.variable)
 
     def mayHaveSideEffects(self):
         # Can't happen
@@ -283,18 +262,74 @@ class ExpressionTempVariableRef(NodeBase, ExpressionMixin):
         self.variable = variable
 
 
+class ExpressionTargetVariableRef(ExpressionVariableRef):
+    kind = "EXPRESSION_TARGET_VARIABLE_REF"
+
+    # TODO: Remove default and correct argument order later.
+    def __init__(self, variable_name, source_ref, variable = None):
+        ExpressionVariableRef.__init__(self, variable_name, source_ref)
+
+        self.variable_version = None
+
+        # TODO: Remove setVariable, once not needed anymore and in-line to
+        # here.
+        if variable is not None:
+            self.setVariable(variable)
+            assert variable.getName() == variable_name
+
+    def getDetails(self):
+        if self.variable is None:
+            return {
+                "name" : self.variable_name
+            }
+        else:
+            return {
+                "name"     : self.variable_name,
+                "variable" : self.variable,
+                "version"  : self.variable_version
+            }
+
+    def makeCloneAt(self, source_ref):
+        result = self.__class__(
+            variable_name = self.variable_name,
+            source_ref    = source_ref,
+            variable      = self.variable
+        )
+
+        return result
+
+    def computeExpression(self, constraint_collection):
+        assert False
+
+    @staticmethod
+    def isTargetVariableRef():
+        return True
+
+    def getVariableVersion(self):
+        assert self.variable_version is not None, self
+
+        return self.variable_version
+
+    def setVariable(self, variable):
+        ExpressionVariableRef.setVariable(self, variable)
+
+        self.variable_version = variable.allocateTargetNumber()
+        assert self.variable_version is not None
+
+
 class ExpressionTargetTempVariableRef(ExpressionTempVariableRef):
     kind = "EXPRESSION_TARGET_TEMP_VARIABLE_REF"
 
     def __init__(self, variable, source_ref):
-        ExpressionTempVariableRef.__init__( self, variable, source_ref )
+        ExpressionTempVariableRef.__init__(self, variable, source_ref)
 
         self.variable_version = variable.allocateTargetNumber()
 
     def computeExpression(self, constraint_collection):
         assert False, self.parent
 
-    def isTargetVariableRef(self):
+    @staticmethod
+    def isTargetVariableRef():
         return True
 
     def getVariableVersion(self):
@@ -302,6 +337,6 @@ class ExpressionTargetTempVariableRef(ExpressionTempVariableRef):
 
     # Python3 only, it updates temporary variables that are closure variables.
     def setVariable(self, variable):
-        ExpressionTempVariableRef.setVariable( self, variable )
+        ExpressionTempVariableRef.setVariable(self, variable)
 
         self.variable_version = self.variable.allocateTargetNumber()
