@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -15,7 +15,7 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-""" Optimizations of builtins to builtin calls.
+""" Optimizations of built-ins to built-in calls.
 
 """
 import math
@@ -26,7 +26,7 @@ from nuitka.nodes.ParameterSpecs import (
     TooManyArguments,
     matchCall
 )
-from nuitka.Utils import python_version
+from nuitka.utils.Utils import python_version
 
 
 class BuiltinParameterSpec(ParameterSpec):
@@ -34,15 +34,15 @@ class BuiltinParameterSpec(ParameterSpec):
                   dict_star_arg = None):
         ParameterSpec.__init__(
             self,
-            name           = name,
-            normal_args    = arg_names,
-            list_star_arg  = list_star_arg,
-            dict_star_arg  = dict_star_arg,
-            default_count  = default_count,
-            kw_only_args   = ()
+            name          = name,
+            normal_args   = arg_names,
+            list_star_arg = list_star_arg,
+            dict_star_arg = dict_star_arg,
+            default_count = default_count,
+            kw_only_args  = ()
         )
 
-        self.builtin = __builtins__[ name ]
+        self.builtin = __builtins__[name]
 
     def __repr__(self):
         return "<BuiltinParameterSpec %s>" % self.name
@@ -51,15 +51,18 @@ class BuiltinParameterSpec(ParameterSpec):
         return self.name
 
     def isCompileTimeComputable(self, values):
+        # By default, we make this dependent on the ability to compute the
+        # arguments, which is of course a good start for most cases, so this
+        # is for overloads, pylint: disable=R0201
+
         for value in values:
             if value is not None and not value.isCompileTimeConstant():
                 return False
-        else:
-            return True
+        return True
 
     def simulateCall(self, given_values):
         # Using star dict call for simulation and catch any exception as really
-        # fatal, pylint: disable=W0142,W0703
+        # fatal, pylint: disable=W0703
 
         try:
             given_normal_args = given_values[:len(self.normal_args)]
@@ -109,7 +112,7 @@ class BuiltinParameterSpecNoKeywords(BuiltinParameterSpec):
 
     def simulateCall(self, given_values):
         # Using star dict call for simulation and catch any exception as really fatal,
-        # pylint: disable=W0142,W0703
+        # pylint: disable=W0703
 
         try:
             if self.list_star_arg:
@@ -135,6 +138,8 @@ class BuiltinParameterSpecNoKeywords(BuiltinParameterSpec):
                 arg_list += [ value.getCompileTimeConstant() for value in given_list_star_arg ]
         except Exception as e:
             print >> sys.stderr, "Fatal error: ",
+            import traceback
+            traceback.print_exc()
             sys.exit(repr(e))
 
         return self.builtin(*arg_list)
@@ -172,13 +177,13 @@ def makeBuiltinParameterSpec(exception_name):
         default_count  = 0
     )
 
-builtin_int_spec = BuiltinParameterSpec("int", ("x", "base"), 2)
+builtin_int_spec = BuiltinParameterSpec("int", ('x', "base"), 2)
 
 # These builtins are only available for Python2
 if python_version < 300:
     builtin_long_spec = BuiltinParameterSpec(
         "long",
-        ("x", "base"),
+        ('x', "base"),
         2
     )
     builtin_execfile_spec = BuiltinParameterSpecNoKeywords(
@@ -198,10 +203,11 @@ if python_version < 300:
     )
 
 
-builtin_bool_spec = BuiltinParameterSpec("bool", ("x",), 1)
-builtin_float_spec = BuiltinParameterSpec("float", ("x",), 1)
+builtin_bool_spec = BuiltinParameterSpec("bool", ('x',), 1)
+builtin_float_spec = BuiltinParameterSpec("float", ('x',), 1)
+builtin_complex_spec = BuiltinParameterSpec("complex", ("real", "imag"), 2)
 
-# This builtin have variable parameters for Python2/3
+# This built-in have variable parameters for Python2/3
 if python_version < 300:
     builtin_str_spec = BuiltinParameterSpec("str", ("object",), 1)
 else:
@@ -216,11 +222,12 @@ builtin_set_spec = BuiltinParameterSpecNoKeywords("set", ("iterable",), 1)
 
 builtin_import_spec = BuiltinParameterSpec("__import__", ("name", "globals", "locals", "fromlist", "level"), 4)
 builtin_open_spec = BuiltinParameterSpec("open", ("name", "mode", "buffering"), 3)
-builtin_chr_spec = BuiltinParameterSpecNoKeywords("chr", ("i",), 0)
-builtin_ord_spec = BuiltinParameterSpecNoKeywords("ord", ("c",), 0)
+builtin_chr_spec = BuiltinParameterSpecNoKeywords("chr", ('i',), 0)
+builtin_ord_spec = BuiltinParameterSpecNoKeywords("ord", ('c',), 0)
 builtin_bin_spec = BuiltinParameterSpecNoKeywords("bin", ("number",), 0)
 builtin_oct_spec = BuiltinParameterSpecNoKeywords("oct", ("number",), 0)
 builtin_hex_spec = BuiltinParameterSpecNoKeywords("hex", ("number",), 0)
+builtin_id_spec = BuiltinParameterSpecNoKeywords("id", ("object",), 0)
 builtin_repr_spec = BuiltinParameterSpecNoKeywords("repr", ("object",), 0)
 
 builtin_dir_spec = BuiltinParameterSpecNoKeywords("dir", ("object",), 1)
@@ -263,14 +270,21 @@ builtin_hasattr_spec = BuiltinParameterSpecNoKeywords("hasattr", ("object", "nam
 builtin_getattr_spec = BuiltinParameterSpecNoKeywords("getattr", ("object", "name", "default"), 1)
 builtin_setattr_spec = BuiltinParameterSpecNoKeywords("setattr", ("object", "name", "value"), 0)
 
-builtin_isinstance_spec = BuiltinParameterSpecNoKeywords("isinstance", ("instance", "cls"), 0)
+builtin_isinstance_spec = BuiltinParameterSpecNoKeywords("isinstance", ("instance", "classes"), 0)
 
+builtin_bytearray_spec = BuiltinParameterSpecNoKeywords("bytearray", ("iterable_of_ints",), 1)
+
+# Beware: One argument defines stop, not start.
+builtin_slice_spec = BuiltinParameterSpecNoKeywords("slice", ("start", "stop", "step"), 2)
 
 class BuiltinRangeSpec(BuiltinParameterSpecNoKeywords):
     def __init__(self, *args):
         BuiltinParameterSpecNoKeywords.__init__(self, *args)
 
     def isCompileTimeComputable(self, values):
+        # For ranges, we need have many cases that can prevent the ability
+        # to pre-compute, pylint: disable=R0911,R0912
+
         result = BuiltinParameterSpecNoKeywords.isCompileTimeComputable(
             self,
             values = values
@@ -339,22 +353,28 @@ def extractBuiltinArgs(node, builtin_spec, builtin_class,
 
         # TODO: Could check for too many / too few, even if they are unknown, we
         # might raise that error, but that need not be optimized immediately.
-        if not kw.isMappingWithConstantStringKeys():
-            return None
+        if kw is not None:
+            if not kw.isMappingWithConstantStringKeys():
+                return None
 
-        pairs = kw.getMappingStringKeyPairs()
+            pairs = kw.getMappingStringKeyPairs()
 
-        if pairs and not builtin_spec.allowsKeywords():
-            raise TooManyArguments(
-                TypeError(builtin_spec.getKeywordRefusalText())
-            )
+            if pairs and not builtin_spec.allowsKeywords():
+                raise TooManyArguments(
+                    TypeError(builtin_spec.getKeywordRefusalText())
+                )
+        else:
+            pairs = ()
 
         args = node.getCallArgs()
 
-        if not args.canPredictIterationValues():
-            return None
+        if args:
+            if not args.canPredictIterationValues():
+                return None
 
-        positional = args.getIterationValues()
+            positional = args.getIterationValues()
+        else:
+            positional = ()
 
         if not positional and not pairs and empty_special_class is not None:
             return empty_special_class(source_ref = node.getSourceReference())
@@ -376,8 +396,8 @@ def extractBuiltinArgs(node, builtin_spec, builtin_class,
 
         return wrapExpressionWithSideEffects(
             new_node     = makeRaiseExceptionReplacementExpressionFromInstance(
-                expression     = node,
-                exception      = e.getRealException()
+                expression = node,
+                exception  = e.getRealException()
             ),
             old_node     = node,
             side_effects = node.extractPreCallSideEffects()
@@ -395,8 +415,10 @@ def extractBuiltinArgs(node, builtin_spec, builtin_class,
         args_list.append(args_dict[builtin_spec.getStarDictArgumentName()])
 
     # Using list reference for passing the arguments without names,
-    # pylint: disable=W0142
-    return builtin_class(
+    result = builtin_class(
         *args_list,
         source_ref = node.getSourceReference()
     )
+    result.setCompatibleSourceReference(node.getCompatibleSourceReference())
+
+    return result
