@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -15,12 +15,15 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-""" Optimize calls to builtins reference builtin nodes.
+""" Optimize calls to built-in references to specific built-in calls.
 
-For builtin name references, we check if it's one of the supported builtin
-types.
+For built-in name references, we check if it's one of the supported built-in
+types, and then specialize for the ones, where it makes sense.
 """
 
+from logging import warning
+
+from nuitka.Builtins import calledWithBuiltinArgumentNamesDecorator
 from nuitka.nodes.AssignNodes import (
     StatementAssignmentVariable,
     StatementDelVariable
@@ -40,6 +43,7 @@ from nuitka.nodes.BuiltinDictNodes import ExpressionBuiltinDict
 from nuitka.nodes.BuiltinFormatNodes import (
     ExpressionBuiltinBin,
     ExpressionBuiltinHex,
+    ExpressionBuiltinId,
     ExpressionBuiltinOct
 )
 from nuitka.nodes.BuiltinIteratorNodes import (
@@ -63,10 +67,13 @@ from nuitka.nodes.BuiltinRefNodes import (
 )
 from nuitka.nodes.BuiltinTypeNodes import (
     ExpressionBuiltinBool,
+    ExpressionBuiltinBytearray,
+    ExpressionBuiltinComplex,
     ExpressionBuiltinFloat,
     ExpressionBuiltinInt,
     ExpressionBuiltinList,
     ExpressionBuiltinSet,
+    ExpressionBuiltinSlice,
     ExpressionBuiltinStr,
     ExpressionBuiltinTuple
 )
@@ -94,7 +101,7 @@ from nuitka.nodes.OperatorNodes import (
     ExpressionOperationUnary
 )
 from nuitka.nodes.StatementNodes import StatementsSequence
-from nuitka.nodes.TryNodes import ExpressionTryFinally
+from nuitka.nodes.TryFinallyNodes import ExpressionTryFinally
 from nuitka.nodes.TypeNodes import (
     ExpressionBuiltinIsinstance,
     ExpressionBuiltinSuper,
@@ -107,7 +114,8 @@ from nuitka.nodes.VariableRefNodes import (
 )
 from nuitka.Options import isDebug, shallMakeModule
 from nuitka.tree.ReformulationExecStatements import wrapEvalGlobalsAndLocals
-from nuitka.Utils import python_version
+from nuitka.utils.Utils import python_version
+from nuitka.VariableRegistry import addVariableUsage
 
 from . import BuiltinOptimization
 
@@ -124,12 +132,12 @@ def dir_extractor(node):
             )
 
         result = ExpressionCallEmpty(
-            called = ExpressionAttributeLookup(
-                expression     = source,
+            called     = ExpressionAttributeLookup(
+                source         = source,
                 attribute_name = "keys",
                 source_ref     = source_ref
             ),
-            source_ref     = source_ref
+            source_ref = source_ref
         )
 
         # For Python3, keys doesn't really return values, but instead a handle
@@ -162,7 +170,7 @@ def vars_extractor(node):
             )
 
     return BuiltinOptimization.extractBuiltinArgs(
-        node          = node,
+        node                = node,
         builtin_class       = ExpressionBuiltinVars,
         builtin_spec        = BuiltinOptimization.builtin_vars_spec,
         empty_special_class = selectVarsEmptyClass
@@ -194,19 +202,16 @@ def type_extractor(node):
         )
 
 def iter_extractor(node):
-    # Note: Iter in fact names its first argument if the default applies
-    # "collection", but it won't matter much, fixed up in a wrapper.  The
-    # "callable" is part of the API, pylint: disable=W0622
-
-    def wrapIterCreation(callable, sentinel, source_ref):
+    @calledWithBuiltinArgumentNamesDecorator
+    def wrapIterCreation(callable_arg, sentinel, source_ref):
         if sentinel is None:
             return ExpressionBuiltinIter1(
-                value      = callable,
+                value      = callable_arg,
                 source_ref = source_ref
             )
         else:
             return ExpressionBuiltinIter2(
-                callable   = callable,
+                callable   = callable_arg,
                 sentinel   = sentinel,
                 source_ref = source_ref
             )
@@ -242,9 +247,8 @@ def next_extractor(node):
 
 
 def dict_extractor(node):
-    # The dict is a bit strange in that it accepts a position parameter, or not,
-    # but won't have a default.
-
+    # The "dict" built-in is a bit strange in that it accepts a position
+    # parameter, or not, but won't have a default value.
     def wrapExpressionBuiltinDictCreation(positional_args, dict_star_arg,
                                           source_ref):
         if len(positional_args) > 1:
@@ -254,8 +258,8 @@ def dict_extractor(node):
             )
 
             result = makeRaiseExceptionReplacementExpressionFromInstance(
-                expression     = node,
-                exception      = TypeError(
+                expression = node,
+                exception  = TypeError(
                     "dict expected at most 1 arguments, got %d" % (
                         len(positional_args)
                     )
@@ -325,6 +329,13 @@ def hex_extractor(node):
         builtin_spec  = BuiltinOptimization.builtin_hex_spec
     )
 
+def id_extractor(node):
+    return BuiltinOptimization.extractBuiltinArgs(
+        node          = node,
+        builtin_class = ExpressionBuiltinId,
+        builtin_spec  = BuiltinOptimization.builtin_id_spec
+    )
+
 def repr_extractor(node):
     def makeReprOperator(operand, source_ref):
         return ExpressionOperationUnary(
@@ -342,11 +353,24 @@ def repr_extractor(node):
 def range_extractor(node):
     def selectRangeBuiltin(low, high, step, source_ref):
         if high is None:
-            return ExpressionBuiltinRange1( low, source_ref )
+            return ExpressionBuiltinRange1(
+                low        = low,
+                source_ref = source_ref
+            )
         elif step is None:
-            return ExpressionBuiltinRange2(low, high, source_ref)
+            return ExpressionBuiltinRange2(
+                low        = low,
+                high       = high,
+                source_ref = source_ref
+            )
         else:
-            return ExpressionBuiltinRange3(low, high, step, source_ref)
+            return ExpressionBuiltinRange3(
+                low        = low,
+                high       = high,
+                step       = step,
+                source_ref = source_ref
+            )
+
 
     return BuiltinOptimization.extractBuiltinArgs(
         node                = node,
@@ -401,6 +425,13 @@ def float_extractor(node):
         builtin_spec  = BuiltinOptimization.builtin_float_spec
     )
 
+def complex_extractor(node):
+    return BuiltinOptimization.extractBuiltinArgs(
+        node          = node,
+        builtin_class = ExpressionBuiltinComplex,
+        builtin_spec  = BuiltinOptimization.builtin_complex_spec
+    )
+
 def str_extractor(node):
     return BuiltinOptimization.extractBuiltinArgs(
         node          = node,
@@ -471,26 +502,17 @@ if python_version < 300:
     from nuitka.nodes.ExecEvalNodes import ExpressionBuiltinExecfile
 
     def execfile_extractor(node):
-        # Need to accept globals and local keyword argument, that is just the
-        # API of execfile, pylint: disable=W0622
-
-        def wrapExpressionBuiltinExecfileCreation(filename, globals_node,
-                                                  locals_node, source_ref):
+        @calledWithBuiltinArgumentNamesDecorator
+        def wrapExpressionBuiltinExecfileCreation(filename, globals_arg,
+                                                  locals_arg, source_ref):
             provider = node.getParentVariableProvider()
 
-            # TODO: Can't really be true, can it?
-            if provider.isExpressionFunctionBody():
-                provider.markAsExecContaining()
-
-                if provider.isClassDictCreation():
-                    provider.markAsUnqualifiedExecContaining( source_ref )
-
             temp_scope = provider.allocateTempScope("execfile")
 
             globals_ref, locals_ref, tried, final = wrapEvalGlobalsAndLocals(
                 provider     = provider,
-                globals_node = globals_node,
-                locals_node  = locals_node,
+                globals_node = globals_arg,
+                locals_node  = locals_arg,
                 temp_scope   = temp_scope,
                 source_ref   = source_ref
             )
@@ -501,7 +523,7 @@ if python_version < 300:
                 expression = ExpressionBuiltinExecfile(
                     source_code = ExpressionCallEmpty(
                         called     = ExpressionAttributeLookup(
-                            expression     = ExpressionBuiltinOpen(
+                            source         = ExpressionBuiltinOpen(
                                 filename   = filename,
                                 mode       = ExpressionConstantRef(
                                     constant   = "rU",
@@ -529,27 +551,27 @@ if python_version < 300:
         )
 
 def eval_extractor(node):
-    # Need to accept globals and local keyword argument, that is just the API of
-    # eval, pylint: disable=W0622
-
-    def wrapEvalBuiltin(source, globals, locals, source_ref):
+    @calledWithBuiltinArgumentNamesDecorator
+    def wrapEvalBuiltin(source, globals_arg, locals_arg, source_ref):
         provider = node.getParentVariableProvider()
 
         temp_scope = provider.allocateTempScope("eval")
 
         globals_ref, locals_ref, tried, final = wrapEvalGlobalsAndLocals(
             provider     = provider,
-            globals_node = globals,
-            locals_node  = locals,
+            globals_node = globals_arg,
+            locals_node  = locals_arg,
             temp_scope   = temp_scope,
             source_ref   = source_ref
         )
 
         # The wrapping should not relocate to the "source_ref".
-        assert globals is None or \
-               globals_ref.getSourceReference() == globals.getSourceReference()
-        assert locals is None or \
-               locals_ref.getSourceReference() == locals.getSourceReference()
+        assert globals_arg is None or \
+               globals_ref.getSourceReference() == \
+               globals_arg.getSourceReference()
+        assert locals_arg is None or \
+               locals_ref.getSourceReference() == \
+               locals_arg.getSourceReference()
 
         source_variable = provider.allocateTempVariable(
             temp_scope = temp_scope,
@@ -570,13 +592,13 @@ def eval_extractor(node):
         )
 
         strip_choice =  ExpressionConstantRef(
-            constant = (" \t",),
+            constant   = (" \t",),
             source_ref = source_ref
         )
 
         if python_version >= 300:
             strip_choice = ExpressionConditional(
-                condition = ExpressionComparisonIs(
+                condition      = ExpressionComparisonIs(
                     left       = ExpressionBuiltinType1(
                         value      = ExpressionTempVariableRef(
                             variable   = source_variable,
@@ -591,7 +613,7 @@ def eval_extractor(node):
                     source_ref = source_ref
                 ),
                 yes_expression = ExpressionConstantRef(
-                    constant = (b" \t",),
+                    constant   = (b" \t",),
                     source_ref = source_ref
                 ),
                 no_expression  = strip_choice,
@@ -607,19 +629,19 @@ def eval_extractor(node):
                     variable   = source_variable,
                     source_ref = source_ref
                 ),
-                source = ExpressionCallNoKeywords(
-                    called = ExpressionAttributeLookup(
-                        expression     = ExpressionTempVariableRef(
+                source       = ExpressionCallNoKeywords(
+                    called     = ExpressionAttributeLookup(
+                        source         = ExpressionTempVariableRef(
                             variable   = source_variable,
                             source_ref = source_ref
                         ),
                         attribute_name = "strip",
                         source_ref     = source_ref
                     ),
-                    args         = strip_choice,
-                    source_ref   = source_ref
+                    args       = strip_choice,
+                    source_ref = source_ref
                 ),
-                source_ref = source_ref
+                source_ref   = source_ref
             )
         ]
 
@@ -633,16 +655,16 @@ def eval_extractor(node):
                 source_ref   = source_ref,
             ),
             StatementConditional(
-                condition = ExpressionOperationNOT(
+                condition  = ExpressionOperationNOT(
                     operand    = ExpressionBuiltinIsinstance(
-                        cls = ExpressionBuiltinAnonymousRef(
-                            builtin_name = "code",
-                            source_ref   = source_ref,
-                        ),
-                        instance = ExpressionTempVariableRef(
+                        instance   = ExpressionTempVariableRef(
                             variable   = source_variable,
                             source_ref = source_ref
                         ),
+                        classes    = ExpressionBuiltinAnonymousRef(
+                            builtin_name = "code",
+                            source_ref   = source_ref,
+                        ),
                         source_ref = source_ref
                     ),
                     source_ref = source_ref
@@ -685,10 +707,8 @@ if python_version >= 300:
     from nuitka.nodes.ExecEvalNodes import ExpressionBuiltinExec
 
     def exec_extractor(node):
-        # Need to accept globals and local keyword argument, that is just the
-        # API of exec, pylint: disable=W0622
-
-        def wrapExpressionBuiltinExecCreation(source, globals, locals,
+        @calledWithBuiltinArgumentNamesDecorator
+        def wrapExpressionBuiltinExecCreation(source, globals_arg, locals_arg,
                                               source_ref):
             provider = node.getParentVariableProvider()
 
@@ -697,14 +717,14 @@ if python_version >= 300:
                 provider.markAsExecContaining()
 
                 if provider.isClassDictCreation():
-                    provider.markAsUnqualifiedExecContaining( source_ref )
+                    provider.markAsUnqualifiedExecContaining(source_ref)
 
             temp_scope = provider.allocateTempScope("exec")
 
             globals_ref, locals_ref, tried, final = wrapEvalGlobalsAndLocals(
                 provider     = provider,
-                globals_node = globals,
-                locals_node  = locals,
+                globals_node = globals_arg,
+                locals_node  = locals_arg,
                 temp_scope   = temp_scope,
                 source_ref   = source_ref
             )
@@ -718,7 +738,7 @@ if python_version >= 300:
                     locals_arg  = locals_ref,
                     source_ref  = source_ref
                 ),
-                source_ref  = source_ref
+                source_ref = source_ref
             )
 
         return BuiltinOptimization.extractBuiltinArgs(
@@ -729,16 +749,16 @@ if python_version >= 300:
 
 def compile_extractor(node):
     def wrapExpressionBuiltinCompileCreation(source_code, filename, mode, flags,
-                                              dont_inherit, optimize = None,
-                                              source_ref = None):
+                                             dont_inherit, optimize = None,
+                                             source_ref = None):
         return ExpressionBuiltinCompile(
-            source_code,
-            filename,
-            mode,
-            flags,
-            dont_inherit,
-            optimize,
-            source_ref
+            source_code  = source_code,
+            filename     = filename,
+            mode         = mode,
+            flags        = flags,
+            dont_inherit = dont_inherit,
+            optimize     = optimize,
+            source_ref   = source_ref
         )
 
     return BuiltinOptimization.extractBuiltinArgs(
@@ -756,20 +776,19 @@ def open_extractor(node):
     )
 
 def super_extractor(node):
-    # Need to accept type and object as keyword argument, that is just the API
-    # of super, pylint: disable=W0622
-    def wrapSuperBuiltin(type, object, source_ref):
-        if type is None and python_version >= 300:
+    @calledWithBuiltinArgumentNamesDecorator
+    def wrapSuperBuiltin(type_arg, object_arg, source_ref):
+        if type_arg is None and python_version >= 300:
             provider = node.getParentVariableProvider()
 
             if python_version < 340 or True: # TODO: Temporarily reverted:
-                type = ExpressionVariableRef(
+                type_arg = ExpressionVariableRef(
                     variable_name = "__class__",
                     source_ref    = source_ref
                 )
 
                 # Ought to be already closure taken.
-                type.setVariable(
+                type_arg.setVariable(
                     provider.getVariableForClosure(
                         variable_name = "__class__"
                     )
@@ -777,11 +796,10 @@ def super_extractor(node):
 
                 # If we already have this as a local variable, then use that
                 # instead.
-                if type.getVariable().getOwner() is provider:
-                    type = None
+                if type_arg.getVariable().getOwner() is provider:
+                    type_arg = None
                 else:
-                    from nuitka.VariableRegistry import addVariableUsage
-                    addVariableUsage(type.getVariable(), provider)
+                    addVariableUsage(type_arg.getVariable(), provider)
             else:
                 parent_provider = provider.getParentVariableProvider()
 
@@ -790,18 +808,16 @@ def super_extractor(node):
                     name       = "__class__"
                 )
 
-                type = ExpressionTempVariableRef(
-                    variable      = class_var,
-                    source_ref    = source_ref
+                type_arg = ExpressionTempVariableRef(
+                    variable   = class_var,
+                    source_ref = source_ref
                 )
-
-                from nuitka.VariableRegistry import addVariableUsage
-                addVariableUsage(type.getVariable(), provider)
+                addVariableUsage(type_arg.getVariable(), provider)
 
             from nuitka.nodes.NodeMakingHelpers import \
                 makeRaiseExceptionReplacementExpression
 
-            if type is None:
+            if type_arg is None:
                 return makeRaiseExceptionReplacementExpression(
                     expression      = node,
                     exception_type  = "SystemError"
@@ -810,24 +826,24 @@ def super_extractor(node):
                     exception_value = "super(): __class__ cell not found",
                 )
 
-            if object is None:
+            if object_arg is None:
                 if provider.getParameters().getArgumentCount() > 0:
                     par1_name = provider.getParameters().getArgumentNames()[0]
                     # TODO: Nested first argument would kill us here, need a
                     # test for that.
 
-                    object = ExpressionVariableRef(
+                    object_arg = ExpressionVariableRef(
                         variable_name = par1_name,
                         source_ref    = source_ref
                     )
 
-                    object.setVariable(
+                    object_arg.setVariable(
                         provider.getVariableForReference(
                             variable_name = par1_name
                         )
                     )
 
-                    if not object.getVariable().isParameterVariable():
+                    if not object_arg.getVariable().isParameterVariable():
                         return makeRaiseExceptionReplacementExpression(
                             expression      = node,
                             exception_type  = "SystemError"
@@ -843,8 +859,8 @@ def super_extractor(node):
                     )
 
         return ExpressionBuiltinSuper(
-            super_type   = type,
-            super_object = object,
+            super_type   = type_arg,
+            super_object = object_arg,
             source_ref   = source_ref
         )
 
@@ -882,6 +898,36 @@ def isinstance_extractor(node):
         builtin_spec  = BuiltinOptimization.builtin_isinstance_spec
     )
 
+def bytearray_extractor(node):
+    return BuiltinOptimization.extractBuiltinArgs(
+        node          = node,
+        builtin_class = ExpressionBuiltinBytearray,
+        builtin_spec  = BuiltinOptimization.builtin_bytearray_spec
+    )
+
+
+def slice_extractor(node):
+    def wrapSlice(start, stop, step, source_ref):
+        if start is not None and stop is None:
+            # Default rules are strange. If one argument is given, it's the
+            # second one then.
+            stop = start
+            start = None
+
+        return ExpressionBuiltinSlice(
+            start      = start,
+            stop       = stop,
+            step       = step,
+            source_ref = source_ref
+        )
+
+    return BuiltinOptimization.extractBuiltinArgs(
+        node          = node,
+        builtin_class = wrapSlice,
+        builtin_spec  = BuiltinOptimization.builtin_slice_spec
+    )
+
+
 _dispatch_dict = {
     "compile"    : compile_extractor,
     "globals"    : globals_extractor,
@@ -895,6 +941,7 @@ _dispatch_dict = {
     "bin"        : bin_extractor,
     "oct"        : oct_extractor,
     "hex"        : hex_extractor,
+    "id"         : id_extractor,
     "type"       : type_extractor,
     "iter"       : iter_extractor,
     "next"       : next_extractor,
@@ -904,6 +951,7 @@ _dispatch_dict = {
     "dict"       : dict_extractor,
     "set"        : set_extractor,
     "float"      : float_extractor,
+    "complex"    : complex_extractor,
     "str"        : str_extractor,
     "bool"       : bool_extractor,
     "int"        : int_extractor,
@@ -913,7 +961,10 @@ _dispatch_dict = {
     "hasattr"    : hasattr_extractor,
     "getattr"    : getattr_extractor,
     "setattr"    : setattr_extractor,
-    "isinstance" : isinstance_extractor
+    "isinstance" : isinstance_extractor,
+    # TODO: Disabled for now, not handling all cases.
+    # "bytearray"  : bytearray_extractor,
+    "slice"      : slice_extractor
 }
 
 if python_version < 300:
@@ -934,7 +985,20 @@ def check():
 
 check()
 
+_builtin_white_list = (
+    # Not supporting 'print', because it could be replaced, and is not
+    # worth the effort yet.
+    "print",
+
+    # TODO: This could, and should be supported, as we could e.g. lower
+    # types easily for it.
+    "sorted",
+)
+
 def computeBuiltinCall(call_node, called):
+    # There is some dispatching for how to output various types of changes,
+    # with lots of cases, pylint: disable=R0912
+
     builtin_name = called.getBuiltinName()
 
     if builtin_name in _dispatch_dict:
@@ -961,38 +1025,38 @@ Replaced dynamic __import__ %s with static module import.""" % (
         elif inspect_node.isExpressionBuiltin() or \
              inspect_node.isStatementExec():
             tags = "new_builtin"
-            message = "Replaced call to builtin %s with builtin call %s." % (
+            message = "Replaced call to built-in '%s' with built-in call '%s'." % (
                 builtin_name,
                 inspect_node.kind,
             )
         elif inspect_node.isExpressionRaiseException():
             tags = "new_raise"
             message = """\
-Replaced call to builtin %s with exception raising call.""" % (
+Replaced call to built-in '%s' with exception raising call.""" % (
                 inspect_node.kind,
             )
         elif inspect_node.isExpressionOperationUnary():
             tags = "new_expression"
             message = """\
-Replaced call to builtin %s with unary operation %s.""" % (
+Replaced call to built-in '%s' with unary operation '%s'.""" % (
                 inspect_node.kind,
                 inspect_node.getOperator()
             )
         elif inspect_node.isExpressionCall():
             tags = "new_expression"
             message = """\
-Replaced call to builtin %s with call.""" % (
+Replaced call to built-in '%s' with call.""" % (
                 inspect_node.kind,
             )
         elif inspect_node.isExpressionTryFinally():
             tags = "new_expression"
             message = """\
-Replaced call to builtin %s with try/finally guarded call.""" % (
+Replaced call to built-in '%s' with try/finally guarded call.""" % (
                 inspect_node.getExpression().kind,
             )
         else:
 
-            assert False, ( builtin_name, "->", inspect_node )
+            assert False, (builtin_name, "->", inspect_node)
 
         # TODO: One day, this should be enabled by default and call either the
         # original built-in or the optimized above one. That should be done,
@@ -1005,20 +1069,20 @@ Replaced call to builtin %s with try/finally guarded call.""" % (
 
             new_node = ExpressionConditional(
                 condition      = ExpressionComparisonIs(
-                    left  = ExpressionBuiltinRef(
+                    left       = ExpressionBuiltinRef(
                         builtin_name = builtin_name,
                         source_ref   = source_ref
                     ),
-                    right = ExpressionBuiltinOriginalRef(
+                    right      = ExpressionBuiltinOriginalRef(
                         builtin_name = builtin_name,
                         source_ref   = source_ref
                     ),
-                    source_ref   = source_ref
+                    source_ref = source_ref
                 ),
                 yes_expression = new_node,
                 no_expression  = makeRaiseExceptionReplacementExpression(
                     exception_type  = "RuntimeError",
-                    exception_value = "Builtin '%s' was overloaded'" % (
+                    exception_value = "Built-in '%s' cannot be replaced." % (
                         builtin_name
                     ),
                     expression      = call_node
@@ -1030,5 +1094,9 @@ Replaced call to builtin %s with try/finally guarded call.""" % (
 
         return new_node, tags, message
     else:
-        # TODO: Consider giving warnings, whitelisted potentially
+        if False and isDebug() and builtin_name not in _builtin_white_list:
+            warning(
+                "Not handling built-in '%s', consider support." % builtin_name
+            )
+
         return call_node, None, None
