@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,13 +20,13 @@
 Does all the Python parsing and puts it into a tree structure for use in later
 stages of the compilation process.
 
-In the nuitka.tree.Helpers module, the dispatching is happening. One function
+In the "nuitka.tree.Helpers" module, the dispatching is happening. One function
 deals with every node kind as found in the AST. The parsing is centered around
 the module "ast" output.
 
 Many higher level language features and translated into lower level ones.
 
-Inplace assignments, for loops, while loops, classes, complex calls, with
+In-place assignments, for loops, while loops, classes, complex calls, with
 statements, and even or/and etc. are all translated to simpler constructs.
 
 The output of this module is a node tree, which contains only relatively low
@@ -53,16 +53,14 @@ catching and passing in exceptions raised.
 import ast
 import sys
 
-from nuitka import (
-    Importing,
-    Options,
-    SourceCodeReferences,
-    SyntaxErrors,
-    Tracing,
-    Utils
-)
+from nuitka import Options, PythonVersions, SourceCodeReferences, Tracing
 from nuitka.__past__ import long, unicode  # pylint: disable=W0622
-from nuitka.nodes.AssignNodes import StatementAssignmentVariable
+from nuitka.importing import Importing
+from nuitka.importing.ImportCache import addImportedModule
+from nuitka.nodes.AssignNodes import (
+    StatementAssignmentVariable,
+    StatementReleaseVariable
+)
 from nuitka.nodes.AttributeNodes import ExpressionAttributeLookup
 from nuitka.nodes.ConditionalNodes import (
     ExpressionConditional,
@@ -70,14 +68,13 @@ from nuitka.nodes.ConditionalNodes import (
 )
 from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
 from nuitka.nodes.ExceptionNodes import StatementRaiseException
-from nuitka.nodes.FutureSpecs import FutureSpec
 from nuitka.nodes.ImportNodes import (
     ExpressionImportModule,
-    ExpressionImportName,
-    StatementImportStar
+    ExpressionImportName
 )
 from nuitka.nodes.LoopNodes import StatementBreakLoop, StatementContinueLoop
 from nuitka.nodes.ModuleNodes import (
+    ExpressionModuleFileAttributeRef,
     PythonMainModule,
     PythonModule,
     PythonPackage,
@@ -88,16 +85,18 @@ from nuitka.nodes.OperatorNodes import (
     ExpressionOperationUnary
 )
 from nuitka.nodes.ReturnNodes import StatementReturn
-from nuitka.nodes.SliceNodes import ExpressionSliceLookup, ExpressionSliceObject
 from nuitka.nodes.StatementNodes import (
     StatementExpressionOnly,
     StatementsSequence
 )
-from nuitka.nodes.SubscriptNodes import ExpressionSubscriptLookup
 from nuitka.nodes.VariableRefNodes import (
+    ExpressionTargetTempVariableRef,
     ExpressionTargetVariableRef,
+    ExpressionTempVariableRef,
     ExpressionVariableRef
 )
+from nuitka.tree import SyntaxErrors
+from nuitka.utils import Utils
 
 from .Helpers import (
     applyLaterWrappers,
@@ -110,16 +109,17 @@ from .Helpers import (
     makeDictCreationOrConstant,
     makeModuleFrame,
     makeSequenceCreationOrConstant,
+    makeStatementsSequenceFromStatement,
     makeStatementsSequenceOrStatement,
+    makeTryFinallyStatement,
+    mangleName,
     mergeStatements,
-    setBuildDispatchers
+    setBuildingDispatchers
 )
-from .ImportCache import addImportedModule
 from .ReformulationAssertStatements import buildAssertNode
 from .ReformulationAssignmentStatements import (
     buildAssignNode,
     buildDeleteNode,
-    buildExtSliceNode,
     buildInplaceAssignNode
 )
 from .ReformulationBooleanExpressions import buildBoolOpNode
@@ -134,14 +134,22 @@ from .ReformulationContractionExpressions import (
 )
 from .ReformulationExecStatements import buildExecNode
 from .ReformulationFunctionStatements import buildFunctionNode
+from .ReformulationImportStatements import (
+    buildImportFromNode,
+    checkFutureImportsOnlyAtStart
+)
 from .ReformulationLambdaExpressions import buildLambdaNode
 from .ReformulationLoopStatements import buildForLoopNode, buildWhileLoopNode
-from .ReformulationNamespacePackages import createNamespacePackage
+from .ReformulationNamespacePackages import (
+    createNamespacePackage,
+    createPathAssignment
+)
 from .ReformulationPrintStatements import buildPrintNode
+from .ReformulationSubscriptExpressions import buildSubscriptNode
 from .ReformulationTryExceptStatements import buildTryExceptionNode
 from .ReformulationTryFinallyStatements import (
     buildTryFinallyNode,
-    makeTryFinallyIndicator
+    makeTryFinallyIndicatorStatements
 )
 from .ReformulationWithStatements import buildWithNode
 from .ReformulationYieldExpressions import buildYieldFromNode, buildYieldNode
@@ -158,7 +166,7 @@ def buildVariableReferenceNode(provider, node, source_ref):
         provider.markAsClassClosureTaker()
 
     return ExpressionVariableRef(
-        variable_name = node.id,
+        variable_name = mangleName(node.id, provider),
         source_ref    = source_ref
     )
 
@@ -191,7 +199,7 @@ def buildConditionNode(provider, node, source_ref):
     # into nested conditional statements.
 
     return StatementConditional(
-        condition  = buildNode( provider, node.test, source_ref ),
+        condition  = buildNode(provider, node.test, source_ref),
         yes_branch = buildStatementsNode(
             provider   = provider,
             nodes      = node.body,
@@ -261,86 +269,44 @@ def buildRaiseNode(provider, node, source_ref):
     # attached, for Python3, you can only give type (actually value) and cause.
 
     if Utils.python_version < 300:
-        return StatementRaiseException(
-            exception_type  = buildNode( provider, node.type, source_ref, allow_none = True ),
-            exception_value = buildNode( provider, node.inst, source_ref, allow_none = True ),
-            exception_trace = buildNode( provider, node.tback, source_ref, allow_none = True ),
-            exception_cause = None,
-            source_ref      = source_ref
-        )
+        exception_type  = buildNode(provider, node.type, source_ref, allow_none = True)
+        exception_value = buildNode(provider, node.inst, source_ref, allow_none = True)
+        exception_trace = buildNode(provider, node.tback, source_ref, allow_none = True)
+        exception_cause = None
     else:
-        return StatementRaiseException(
-            exception_type  = buildNode( provider, node.exc, source_ref, allow_none = True ),
-            exception_value = None,
-            exception_trace = None,
-            exception_cause = buildNode( provider, node.cause, source_ref, allow_none = True ),
-            source_ref      = source_ref
+        exception_type  = buildNode(provider, node.exc, source_ref, allow_none = True)
+        exception_value = None
+        exception_trace = None
+        exception_cause = buildNode(provider, node.cause, source_ref, allow_none = True)
+
+    result = StatementRaiseException(
+        exception_type  = exception_type,
+        exception_value = exception_value,
+        exception_trace = exception_trace,
+        exception_cause = exception_cause,
+        source_ref      = source_ref
+    )
+
+    if exception_cause is not None:
+        result.setCompatibleSourceReference(
+            source_ref = exception_cause.getCompatibleSourceReference()
+        )
+    elif exception_trace is not None:
+        result.setCompatibleSourceReference(
+            source_ref = exception_trace.getCompatibleSourceReference()
+        )
+    elif exception_value is not None:
+        result.setCompatibleSourceReference(
+            source_ref = exception_value.getCompatibleSourceReference()
+        )
+    elif exception_type is not None:
+        result.setCompatibleSourceReference(
+            source_ref = exception_type.getCompatibleSourceReference()
         )
 
-def buildSubscriptNode(provider, node, source_ref):
-    # Subscript expression nodes.
+    return result
 
-    assert getKind( node.ctx ) == "Load", source_ref
-
-    # The subscribt "[]" operator is one of many different things. This is
-    # expressed by this kind, there are "slice" lookups (two values, even if one
-    # is using default), and then "index" lookups. The form with three argument
-    # is really an "index" lookup, with a slice object. And the "..." lookup is
-    # also an index loopup, with it as the argument. So this splits things into
-    # two different operations, "subscript" with a single "subscript" object. Or
-    # a slice lookup with a lower and higher boundary. These things should
-    # behave similar, but they are different slots.
-    kind = getKind( node.slice )
-
-    if kind == "Index":
-        return ExpressionSubscriptLookup(
-            expression = buildNode( provider, node.value, source_ref ),
-            subscript  = buildNode( provider, node.slice.value, source_ref ),
-            source_ref = source_ref
-        )
-    elif kind == "Slice":
-        lower = buildNode( provider, node.slice.lower, source_ref, True )
-        upper = buildNode( provider, node.slice.upper, source_ref, True )
-
-        if node.slice.step is not None:
-            step = buildNode( provider, node.slice.step,  source_ref )
-
-            return ExpressionSubscriptLookup(
-                expression = buildNode( provider, node.value, source_ref ),
-                subscript  = ExpressionSliceObject(
-                    lower      = lower,
-                    upper      = upper,
-                    step       = step,
-                    source_ref = source_ref
-                ),
-                source_ref = source_ref
-            )
-        else:
-            return ExpressionSliceLookup(
-                expression = buildNode( provider, node.value, source_ref ),
-                lower      = lower,
-                upper      = upper,
-                source_ref = source_ref
-            )
-    elif kind == "ExtSlice":
-        return ExpressionSubscriptLookup(
-            expression = buildNode( provider, node.value, source_ref ),
-            subscript  = buildExtSliceNode( provider, node, source_ref ),
-            source_ref = source_ref
-        )
-    elif kind == "Ellipsis":
-        return ExpressionSubscriptLookup(
-            expression = buildNode( provider, node.value, source_ref ),
-            subscript  = ExpressionConstantRef(
-                constant   = Ellipsis,
-                source_ref = source_ref
-            ),
-            source_ref = source_ref
-        )
-    else:
-        assert False, kind
-
-def buildImportModulesNode(node, source_ref):
+def buildImportModulesNode(provider, node, source_ref):
     # Import modules statement. As described in the developer manual, these
     # statements can be treated as several ones.
 
@@ -355,13 +321,16 @@ def buildImportModulesNode(node, source_ref):
     for import_desc in import_names:
         module_name, local_name = import_desc
 
-        module_topname = module_name.split(".")[0]
+        module_topname = module_name.split('.')[0]
 
         # Note: The "level" of import is influenced by the future absolute
         # imports.
         level = 0 if source_ref.getFutureSpec().isAbsoluteImport() else -1
 
         if local_name:
+            # If is gets a local name, the real name must be used as a
+            # temporary value only, being looked up recursively.
+
             import_node = ExpressionImportModule(
                 module_name = module_name,
                 import_list = None,
@@ -369,7 +338,7 @@ def buildImportModulesNode(node, source_ref):
                 source_ref  = source_ref
             )
 
-            for import_name in module_name.split(".")[1:]:
+            for import_name in module_name.split('.')[1:]:
                 import_node = ExpressionImportName(
                     module      = import_node,
                     import_name = import_name,
@@ -390,13 +359,16 @@ def buildImportModulesNode(node, source_ref):
         import_nodes.append(
             StatementAssignmentVariable(
                 variable_ref = ExpressionTargetVariableRef(
-                    variable_name = local_name
-                                      if local_name is not None else
-                                    module_topname,
+                    variable_name = mangleName(
+                                      local_name
+                                        if local_name is not None else
+                                      module_topname,
+                                      provider
+                    ),
                     source_ref    = source_ref
                 ),
-                source     = import_node,
-                source_ref = source_ref
+                source       = import_node,
+                source_ref   = source_ref
             )
         )
 
@@ -408,141 +380,6 @@ def buildImportModulesNode(node, source_ref):
         source_ref = source_ref
     )
 
-def enableFutureFeature(object_name, future_spec, source_ref):
-    if object_name == "unicode_literals":
-        future_spec.enableUnicodeLiterals()
-    elif object_name == "absolute_import":
-        future_spec.enableAbsoluteImport()
-    elif object_name == "division":
-        future_spec.enableFutureDivision()
-    elif object_name == "print_function":
-        future_spec.enableFuturePrint()
-    elif object_name == "barry_as_FLUFL" and Utils.python_version >= 300:
-        future_spec.enableBarry()
-    elif object_name == "braces":
-        SyntaxErrors.raiseSyntaxError(
-            "not a chance",
-            source_ref
-        )
-    elif object_name in ( "nested_scopes", "generators", "with_statement" ):
-        # These are enabled in all cases already.
-        pass
-    else:
-        SyntaxErrors.raiseSyntaxError(
-            "future feature %s is not defined" % object_name,
-            source_ref
-        )
-
-# For checking afterwards, if it was at the beginning of the file.
-_future_import_nodes = []
-
-def buildImportFromNode(provider, node, source_ref):
-    # "from .. import .." statements. This may trigger a star import, or
-    # multiple names being looked up from the given module variable name.
-
-    module_name = node.module if node.module is not None else ""
-    level = node.level
-
-    # Importing from "__future__" module may enable flags.
-    if module_name == "__future__":
-        if not provider.isPythonModule():
-            SyntaxErrors.raiseSyntaxError(
-                reason     = """\
-from __future__ imports must occur at the beginning of the file""",
-                col_offset = 8
-                  if Utils.python_version >= 300 or \
-                  not Options.isFullCompat()
-                else None,
-                source_ref = source_ref
-            )
-
-
-        for import_desc in node.names:
-            object_name, _local_name = import_desc.name, import_desc.asname
-
-            enableFutureFeature(
-                object_name = object_name,
-                future_spec = source_ref.getFutureSpec(),
-                source_ref  = source_ref
-            )
-
-        # Remember it for checks to be applied once module is complete.
-        node.source_ref = source_ref
-        _future_import_nodes.append(node)
-
-    target_names = []
-    import_names = []
-
-    for import_desc in node.names:
-        object_name, local_name = import_desc.name, import_desc.asname
-
-        if object_name == "*":
-            target_names.append(None)
-        else:
-            target_names.append(
-                local_name
-                  if local_name is not None else
-                object_name
-            )
-
-        import_names.append(object_name)
-
-    if None in target_names:
-        # More than "*" is a syntax error in Python, need not care about this at
-        # all, it's only allowed value for import list in  this case.
-        assert target_names == [None]
-
-        # Python3 made this a syntax error unfortunately.
-        if not provider.isPythonModule() and Utils.python_version >= 300:
-            SyntaxErrors.raiseSyntaxError(
-                "import * only allowed at module level",
-                provider.getSourceReference()
-            )
-
-        if provider.isExpressionFunctionBody():
-            provider.markAsStarImportContaining()
-
-        return StatementImportStar(
-            module_import = ExpressionImportModule(
-                module_name = module_name,
-                import_list = ("*",),
-                level       = level,
-                source_ref  = source_ref
-            ),
-            source_ref  = source_ref
-        )
-    else:
-        import_nodes = []
-
-        for target_name, import_name in zip( target_names, import_names ):
-            import_nodes.append(
-                StatementAssignmentVariable(
-                    variable_ref = ExpressionTargetVariableRef(
-                        variable_name = target_name,
-                        source_ref    = source_ref
-                    ),
-                    source     = ExpressionImportName(
-                        module      = ExpressionImportModule(
-                            module_name = module_name,
-                            import_list = tuple(import_names),
-                            level       = level,
-                            source_ref  = source_ref
-                        ),
-                        import_name = import_name,
-                        source_ref  = source_ref
-                    ),
-                    source_ref = source_ref
-                )
-            )
-
-        # Note: Each import is sequential. It can succeed, and the failure of a
-        # later one is not changing one. We can therefore have a sequence of
-        # imports that only import one thing therefore.
-        return StatementsSequence(
-            statements = import_nodes,
-            source_ref = source_ref
-        )
-
 
 def handleGlobalDeclarationNode(provider, node, source_ref):
 
@@ -583,8 +420,8 @@ def handleGlobalDeclarationNode(provider, node, source_ref):
         closure_variable = None
 
         # Re-use already taken global variables, in order to avoid creating yet
-        # another instance, esp. as the markups could then potentially not be
-        # shared.
+        # another instance, esp. as the indications could then potentially not
+        # be shared.
         if provider.hasTakenVariable(variable_name):
             closure_variable = provider.getTakenVariable(variable_name)
 
@@ -604,6 +441,14 @@ def handleGlobalDeclarationNode(provider, node, source_ref):
 
         assert closure_variable.isModuleVariable()
 
+        if Utils.python_version < 340 and \
+           provider.isClassDictCreation() and \
+           closure_variable.getName() == "__class__":
+            SyntaxErrors.raiseSyntaxError(
+                reason     = "cannot make __class__ global",
+                source_ref = source_ref
+            )
+
         provider.registerProvidedVariable(
             variable = closure_variable
         )
@@ -612,9 +457,6 @@ def handleGlobalDeclarationNode(provider, node, source_ref):
 
 
 def handleNonlocalDeclarationNode(provider, node, source_ref):
-    # The source reference of the nonlocal really doesn't matter.
-    # pylint: disable=W0613
-
     # Need to catch the error of declaring a parameter variable as global
     # ourselves here. The AST parsing doesn't catch it, but we can do it here.
     parameters = provider.getParameters()
@@ -696,11 +538,20 @@ def buildStatementContinueLoop(node, source_ref):
         )
 
 
-    return makeTryFinallyIndicator(
-        statement    = StatementContinueLoop(
+    statements = makeTryFinallyIndicatorStatements(
+        is_loop_exit = True,
+        source_ref   = source_ref
+    )
+
+    statements.append(
+        StatementContinueLoop(
             source_ref = source_ref
-        ),
-        is_loop_exit = True
+        )
+    )
+
+    return makeStatementsSequenceOrStatement(
+        statements = statements,
+        source_ref = source_ref
     )
 
 
@@ -708,17 +559,26 @@ def buildStatementBreakLoop(provider, node, source_ref):
     # A bit unusual, we need the provider, but not the node,
     # pylint: disable=W0613
 
-    return makeTryFinallyIndicator(
-        statement    = StatementBreakLoop(
+    statements = makeTryFinallyIndicatorStatements(
+        is_loop_exit = True,
+        source_ref   = source_ref
+    )
+
+    statements.append(
+        StatementBreakLoop(
             source_ref = source_ref
-        ),
-        is_loop_exit = True
+        )
+    )
+
+    return makeStatementsSequenceOrStatement(
+        statements = statements,
+        source_ref = source_ref
     )
 
 
 def buildAttributeNode(provider, node, source_ref):
     return ExpressionAttributeLookup(
-        expression     = buildNode( provider, node.value, source_ref ),
+        source         = buildNode(provider, node.value, source_ref),
         attribute_name = node.attr,
         source_ref     = source_ref
     )
@@ -746,25 +606,67 @@ def buildReturnNode(provider, node, source_ref):
             user_provided = True
         )
 
-
-    return makeTryFinallyIndicator(
-        statement   = StatementReturn(
-            expression = expression,
-            source_ref = source_ref
-        ),
-        is_loop_exit = False
+    # Indicate exceptions to potentially try/finally structures.
+    indicator_statements = makeTryFinallyIndicatorStatements(
+        is_loop_exit = False,
+        source_ref   = source_ref
     )
 
+    if indicator_statements and expression.mayRaiseException(BaseException):
+        tmp_variable = provider.allocateTempVariable(
+            temp_scope = provider.allocateTempScope("return"),
+            name       = "value"
+        )
+
+        statements = [
+                StatementAssignmentVariable(
+                variable_ref = ExpressionTargetTempVariableRef(
+                    variable   = tmp_variable,
+                    source_ref = expression.getSourceReference()
+                ),
+                source       = expression,
+                source_ref   = source_ref
+            )
+        ] + indicator_statements + [
+            StatementReturn(
+                expression = ExpressionTempVariableRef(
+                    variable   = tmp_variable,
+                    source_ref = expression.getSourceReference()
+                ),
+                source_ref = source_ref
+            )
+        ]
+
+        return makeTryFinallyStatement(
+            tried      = statements,
+            final      = StatementReleaseVariable(
+                variable   = tmp_variable,
+                tolerant   = True,
+                source_ref = source_ref
+            ),
+            source_ref = source_ref
+        )
+    else:
+        return makeStatementsSequenceOrStatement(
+            statements = indicator_statements + [
+                StatementReturn(
+                    expression = expression,
+                    source_ref = source_ref
+                )
+            ],
+            source_ref = source_ref
+        )
+
 
 def buildExprOnlyNode(provider, node, source_ref):
     return StatementExpressionOnly(
-        expression = buildNode( provider, node.value, source_ref ),
+        expression = buildNode(provider, node.value, source_ref),
         source_ref = source_ref
     )
 
 
 def buildUnaryOpNode(provider, node, source_ref):
-    if getKind( node.op ) == "Not":
+    if getKind(node.op) == "Not":
         return buildBoolOpNode(
             provider   = provider,
             node       = node,
@@ -772,8 +674,8 @@ def buildUnaryOpNode(provider, node, source_ref):
         )
     else:
         return ExpressionOperationUnary(
-            operator   = getKind( node.op ),
-            operand    = buildNode( provider, node.operand, source_ref ),
+            operator   = getKind(node.op),
+            operand    = buildNode(provider, node.operand, source_ref),
             source_ref = source_ref
         )
 
@@ -784,13 +686,22 @@ def buildBinaryOpNode(provider, node, source_ref):
     if operator == "Div" and source_ref.getFutureSpec().isFutureDivision():
         operator = "TrueDiv"
 
-    return ExpressionOperationBinary(
+    left       = buildNode(provider, node.left, source_ref)
+    right      = buildNode(provider, node.right, source_ref)
+
+    result = ExpressionOperationBinary(
         operator   = operator,
-        left       = buildNode(provider, node.left, source_ref),
-        right      = buildNode(provider, node.right, source_ref),
+        left       = left,
+        right      = right,
         source_ref = source_ref
     )
 
+    result.setCompatibleSourceReference(
+        source_ref = right.getCompatibleSourceReference()
+    )
+
+    return result
+
 
 def buildReprNode(provider, node, source_ref):
     return ExpressionOperationUnary(
@@ -809,7 +720,7 @@ def buildConditionalExpressionNode(provider, node, source_ref):
     )
 
 
-setBuildDispatchers(
+setBuildingDispatchers(
     path_args3 = {
         "Name"         : buildVariableReferenceNode,
         "Assign"       : buildAssignNode,
@@ -833,6 +744,7 @@ setBuildDispatchers(
         "TryFinally"   : _buildTryFinallyNode,
         "Try"          : buildTryNode,
         "Raise"        : buildRaiseNode,
+        "Import"       : buildImportModulesNode,
         "ImportFrom"   : buildImportFromNode,
         "Assert"       : buildAssertNode,
         "Exec"         : buildExecNode,
@@ -857,7 +769,6 @@ setBuildDispatchers(
     },
     path_args2 = {
         "NameConstant" : buildNamedConstantNode,
-        "Import"       : buildImportModulesNode,
         "Str"          : buildStringNode,
         "Num"          : buildNumberNode,
         "Bytes"        : buildBytesNode,
@@ -868,16 +779,43 @@ setBuildDispatchers(
     }
 )
 
+
+def _makeSyntaxErrorCompatible(e):
+    # Encoding problems for Python happen here, for Python3, this was
+    # already done when we read the source code.
+    if Options.isFullCompat() and \
+       (e.args[0].startswith("unknown encoding:") or \
+        e.args[0].startswith("encoding problem:")):
+        if PythonVersions.doShowUnknownEncodingName():
+            complaint = e.args[0].split(':',2)[1]
+        else:
+            complaint = " with BOM"
+
+        e.args = (
+            "encoding problem:%s" % complaint,
+            (e.args[1][0], 1, None, None)
+        )
+
+        if hasattr(e, "msg"):
+            e.msg = e.args[0]
+
+
+
 def buildParseTree(provider, source_code, source_ref, is_module, is_main):
     # There are a bunch of branches here, mostly to deal with version
     # differences for module default variables. pylint: disable=R0912
 
     # Workaround: ast.parse cannot cope with some situations where a file is not
     # terminated by a new line.
-    if not source_code.endswith("\n"):
-        source_code = source_code + "\n"
+    if not source_code.endswith('\n'):
+        source_code = source_code + '\n'
 
-    body = ast.parse(source_code, source_ref.getFilename())
+    try:
+        body = ast.parse(source_code, source_ref.getFilename())
+    except SyntaxError as e:
+        _makeSyntaxErrorCompatible(e)
+
+        raise e
 
     assert getKind(body) == "Module"
 
@@ -896,21 +834,7 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
         source_ref = source_ref
     )
 
-    # Check if a __future__ imports really were at the beginning of the file.
-    for node in body:
-        if node in _future_import_nodes:
-            _future_import_nodes.remove(node)
-        else:
-            if _future_import_nodes:
-                SyntaxErrors.raiseSyntaxError(
-                    reason     = """\
-from __future__ imports must occur at the beginning of the file""",
-                    col_offset = 1
-                      if Utils.python_version >= 300 or \
-                      not Options.isFullCompat() else
-                    None,
-                    source_ref = _future_import_nodes[0].source_ref
-                )
+    checkFutureImportsOnlyAtStart(body)
 
     internal_source_ref = source_ref.atInternal()
 
@@ -923,12 +847,12 @@ from __future__ imports must occur at the beginning of the file""",
             statements.append(
                 StatementExpressionOnly(
                     expression = ExpressionImportModule(
-                        module_name    = "site",
-                        import_list    = (),
-                        level          = 0,
-                        source_ref     = source_ref,
+                        module_name = "site",
+                        import_list = (),
+                        level       = 0,
+                        source_ref  = source_ref,
                     ),
-                    source_ref  = source_ref
+                    source_ref = source_ref
                 )
             )
 
@@ -953,33 +877,17 @@ from __future__ imports must occur at the beginning of the file""",
                     variable_name = "__file__",
                     source_ref    = internal_source_ref
                 ),
-                source       = ExpressionConstantRef(
-                    constant      = source_ref.getFilename(),
-                    source_ref    = internal_source_ref,
-                    user_provided = True
+                source       = ExpressionModuleFileAttributeRef(
+                    source_ref = internal_source_ref,
                 ),
                 source_ref   = internal_source_ref
             )
         )
 
         if provider.isPythonPackage():
-            # TODO: __package__ is not set here, but automatically, which makes
-            # it invisible though
+            # This assigns "__path__" value.
             statements.append(
-                StatementAssignmentVariable(
-                    variable_ref = ExpressionTargetVariableRef(
-                        variable_name = "__path__",
-                        source_ref    = internal_source_ref
-                    ),
-                    source       = ExpressionConstantRef(
-                        constant      = [
-                            Utils.dirname(source_ref.getFilename())
-                        ],
-                        source_ref    = internal_source_ref,
-                        user_provided = True
-                    ),
-                    source_ref   = internal_source_ref
-                )
+                createPathAssignment(internal_source_ref)
             )
 
     if Utils.python_version >= 300:
@@ -1022,7 +930,7 @@ from __future__ imports must occur at the beginning of the file""",
       (Utils.python_version >= 330 and Utils.python_version < 340)
 
     if needs__initializing__:
-        # Set initialzing at the beginning to True
+        # Set "__initializing__" at the beginning to True
         statements.append(
             StatementAssignmentVariable(
                 variable_ref = ExpressionTargetVariableRef(
@@ -1045,7 +953,7 @@ from __future__ imports must occur at the beginning of the file""",
         )
 
     if needs__initializing__:
-        # Set initialzing at the beginning to True
+        # Set "__initializing__" at the end to False
         statements.append(
             StatementAssignmentVariable(
                 variable_ref = ExpressionTargetVariableRef(
@@ -1062,7 +970,6 @@ from __future__ imports must occur at the beginning of the file""",
         )
 
 
-
     if is_module:
         result = makeModuleFrame(
             module     = provider,
@@ -1081,6 +988,7 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
     # Many variables, branches, due to the many cases, pylint: disable=R0912
 
     assert package is None or type(package) is str
+    assert filename is not None
 
     if is_main and Utils.isDir(filename):
         source_filename = Utils.joinpath(filename, "__main__.py")
@@ -1104,8 +1012,7 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
         source_filename = filename
 
         source_ref = SourceCodeReferences.fromFilename(
-            filename    = filename,
-            future_spec = FutureSpec()
+            filename = filename,
         )
 
         if is_main:
@@ -1117,9 +1024,9 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
                 module_name = module_name[:-3]
 
             if is_shlib:
-                module_name = module_name.split(".")[0]
+                module_name = module_name.split('.')[0]
 
-            if "." in module_name:
+            if '.' in module_name:
                 sys.stderr.write(
                     "Error, '%s' is not a proper python module name.\n" % (
                         module_name
@@ -1131,13 +1038,13 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
         if is_shlib:
             result = PythonShlibModule(
                 name         = module_name,
-                source_ref   = source_ref,
                 package_name = package,
+                source_ref   = source_ref
             )
         elif is_main:
             result = PythonMainModule(
-                source_ref = source_ref,
-                main_added = main_added
+                main_added = main_added,
+                source_ref = source_ref
             )
         else:
             result = PythonModule(
@@ -1153,18 +1060,15 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
 
         source_filename = Utils.joinpath(filename, "__init__.py")
 
-        if not Utils.isFile( source_filename ):
-            assert Utils.python_version >= 330, source_filename
-
+        if not Utils.isFile(source_filename):
             source_ref, result = createNamespacePackage(
-                package_name = package_name,
+                package_name   = package_name,
                 module_relpath = filename
             )
             source_filename = None
         else:
             source_ref = SourceCodeReferences.fromFilename(
-                filename    = Utils.abspath( source_filename ),
-                future_spec = FutureSpec()
+                filename = Utils.abspath(source_filename),
             )
 
             result = PythonPackage(
@@ -1175,11 +1079,11 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
     else:
         sys.stderr.write(
             "%s: can't open file '%s'.\n" % (
-                Utils.basename( sys.argv[0] ),
+                Utils.basename(sys.argv[0]),
                 filename
             )
         )
-        sys.exit( 2 )
+        sys.exit(2)
 
     if not Options.shallHaveStatementLines():
         source_ref = source_ref.atInternal()
@@ -1187,12 +1091,10 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
     return result, source_ref, source_filename
 
 
-def createModuleTree(module, source_ref, source_filename, is_main):
+def createModuleTree(module, source_ref, source_code, is_main):
     if Options.isShowProgress():
         memory_watch = Utils.MemoryWatch()
 
-    source_code = readSourceCodeFromFilename(source_filename)
-
     module_body = buildParseTree(
         provider    = module,
         source_code = source_code,
@@ -1201,9 +1103,12 @@ def createModuleTree(module, source_ref, source_filename, is_main):
         is_main     = is_main
     )
 
-    module.setBody(
-        module_body
-    )
+    if module_body.isStatementsFrame():
+        module_body = makeStatementsSequenceFromStatement(
+            statement = module_body,
+        )
+
+    module.setBody(module_body)
 
     completeVariableClosures(module)
 
@@ -1228,18 +1133,19 @@ def buildModuleTree(filename, package, is_top, is_main):
     )
 
     addImportedModule(
-        module_relpath = Utils.relpath(filename),
+        module_relpath  = Utils.relpath(filename),
         imported_module = module
     )
 
     # If there is source code associated (not the case for namespace packages of
     # Python3.3 or higher, then read it.
     if source_filename is not None:
+        # Read source code.
         createModuleTree(
-            module          = module,
-            source_ref      = source_ref,
-            source_filename = source_filename,
-            is_main         = is_main
+            module      = module,
+            source_ref  = source_ref,
+            source_code = readSourceCodeFromFilename(module, source_filename),
+            is_main     = is_main
         )
 
     return module
