@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,7 +22,7 @@ source code comments with developer manual sections.
 
 """
 
-from nuitka.nodes.CallNodes import ExpressionCall
+from nuitka.nodes.CallNodes import makeExpressionCall
 from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
 from nuitka.nodes.FunctionNodes import (
     ExpressionFunctionCall,
@@ -78,21 +78,32 @@ def _makeCallNode(called, positional_args, keys, values, list_star_arg,
     # pylint: disable=R0914
 
     if list_star_arg is None and dict_star_arg is None:
-        return ExpressionCall(
-            called  = called,
-            args    = makeSequenceCreationOrConstant(
+        result = makeExpressionCall(
+            called     = called,
+            args       = makeSequenceCreationOrConstant(
                 sequence_kind = "tuple",
                 elements      = positional_args,
                 source_ref    = source_ref
             ),
-            kw      = makeDictCreationOrConstant(
+            kw         = makeDictCreationOrConstant(
                 keys       = keys,
                 values     = values,
                 lazy_order = True,
                 source_ref = source_ref
             ),
-            source_ref      = source_ref,
+            source_ref = source_ref,
         )
+
+        if values:
+            result.setCompatibleSourceReference(
+                source_ref = values[-1].getCompatibleSourceReference()
+            )
+        elif positional_args:
+            result.setCompatibleSourceReference(
+                source_ref = positional_args[-1].getCompatibleSourceReference()
+            )
+
+        return result
     else:
         # Dispatch to complex helper function for each case. These do
         # re-formulation of complex calls according to developer manual.
@@ -120,35 +131,35 @@ def _makeCallNode(called, positional_args, keys, values, list_star_arg,
         )
 
         table = {
-            (  True,   True,  True, False ) :
+            (True,   True,  True, False) :
                 getFunctionCallHelperPosKeywordsStarList,
-            (  True,  False,  True, False ) :
+            (True,  False,  True, False) :
                 getFunctionCallHelperPosStarList,
-            ( False,   True,  True, False ) :
+            (False,   True,  True, False) :
                 getFunctionCallHelperKeywordsStarList,
-            ( False,  False,  True, False ) :
+            (False,  False,  True, False) :
                 getFunctionCallHelperStarList,
-            (  True,   True, False,  True ) :
+            (True,   True, False,  True) :
                 getFunctionCallHelperPosKeywordsStarDict,
-            (  True,  False, False,  True ) :
+            (True,  False, False,  True) :
                 getFunctionCallHelperPosStarDict,
-            ( False,   True, False,  True ) :
+            (False,   True, False,  True) :
                 getFunctionCallHelperKeywordsStarDict,
-            ( False,  False, False,  True ) :
+            (False,  False, False,  True) :
                 getFunctionCallHelperStarDict,
-            (  True,   True,  True,  True ) :
+            (True,   True,  True,  True) :
                 getFunctionCallHelperPosKeywordsStarListStarDict,
-            (  True,  False,  True,  True ) :
+            (True,  False,  True,  True) :
                 getFunctionCallHelperPosStarListStarDict,
-            ( False,   True,  True,  True ) :
+            (False,   True,  True,  True) :
                 getFunctionCallHelperKeywordsStarListStarDict,
-            ( False,  False,  True,  True ) :
+            (False,  False,  True,  True) :
                 getFunctionCallHelperStarListStarDict,
         }
 
         get_helper = table[ key ]
 
-        helper_args = [ called ]
+        helper_args = [called]
 
         if positional_args:
             helper_args.append(
@@ -175,7 +186,7 @@ def _makeCallNode(called, positional_args, keys, values, list_star_arg,
         if dict_star_arg is not None:
             helper_args.append(dict_star_arg)
 
-        return ExpressionFunctionCall(
+        result = ExpressionFunctionCall(
             function   = ExpressionFunctionCreation(
                 function_ref = ExpressionFunctionRef(
                     function_body = get_helper(),
@@ -189,3 +200,9 @@ def _makeCallNode(called, positional_args, keys, values, list_star_arg,
             values     = helper_args,
             source_ref = source_ref,
         )
+
+        result.setCompatibleSourceReference(
+            source_ref = helper_args[-1].getCompatibleSourceReference()
+        )
+
+        return result
