@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -17,15 +17,14 @@
 #
 """ Reformulation of class statements.
 
-Consult the developmer manual for information. TODO: Add ability to sync
+Consult the developer manual for information. TODO: Add ability to sync
 source code comments with developer manual sections.
 
 """
 
-from nuitka import Utils
 from nuitka.nodes.AssignNodes import (
     StatementAssignmentVariable,
-    StatementDelVariable
+    StatementReleaseVariable
 )
 from nuitka.nodes.AttributeNodes import (
     ExpressionAttributeLookup,
@@ -49,6 +48,7 @@ from nuitka.nodes.FunctionNodes import (
     ExpressionFunctionBody,
     ExpressionFunctionCall,
     ExpressionFunctionCreation,
+    ExpressionFunctionQualnameRef,
     ExpressionFunctionRef
 )
 from nuitka.nodes.GlobalsLocalsNodes import (
@@ -65,6 +65,7 @@ from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
     ExpressionVariableRef
 )
+from nuitka.utils import Utils
 
 from .Helpers import (
     buildNode,
@@ -145,16 +146,19 @@ def _buildClassNode3(provider, node, source_ref):
     )
 
     source_ref_orig = source_ref
-    source_ref = source_ref.atInternal()
 
     if body is not None:
         # The frame guard has nothing to tell its line number to.
         body.source_ref = source_ref
 
+    module_variable = class_creation_function.getVariableForAssignment(
+        "__module__"
+    )
+
     statements = [
         StatementSetLocals(
             new_locals = ExpressionTempVariableRef(
-                variable   = tmp_prepared.makeReference(provider),
+                variable   = tmp_prepared,
                 source_ref = source_ref
             ),
             source_ref = source_ref
@@ -162,9 +166,10 @@ def _buildClassNode3(provider, node, source_ref):
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetVariableRef(
                 variable_name = "__module__",
+                variable      = module_variable,
                 source_ref    = source_ref
             ),
-            source        = ExpressionConstantRef(
+            source       = ExpressionConstantRef(
                 constant      = provider.getParentModule().getFullName(),
                 source_ref    = source_ref,
                 user_provided = True
@@ -174,40 +179,60 @@ def _buildClassNode3(provider, node, source_ref):
     ]
 
     if class_doc is not None:
+        doc_variable = class_creation_function.getVariableForAssignment(
+            "__doc__"
+        )
+
         statements.append(
             StatementAssignmentVariable(
                 variable_ref = ExpressionTargetVariableRef(
                     variable_name = "__doc__",
+                    variable      = doc_variable,
                     source_ref    = source_ref
                 ),
-                source        = ExpressionConstantRef(
+                source       = ExpressionConstantRef(
                     constant      = class_doc,
                     source_ref    = source_ref,
                     user_provided = True
                 ),
-                source_ref   = source_ref.atInternal()
-            )
-        )
-
-    # The "__qualname__" attribute is new in Python 3.3.
-    if Utils.python_version >= 330:
-        qualname = class_creation_function.getFunctionQualname()
-
-        statements.append(
-            StatementAssignmentVariable(
-                variable_ref = ExpressionTargetVariableRef(
-                    variable_name = "__qualname__",
-                    source_ref    = source_ref
-                ),
-                source        = ExpressionConstantRef(
-                    constant      = qualname,
-                    source_ref    = source_ref,
-                    user_provided = True
-                ),
                 source_ref   = source_ref
             )
         )
 
+    # The "__qualname__" attribute is new in Python 3.3.
+    if Utils.python_version >= 330:
+        qualname = class_creation_function.getFunctionQualname()
+        qualname_variable = class_creation_function.getVariableForAssignment(
+            "__qualname__"
+        )
+
+        if Utils.python_version < 340:
+            qualname_ref = ExpressionConstantRef(
+                constant      = qualname,
+                source_ref    = source_ref,
+                user_provided = True
+            )
+        else:
+            qualname_ref = ExpressionFunctionQualnameRef(
+                function_body = class_creation_function,
+                source_ref    = source_ref,
+            )
+
+        statements.append(
+            StatementAssignmentVariable(
+                variable_ref = ExpressionTargetVariableRef(
+                    variable_name = "__qualname__",
+                    variable      = qualname_variable,
+                    source_ref    = source_ref
+                ),
+                source       = qualname_ref,
+                source_ref   = source_ref
+            )
+        )
+
+        if Utils.python_version >= 340:
+            qualname_assign = statements[-1]
+
     if Utils.python_version >= 340 and False: # TODO: Temporarily reverted:
         tmp_class = class_creation_function.allocateTempVariable(
             temp_scope = None,
@@ -215,20 +240,26 @@ def _buildClassNode3(provider, node, source_ref):
         )
 
         class_target_variable_ref = ExpressionTargetTempVariableRef(
-            variable   = tmp_class.makeReference(class_creation_function),
+            variable   = tmp_class,
             source_ref = source_ref
         )
         class_variable_ref = ExpressionTempVariableRef(
-            variable   = tmp_class.makeReference(class_creation_function),
+            variable   = tmp_class,
             source_ref = source_ref
         )
     else:
+        class_variable = class_creation_function.getVariableForAssignment(
+            "__class__"
+        )
+
         class_target_variable_ref = ExpressionTargetVariableRef(
             variable_name = "__class__",
+            variable      = class_variable,
             source_ref    = source_ref
         )
         class_variable_ref = ExpressionVariableRef(
             variable_name = "__class__",
+            variable      = class_variable,
             source_ref    = source_ref
         )
 
@@ -238,7 +269,7 @@ def _buildClassNode3(provider, node, source_ref):
             variable_ref = class_target_variable_ref,
             source       = ExpressionCall(
                 called     = ExpressionTempVariableRef(
-                    variable   = tmp_metaclass.makeReference(provider),
+                    variable   = tmp_metaclass,
                     source_ref = source_ref
                 ),
                 args       = makeSequenceCreationOrConstant(
@@ -250,7 +281,7 @@ def _buildClassNode3(provider, node, source_ref):
                             user_provided = True
                         ),
                         ExpressionTempVariableRef(
-                            variable   = tmp_bases.makeReference(provider),
+                            variable   = tmp_bases,
                             source_ref = source_ref
                         ),
                         ExpressionBuiltinLocals(
@@ -260,7 +291,7 @@ def _buildClassNode3(provider, node, source_ref):
                     source_ref    = source_ref
                 ),
                 kw         = ExpressionTempVariableRef(
-                    variable   = tmp_class_decl_dict.makeReference(provider),
+                    variable   = tmp_class_decl_dict,
                     source_ref = source_ref
                 ),
                 source_ref = source_ref
@@ -279,12 +310,12 @@ def _buildClassNode3(provider, node, source_ref):
         source_ref = source_ref
     )
 
-    # The class body is basically a function that implicitely, at the end
+    # The class body is basically a function that implicitly, at the end
     # returns its locals and cannot have other return statements contained.
 
     class_creation_function.setBody(body)
 
-    # The class body is basically a function that implicitely, at the end
+    # The class body is basically a function that implicitly, at the end
     # returns its created class and cannot have other return statements
     # contained.
 
@@ -322,7 +353,7 @@ def _buildClassNode3(provider, node, source_ref):
     statements = (
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_bases.makeReference( provider ),
+                variable   = tmp_bases,
                 source_ref = source_ref
             ),
             source       = makeSequenceCreationOrConstant(
@@ -336,11 +367,11 @@ def _buildClassNode3(provider, node, source_ref):
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class_decl_dict.makeReference(provider),
+                variable   = tmp_class_decl_dict,
                 source_ref = source_ref
             ),
             source       = makeDictCreationOrConstant(
-                keys      = [
+                keys       = [
                     ExpressionConstantRef(
                         constant      = keyword.arg,
                         source_ref    = source_ref,
@@ -349,7 +380,7 @@ def _buildClassNode3(provider, node, source_ref):
                     for keyword in
                     node.keywords
                 ],
-                values = [
+                values     = [
                     buildNode(provider, keyword.value, source_ref)
                     for keyword in
                     node.keywords
@@ -357,16 +388,16 @@ def _buildClassNode3(provider, node, source_ref):
                 lazy_order = False,
                 source_ref = source_ref
             ),
-            source_ref = source_ref
+            source_ref   = source_ref
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_metaclass.makeReference(provider),
+                variable   = tmp_metaclass,
                 source_ref = source_ref
             ),
             source       = ExpressionSelectMetaclass(
-                metaclass = ExpressionConditional(
-                    condition = ExpressionComparison(
+                metaclass  = ExpressionConditional(
+                    condition      = ExpressionComparison(
                         comparator = "In",
                         left       = ExpressionConstantRef(
                             constant      = "metaclass",
@@ -374,18 +405,14 @@ def _buildClassNode3(provider, node, source_ref):
                             user_provided = True
                         ),
                         right      = ExpressionTempVariableRef(
-                            variable   = tmp_class_decl_dict.makeReference(
-                                provider
-                            ),
+                            variable   = tmp_class_decl_dict,
                             source_ref = source_ref
                         ),
                         source_ref = source_ref
                     ),
                     yes_expression = ExpressionDictOperationGet(
                         dicte      = ExpressionTempVariableRef(
-                            variable   = tmp_class_decl_dict.makeReference(
-                                provider
-                            ),
+                            variable   = tmp_class_decl_dict,
                             source_ref = source_ref
                         ),
                         key        = ExpressionConstantRef(
@@ -397,7 +424,7 @@ def _buildClassNode3(provider, node, source_ref):
                     ),
                     no_expression  = ExpressionConditional(
                         condition      = ExpressionTempVariableRef(
-                            variable   = tmp_bases.makeReference(provider),
+                            variable   = tmp_bases,
                             source_ref = source_ref
                         ),
                         no_expression  = ExpressionBuiltinRef(
@@ -406,10 +433,8 @@ def _buildClassNode3(provider, node, source_ref):
                         ),
                         yes_expression = ExpressionBuiltinType1(
                             value      = ExpressionSubscriptLookup(
-                                expression = ExpressionTempVariableRef(
-                                    variable   = tmp_bases.makeReference(
-                                        provider
-                                    ),
+                                subscribed = ExpressionTempVariableRef(
+                                    variable   = tmp_bases,
                                     source_ref = source_ref
                                 ),
                                 subscript  = ExpressionConstantRef(
@@ -425,13 +450,13 @@ def _buildClassNode3(provider, node, source_ref):
                     ),
                     source_ref     = source_ref
                 ),
-                bases     = ExpressionTempVariableRef(
-                    variable   = tmp_bases.makeReference(provider),
+                bases      = ExpressionTempVariableRef(
+                    variable   = tmp_bases,
                     source_ref = source_ref
                 ),
                 source_ref = source_ref
             ),
-            source_ref = source_ref_orig
+            source_ref   = source_ref_orig
         ),
         StatementConditional(
             condition  = ExpressionComparison(
@@ -442,9 +467,7 @@ def _buildClassNode3(provider, node, source_ref):
                     user_provided = True
                 ),
                 right      = ExpressionTempVariableRef(
-                    variable   = tmp_class_decl_dict.makeReference(
-                        provider
-                    ),
+                    variable   = tmp_class_decl_dict,
                     source_ref = source_ref
                 ),
                 source_ref = source_ref
@@ -452,13 +475,11 @@ def _buildClassNode3(provider, node, source_ref):
             no_branch  = None,
             yes_branch = makeStatementsSequenceFromStatement(
                 statement = StatementDictOperationRemove(
-                    dicte = ExpressionTempVariableRef(
-                        variable   = tmp_class_decl_dict.makeReference(
-                            provider
-                        ),
+                    dicte      = ExpressionTempVariableRef(
+                        variable   = tmp_class_decl_dict,
                         source_ref = source_ref
                     ),
-                    key   = ExpressionConstantRef(
+                    key        = ExpressionConstantRef(
                         constant      = "metaclass",
                         source_ref    = source_ref,
                         user_provided = True
@@ -470,17 +491,13 @@ def _buildClassNode3(provider, node, source_ref):
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_prepared.makeReference(
-                    provider
-                ),
+                variable   = tmp_prepared,
                 source_ref = source_ref
             ),
             source       = ExpressionConditional(
-                condition = ExpressionBuiltinHasattr(
+                condition      = ExpressionBuiltinHasattr( # pylint: disable=E1123,E1120
                     object     = ExpressionTempVariableRef(
-                        variable   = tmp_metaclass.makeReference(
-                            provider
-                        ),
+                        variable   = tmp_metaclass,
                         source_ref = source_ref
                     ),
                     name       = ExpressionConstantRef(
@@ -490,17 +507,15 @@ def _buildClassNode3(provider, node, source_ref):
                     ),
                     source_ref = source_ref
                 ),
-                no_expression = ExpressionConstantRef(
+                no_expression  = ExpressionConstantRef(
                     constant      = {},
                     source_ref    = source_ref,
                     user_provided = True
                 ),
                 yes_expression = ExpressionCall(
                     called     = ExpressionAttributeLookup(
-                        expression     = ExpressionTempVariableRef(
-                            variable   = tmp_metaclass.makeReference(
-                                provider
-                            ),
+                        source         = ExpressionTempVariableRef(
+                            variable   = tmp_metaclass,
                             source_ref = source_ref
                         ),
                         attribute_name = "__prepare__",
@@ -514,66 +529,58 @@ def _buildClassNode3(provider, node, source_ref):
                                 user_provided = True
                             ),
                             ExpressionTempVariableRef(
-                                variable   = tmp_bases.makeReference(
-                                    provider
-                                ),
+                                variable   = tmp_bases,
                                 source_ref = source_ref
                             )
                         ),
                         source_ref = source_ref
                     ),
                     kw         = ExpressionTempVariableRef(
-                        variable   = tmp_class_decl_dict.makeReference(
-                            provider
-                        ),
+                        variable   = tmp_class_decl_dict,
                         source_ref = source_ref
                     ),
                     source_ref = source_ref
                 ),
-                source_ref = source_ref
+                source_ref     = source_ref
             ),
-            source_ref = source_ref
+            source_ref   = source_ref
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetVariableRef(
                 variable_name = node.name,
                 source_ref    = source_ref
             ),
-            source     = decorated_body,
-            source_ref = source_ref
+            source       = decorated_body,
+            source_ref   = source_ref
         ),
     )
 
+    if Utils.python_version >= 340:
+        class_assign = statements[-1]
+
+        # assert False, class_creation_function
+        class_creation_function.qualname_setup = class_assign, qualname_assign
+
+
+
     final = (
-        StatementDelVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_bases.makeReference(provider),
-                source_ref = source_ref
-            ),
+        StatementReleaseVariable(
+            variable   = tmp_bases,
             tolerant   = True,
             source_ref = source_ref
         ),
-        StatementDelVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class_decl_dict.makeReference(provider),
-                source_ref = source_ref
-            ),
+        StatementReleaseVariable(
+            variable   = tmp_class_decl_dict,
             tolerant   = True,
             source_ref = source_ref
         ),
-        StatementDelVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_metaclass.makeReference(provider),
-                source_ref = source_ref
-            ),
+        StatementReleaseVariable(
+            variable   = tmp_metaclass,
             tolerant   = True,
             source_ref = source_ref
         ),
-        StatementDelVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_prepared.makeReference(provider),
-                source_ref = source_ref
-            ),
+        StatementReleaseVariable(
+            variable   = tmp_prepared,
             tolerant   = True,
             source_ref = source_ref
         )
@@ -612,7 +619,7 @@ def _buildClassNode2(provider, node, source_ref):
         # The frame guard has nothing to tell its line number to.
         body.source_ref = source_ref.atInternal()
 
-    # The class body is basically a function that implicitely, at the end
+    # The class body is basically a function that implicitly, at the end
     # returns its locals and cannot have other return statements contained, and
     # starts out with a variables "__module__" and potentially "__doc__" set.
     statements = [
@@ -621,7 +628,7 @@ def _buildClassNode2(provider, node, source_ref):
                 variable_name = "__module__",
                 source_ref    = source_ref
             ),
-            source        = ExpressionConstantRef(
+            source       = ExpressionConstantRef(
                 constant      = provider.getParentModule().getFullName(),
                 source_ref    = source_ref,
                 user_provided = True
@@ -637,7 +644,7 @@ def _buildClassNode2(provider, node, source_ref):
                     variable_name = "__doc__",
                     source_ref    = source_ref
                 ),
-                source        = ExpressionConstantRef(
+                source       = ExpressionConstantRef(
                     constant      = class_doc,
                     source_ref    = source_ref,
                     user_provided = True
@@ -662,14 +669,14 @@ def _buildClassNode2(provider, node, source_ref):
         source_ref = source_ref
     )
 
-    # The class body is basically a function that implicitely, at the end
+    # The class body is basically a function that implicitly, at the end
     # returns its locals and cannot have other return statements contained.
 
-    function_body.setBody( body )
+    function_body.setBody(body)
 
-    temp_scope = provider.allocateTempScope( "class_creation" )
+    temp_scope = provider.allocateTempScope("class_creation")
 
-    tmp_bases = provider.allocateTempVariable( temp_scope, "bases" )
+    tmp_bases = provider.allocateTempVariable(temp_scope, "bases")
     tmp_class_dict = provider.allocateTempVariable(temp_scope, "class_dict")
     tmp_metaclass = provider.allocateTempVariable(temp_scope, "metaclass")
     tmp_class = provider.allocateTempVariable(temp_scope, "class")
@@ -677,7 +684,7 @@ def _buildClassNode2(provider, node, source_ref):
     statements = [
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_bases.makeReference( provider ),
+                variable   = tmp_bases,
                 source_ref = source_ref
             ),
             source       = makeSequenceCreationOrConstant(
@@ -691,11 +698,11 @@ def _buildClassNode2(provider, node, source_ref):
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class_dict.makeReference(provider),
+                variable   = tmp_class_dict,
                 source_ref = source_ref
             ),
             source       =   ExpressionFunctionCall(
-                function = ExpressionFunctionCreation(
+                function   = ExpressionFunctionCreation(
                     function_ref = ExpressionFunctionRef(
                         function_body = function_body,
                         source_ref    = source_ref
@@ -712,11 +719,11 @@ def _buildClassNode2(provider, node, source_ref):
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_metaclass.makeReference(provider),
+                variable   = tmp_metaclass,
                 source_ref = source_ref
             ),
             source       = ExpressionConditional(
-                condition =  ExpressionComparison(
+                condition      =  ExpressionComparison(
                     comparator = "In",
                     left       = ExpressionConstantRef(
                         constant      = "__metaclass__",
@@ -724,46 +731,46 @@ def _buildClassNode2(provider, node, source_ref):
                         user_provided = True
                     ),
                     right      = ExpressionTempVariableRef(
-                        variable   = tmp_class_dict.makeReference(provider),
+                        variable   = tmp_class_dict,
                         source_ref = source_ref
                     ),
                     source_ref = source_ref
                 ),
                 yes_expression = ExpressionDictOperationGet(
-                    dicte = ExpressionTempVariableRef(
-                        variable   = tmp_class_dict.makeReference(provider),
+                    dicte      = ExpressionTempVariableRef(
+                        variable   = tmp_class_dict,
                         source_ref = source_ref
                     ),
-                    key   = ExpressionConstantRef(
+                    key        = ExpressionConstantRef(
                         constant      = "__metaclass__",
                         source_ref    = source_ref,
                         user_provided = True
                     ),
                     source_ref = source_ref
                 ),
-                no_expression = ExpressionSelectMetaclass(
-                    metaclass = None,
-                    bases     = ExpressionTempVariableRef(
-                        variable   = tmp_bases.makeReference( provider ),
+                no_expression  = ExpressionSelectMetaclass(
+                    metaclass  = None,
+                    bases      = ExpressionTempVariableRef(
+                        variable   = tmp_bases,
                         source_ref = source_ref
                     ),
                     source_ref = source_ref
                 ),
-                source_ref = source_ref
+                source_ref     = source_ref
             ),
-            source_ref = source_ref
+            source_ref   = source_ref
         ),
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class.makeReference(provider),
+                variable   = tmp_class,
                 source_ref = source_ref
             ),
-            source     = ExpressionCallNoKeywords(
-                called         = ExpressionTempVariableRef(
-                    variable   = tmp_metaclass.makeReference(provider),
+            source       = ExpressionCallNoKeywords(
+                called     = ExpressionTempVariableRef(
+                    variable   = tmp_metaclass,
                     source_ref = source_ref
                 ),
-                args           = ExpressionMakeTuple(
+                args       = ExpressionMakeTuple(
                     elements   = (
                         ExpressionConstantRef(
                             constant      = node.name,
@@ -771,13 +778,11 @@ def _buildClassNode2(provider, node, source_ref):
                             user_provided = True
                         ),
                         ExpressionTempVariableRef(
-                            variable   = tmp_bases.makeReference( provider ),
+                            variable   = tmp_bases,
                             source_ref = source_ref
                         ),
                         ExpressionTempVariableRef(
-                            variable   = tmp_class_dict.makeReference(
-                                provider
-                            ),
+                            variable   = tmp_class_dict,
                             source_ref = source_ref
                         )
                     ),
@@ -785,29 +790,27 @@ def _buildClassNode2(provider, node, source_ref):
                 ),
                 source_ref = source_ref
             ),
-            source_ref = source_ref
+            source_ref   = source_ref
         ),
     ]
 
     for decorator in buildNodeList(
             provider,
-            reversed( node.decorator_list ),
+            reversed(node.decorator_list),
             source_ref
         ):
         statements.append(
             StatementAssignmentVariable(
                 variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = tmp_class.makeReference(provider),
+                    variable   = tmp_class,
                     source_ref = source_ref
                 ),
                 source       = ExpressionCallNoKeywords(
                     called     = decorator,
                     args       = ExpressionMakeTuple(
-                        elements  = (
+                        elements   = (
                             ExpressionTempVariableRef(
-                                variable   = tmp_class.makeReference(
-                                    provider
-                                ),
+                                variable   = tmp_class,
                                 source_ref = source_ref
                             ),
                         ),
@@ -825,44 +828,32 @@ def _buildClassNode2(provider, node, source_ref):
                 variable_name = node.name,
                 source_ref    = source_ref
             ),
-            source     = ExpressionTempVariableRef(
-                variable   = tmp_class.makeReference(provider),
+            source       = ExpressionTempVariableRef(
+                variable   = tmp_class,
                 source_ref = source_ref
             ),
-            source_ref = source_ref
+            source_ref   = source_ref
         )
     )
 
     final = (
-        StatementDelVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class.makeReference(provider),
-                source_ref = source_ref
-            ),
+        StatementReleaseVariable(
+            variable   = tmp_class,
             tolerant   = True,
             source_ref = source_ref
         ),
-        StatementDelVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_bases.makeReference( provider ),
-                source_ref = source_ref
-            ),
+        StatementReleaseVariable(
+            variable   = tmp_bases,
             tolerant   = True,
             source_ref = source_ref
         ),
-        StatementDelVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_class_dict.makeReference(provider),
-                source_ref = source_ref
-            ),
+        StatementReleaseVariable(
+            variable   = tmp_class_dict,
             tolerant   = True,
             source_ref = source_ref
         ),
-        StatementDelVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_metaclass.makeReference(provider),
-                source_ref = source_ref
-            ),
+        StatementReleaseVariable(
+            variable   = tmp_metaclass,
             tolerant   = True,
             source_ref = source_ref
         )
@@ -876,7 +867,7 @@ def _buildClassNode2(provider, node, source_ref):
 
 
 def buildClassNode(provider, node, source_ref):
-    assert getKind( node ) == "ClassDef"
+    assert getKind(node) == "ClassDef"
 
     # Python2 and Python3 are similar, but fundamentally different, so handle
     # them in dedicated code.
@@ -884,6 +875,12 @@ def buildClassNode(provider, node, source_ref):
     pushIndicatorVariable(Ellipsis)
 
     try:
+        # There appears to be a inconsistency with the top level line number
+        # not being the one really the class has, if there are bases, and a
+        # decorator.
+        if node.bases:
+            source_ref = source_ref.atLineNumber(node.bases[-1].lineno)
+
         if Utils.python_version >= 300:
             return _buildClassNode3(provider, node, source_ref)
         else:
