@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,7 +24,7 @@ source code comments with developer manual sections.
 
 from nuitka.nodes.AssignNodes import (
     StatementAssignmentVariable,
-    StatementDelVariable
+    StatementReleaseVariable
 )
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTargetTempVariableRef,
@@ -99,26 +99,23 @@ def buildComparisonNode(provider, node, source_ref):
                 source_ref   = source_ref,
             )
 
-            # TODO: The delete must be placed later.
+            # TODO: The release ought to be placed later.
             final.append(
-                StatementDelVariable(
-                    variable_ref = ExpressionTargetTempVariableRef(
-                        variable   = keeper_variable,
-                        source_ref = source_ref
-                    ),
-                    tolerant     = True,
-                    source_ref   = source_ref,
+                StatementReleaseVariable(
+                    variable   = keeper_variable,
+                    tolerant   = True,
+                    source_ref = source_ref,
                 )
             )
 
             right = makeTryFinallyExpression(
-                tried       = tried,
-                final       = None,
-                expression  = ExpressionTempVariableRef(
+                tried      = tried,
+                final      = None,
+                expression = ExpressionTempVariableRef(
                     variable   = keeper_variable,
                     source_ref = source_ref
                 ),
-                source_ref  = source_ref
+                source_ref = source_ref
             )
 
         comparator = getKind(comparator)
