@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -17,15 +17,14 @@
 #
 """ Reformulation of lambda expressions.
 
-Consult the developmer manual for information. TODO: Add ability to sync
+Consult the developer manual for information. TODO: Add ability to sync
 source code comments with developer manual sections.
 
 """
 
-from nuitka import Utils
 from nuitka.nodes.AssignNodes import (
     StatementAssignmentVariable,
-    StatementDelVariable
+    StatementReleaseVariable
 )
 from nuitka.nodes.ComparisonNodes import ExpressionComparisonIsNOT
 from nuitka.nodes.ConditionalNodes import StatementConditional
@@ -42,6 +41,7 @@ from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef
 )
 from nuitka.nodes.YieldNodes import ExpressionYield
+from nuitka.utils import Utils
 
 from .Helpers import (
     buildNode,
@@ -59,9 +59,9 @@ from .ReformulationFunctionStatements import (
 
 
 def buildLambdaNode(provider, node, source_ref):
-    assert getKind( node ) == "Lambda"
+    assert getKind(node) == "Lambda"
 
-    parameters = buildParameterSpec( "<lambda>", node, source_ref )
+    parameters = buildParameterSpec(provider, "<lambda>", node, source_ref)
 
     function_body = ExpressionFunctionBody(
         provider   = provider,
@@ -95,20 +95,16 @@ def buildLambdaNode(provider, node, source_ref):
             statements = (
                 StatementAssignmentVariable(
                     variable_ref = ExpressionTargetTempVariableRef(
-                        variable = tmp_return_value.makeReference(
-                            function_body
-                        ),
+                        variable   = tmp_return_value,
                         source_ref = source_ref,
                     ),
-                    source     = body,
-                    source_ref = source_ref
+                    source       = body,
+                    source_ref   = source_ref
                 ),
                 StatementConditional(
-                    condition = ExpressionComparisonIsNOT(
+                    condition  = ExpressionComparisonIsNOT(
                         left       = ExpressionTempVariableRef(
-                            variable = tmp_return_value.makeReference(
-                                function_body
-                            ),
+                            variable   = tmp_return_value,
                             source_ref = source_ref,
                         ),
                         right      = ExpressionConstantRef(
@@ -121,9 +117,7 @@ def buildLambdaNode(provider, node, source_ref):
                         statement = StatementExpressionOnly(
                             expression = ExpressionYield(
                                 expression = ExpressionTempVariableRef(
-                                    variable = tmp_return_value.makeReference(
-                                        function_body
-                                    ),
+                                    variable   = tmp_return_value,
                                     source_ref = source_ref,
                                 ),
                                 source_ref = source_ref
@@ -137,13 +131,8 @@ def buildLambdaNode(provider, node, source_ref):
             )
             body = makeTryFinallyStatement(
                 tried      = statements,
-                final      = StatementDelVariable(
-                    variable_ref = ExpressionTargetTempVariableRef(
-                        variable = tmp_return_value.makeReference(
-                            function_body
-                        ),
-                        source_ref = source_ref,
-                    ),
+                final      = StatementReleaseVariable(
+                    variable   = tmp_return_value,
                     tolerant   = True,
                     source_ref = source_ref
                 ),
@@ -174,6 +163,11 @@ def buildLambdaNode(provider, node, source_ref):
         source_ref    = body.getSourceReference()
     )
 
+
+    body = makeStatementsSequenceFromStatement(
+        statement = body,
+    )
+
     function_body.setBody(body)
 
     annotations = buildParameterAnnotations(provider, node, source_ref)
