@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -15,37 +15,43 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
+""" Reformulation of loop statements.
 
+Consult the developer manual for information. TODO: Add ability to sync
+source code comments with developer manual sections.
+
+"""
+
+from nuitka.nodes.AssignNodes import (
+    StatementAssignmentVariable,
+    StatementReleaseVariable
+)
+from nuitka.nodes.BuiltinIteratorNodes import (
+    ExpressionBuiltinIter1,
+    ExpressionBuiltinNext1
+)
+from nuitka.nodes.ComparisonNodes import ExpressionComparisonIs
+from nuitka.nodes.ConditionalNodes import StatementConditional
+from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.LoopNodes import StatementBreakLoop, StatementLoop
+from nuitka.nodes.OperatorNodes import ExpressionOperationNOT
+from nuitka.nodes.StatementNodes import StatementsSequence
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTargetTempVariableRef,
     ExpressionTempVariableRef
 )
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
-
-from nuitka.nodes.BuiltinIteratorNodes import (
-    ExpressionBuiltinNext1,
-    ExpressionBuiltinIter1
-)
-from nuitka.nodes.ComparisonNodes import ExpressionComparisonIs
-from nuitka.nodes.StatementNodes import StatementsSequence
-from nuitka.nodes.LoopNodes import (
-    StatementBreakLoop,
-    StatementLoop
-)
-from nuitka.nodes.ConditionalNodes import StatementConditional
-from nuitka.nodes.TryNodes import StatementTryFinally
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementDelVariable
-)
 
 from .Helpers import (
-    makeStatementsSequenceFromStatement,
-    makeStatementsSequence,
+    buildNode,
     buildStatementsNode,
-    buildNode
+    makeStatementsSequence,
+    makeStatementsSequenceFromStatement,
+    makeTryFinallyStatement,
+    popBuildContext,
+    popIndicatorVariable,
+    pushBuildContext,
+    pushIndicatorVariable
 )
-
 from .ReformulationAssignmentStatements import buildAssignmentStatements
 from .ReformulationTryExceptStatements import makeTryExceptSingleHandlerNode
 
@@ -56,9 +62,9 @@ def buildForLoopNode(provider, node, source_ref):
     # taken if a for loop exits normally, i.e. because of iterator
     # exhaustion. We do this by introducing an indicator variable.
 
-    source = buildNode( provider, node.iter, source_ref )
+    source = buildNode(provider, node.iter, source_ref)
 
-    temp_scope = provider.allocateTempScope( "for_loop" )
+    temp_scope = provider.allocateTempScope("for_loop")
 
     tmp_iter_variable = provider.allocateTempVariable(
         temp_scope = temp_scope,
@@ -77,7 +83,7 @@ def buildForLoopNode(provider, node, source_ref):
     )
 
     if else_block is not None:
-        tmp_break_indicator_variable = provider.allocateTempVariable(
+        tmp_break_indicator = provider.allocateTempVariable(
             temp_scope = temp_scope,
             name       = "break_indicator"
         )
@@ -85,9 +91,7 @@ def buildForLoopNode(provider, node, source_ref):
         statements = [
             StatementAssignmentVariable(
                 variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = tmp_break_indicator_variable.makeReference(
-                        provider
-                    ),
+                    variable   = tmp_break_indicator,
                     source_ref = source_ref
                 ),
                 source       = ExpressionConstantRef(
@@ -102,7 +106,7 @@ def buildForLoopNode(provider, node, source_ref):
 
     statements.append(
         StatementBreakLoop(
-            source_ref = source_ref.atInternal()
+            source_ref = source_ref
         )
     )
 
@@ -114,19 +118,15 @@ def buildForLoopNode(provider, node, source_ref):
 
     statements = (
         makeTryExceptSingleHandlerNode(
-            tried         = makeStatementsSequenceFromStatement(
+            tried          = makeStatementsSequenceFromStatement(
                 statement = StatementAssignmentVariable(
                     variable_ref = ExpressionTargetTempVariableRef(
-                        variable   = tmp_value_variable.makeReference(
-                            provider
-                        ),
+                        variable   = tmp_value_variable,
                         source_ref = source_ref
                     ),
                     source       = ExpressionBuiltinNext1(
                         value      = ExpressionTempVariableRef(
-                            variable   = tmp_iter_variable.makeReference(
-                                provider
-                            ),
+                            variable   = tmp_iter_variable,
                             source_ref = source_ref
                         ),
                         source_ref = source_ref
@@ -136,21 +136,22 @@ def buildForLoopNode(provider, node, source_ref):
             ),
             exception_name = "StopIteration",
             handler_body   = handler_body,
+            public_exc     = False,
             source_ref     = source_ref
         ),
         buildAssignmentStatements(
             provider   = provider,
             node       = node.target,
             source     = ExpressionTempVariableRef(
-                variable   = tmp_value_variable.makeReference(
-                    provider
-                ),
+                variable   = tmp_value_variable,
                 source_ref = source_ref
             ),
             source_ref = source_ref
         )
     )
 
+    pushBuildContext("loop_body")
+    pushIndicatorVariable(None)
     statements += (
         buildStatementsNode(
             provider   = provider,
@@ -158,6 +159,8 @@ def buildForLoopNode(provider, node, source_ref):
             source_ref = source_ref
         ),
     )
+    popIndicatorVariable()
+    popBuildContext()
 
     loop_body = makeStatementsSequence(
         statements = statements,
@@ -165,17 +168,28 @@ def buildForLoopNode(provider, node, source_ref):
         source_ref = source_ref
     )
 
+    cleanup_statements = [
+        StatementReleaseVariable(
+            variable   = tmp_value_variable,
+            tolerant   = True,
+            source_ref = source_ref
+        ),
+        StatementReleaseVariable(
+            variable   = tmp_iter_variable,
+            tolerant   = True,
+            source_ref = source_ref
+        )
+    ]
+
     if else_block is not None:
         statements = [
             StatementAssignmentVariable(
                 variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = tmp_break_indicator_variable.makeReference(
-                        provider
-                    ),
+                    variable   = tmp_break_indicator,
                     source_ref = source_ref
                 ),
                 source       = ExpressionConstantRef(
-                    constant = False,
+                    constant   = False,
                     source_ref = source_ref
                 ),
                 source_ref   = source_ref
@@ -184,52 +198,29 @@ def buildForLoopNode(provider, node, source_ref):
     else:
         statements = []
 
-    cleanup_statements = (
-        StatementDelVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_value_variable.makeReference( provider ),
-                source_ref = source_ref
-            ),
-            tolerant     = True,
-            source_ref   = source_ref.atInternal()
-        ),
-        StatementDelVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_iter_variable.makeReference( provider ),
-                source_ref = source_ref
-            ),
-            tolerant     = False,
-            source_ref   = source_ref.atInternal()
-        )
-    )
-
     statements += [
         # First create the iterator and store it.
         StatementAssignmentVariable(
             variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_iter_variable.makeReference(
-                    provider
-                ),
+                variable   = tmp_iter_variable,
                 source_ref = source_ref
             ),
             source       = ExpressionBuiltinIter1(
-                value       = source,
-                source_ref  = source.getSourceReference()
+                value      = source,
+                source_ref = source.getSourceReference()
             ),
             source_ref   = source_ref
         ),
-        StatementTryFinally(
-            tried = makeStatementsSequenceFromStatement(
-                statement = StatementLoop(
-                    body       = loop_body,
-                    source_ref = source_ref
-                )
+        makeTryFinallyStatement(
+            tried      = StatementLoop(
+                body       = loop_body,
+                source_ref = source_ref
             ),
-            final = StatementsSequence(
+            final      = StatementsSequence(
                 statements = cleanup_statements,
-                source_ref = source_ref.atInternal()
+                source_ref = source_ref
             ),
-            source_ref = source_ref.atInternal()
+            source_ref = source_ref
         )
     ]
 
@@ -238,7 +229,7 @@ def buildForLoopNode(provider, node, source_ref):
             StatementConditional(
                 condition  = ExpressionComparisonIs(
                     left       = ExpressionTempVariableRef(
-                        variable   = tmp_break_indicator_variable.makeReference( provider ),
+                        variable   = tmp_break_indicator,
                         source_ref = source_ref
                     ),
                     right      = ExpressionConstantRef(
@@ -253,11 +244,24 @@ def buildForLoopNode(provider, node, source_ref):
             )
         ]
 
+        statements = (
+            makeTryFinallyStatement(
+                tried      = statements,
+                final      = StatementReleaseVariable(
+                    variable   = tmp_break_indicator,
+                    tolerant   = False,
+                    source_ref = source_ref
+                ),
+                source_ref = source_ref
+            ),
+        )
+
     return StatementsSequence(
         statements = statements,
         source_ref = source_ref
     )
 
+
 def buildWhileLoopNode(provider, node, source_ref):
     # The while loop is re-formulated according to developer manual. The
     # condition becomes an early condition to break the loop. The else block is
@@ -270,11 +274,10 @@ def buildWhileLoopNode(provider, node, source_ref):
         source_ref = source_ref
     )
 
-
     if else_block is not None:
-        temp_scope = provider.allocateTempScope( "while_loop" )
+        temp_scope = provider.allocateTempScope("while_loop")
 
-        tmp_break_indicator_variable = provider.allocateTempVariable(
+        tmp_break_indicator = provider.allocateTempVariable(
             temp_scope = temp_scope,
             name       = "break_indicator"
         )
@@ -282,16 +285,14 @@ def buildWhileLoopNode(provider, node, source_ref):
         statements = (
             StatementAssignmentVariable(
                 variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = tmp_break_indicator_variable.makeReference(
-                        provider
-                    ),
+                    variable   = tmp_break_indicator,
                     source_ref = source_ref
                 ),
-                source     = ExpressionConstantRef(
+                source       = ExpressionConstantRef(
                     constant   = True,
                     source_ref = source_ref
                 ),
-                source_ref = source_ref
+                source_ref   = source_ref
             ),
             StatementBreakLoop(
                 source_ref = source_ref
@@ -304,24 +305,33 @@ def buildWhileLoopNode(provider, node, source_ref):
             ),
         )
 
+    pushBuildContext("loop_body")
+    pushIndicatorVariable(None)
+    loop_statements = buildStatementsNode(
+        provider   = provider,
+        nodes      = node.body,
+        source_ref = source_ref
+    )
+    popIndicatorVariable()
+    popBuildContext()
+
     # The loop body contains a conditional statement at the start that breaks
     # the loop if it fails.
     loop_body = makeStatementsSequence(
         statements = (
             StatementConditional(
-                condition = buildNode( provider, node.test, source_ref ),
-                no_branch = StatementsSequence(
+                condition  = ExpressionOperationNOT(
+                    operand    = buildNode(provider, node.test, source_ref),
+                    source_ref = source_ref,
+                ),
+                yes_branch = StatementsSequence(
                     statements = statements,
                     source_ref = source_ref
                 ),
-                yes_branch = None,
+                no_branch  = None,
                 source_ref = source_ref
             ),
-            buildStatementsNode(
-                provider   = provider,
-                nodes      = node.body,
-                source_ref = source_ref
-            )
+            loop_statements
         ),
         allow_none = True,
         source_ref = source_ref
@@ -338,12 +348,10 @@ def buildWhileLoopNode(provider, node, source_ref):
         statements = (
             StatementAssignmentVariable(
                 variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = tmp_break_indicator_variable.makeReference(
-                        provider
-                    ),
+                    variable   = tmp_break_indicator,
                     source_ref = source_ref
                 ),
-                source = ExpressionConstantRef(
+                source       = ExpressionConstantRef(
                     constant   = False,
                     source_ref = source_ref
                 ),
@@ -353,9 +361,7 @@ def buildWhileLoopNode(provider, node, source_ref):
             StatementConditional(
                 condition  = ExpressionComparisonIs(
                     left       = ExpressionTempVariableRef(
-                        variable   = tmp_break_indicator_variable.makeReference(
-                            provider
-                        ),
+                        variable   = tmp_break_indicator,
                         source_ref = source_ref
                     ),
                     right      = ExpressionConstantRef(
@@ -370,6 +376,18 @@ def buildWhileLoopNode(provider, node, source_ref):
             )
         )
 
+        statements = (
+            makeTryFinallyStatement(
+                tried      = statements,
+                final      = StatementReleaseVariable(
+                    variable   = tmp_break_indicator,
+                    tolerant   = False,
+                    source_ref = source_ref
+                ),
+                source_ref = source_ref
+            ),
+        )
+
         return StatementsSequence(
             statements = statements,
             source_ref = source_ref
