@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,7 +24,7 @@ source code comments with developer manual sections.
 
 from nuitka.nodes.AssignNodes import (
     StatementAssignmentVariable,
-    StatementDelVariable
+    StatementReleaseVariable
 )
 from nuitka.nodes.BuiltinTypeNodes import ExpressionBuiltinStr
 from nuitka.nodes.ComparisonNodes import ExpressionComparisonIs
@@ -51,15 +51,18 @@ def buildPrintNode(provider, node, source_ref):
 
     def wrapValue(value):
         if value.isExpressionConstantRef():
-            return value.getStrValue()
-        else:
-            return ExpressionBuiltinStr(
-                value      = value,
-                source_ref = value.getSourceReference()
-            )
+            str_value = value.getStrValue()
+
+            if str_value is not None:
+                return str_value
+
+        return ExpressionBuiltinStr(
+            value      = value,
+            source_ref = value.getSourceReference()
+        )
 
     if node.dest is not None:
-        temp_scope = provider.allocateTempScope( "print" )
+        temp_scope = provider.allocateTempScope("print")
 
         tmp_target_variable = provider.allocateTempVariable(
             temp_scope = temp_scope,
@@ -117,6 +120,7 @@ def buildPrintNode(provider, node, source_ref):
         nodes      = node.values,
         source_ref = source_ref
     )
+
     values = [
         wrapValue(value)
         for value in
@@ -150,13 +154,10 @@ def buildPrintNode(provider, node, source_ref):
         statements.append(
             makeTryFinallyStatement(
                 tried      = print_statements,
-                final      = StatementDelVariable(
-                    variable_ref = ExpressionTargetTempVariableRef(
-                        variable   = tmp_target_variable,
-                        source_ref = source_ref
-                    ),
-                    tolerant     = False,
-                    source_ref   = source_ref
+                final      = StatementReleaseVariable(
+                    variable   = tmp_target_variable,
+                    tolerant   = False,
+                    source_ref = source_ref
                 ),
                 source_ref = source_ref
             )
