@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,10 +22,10 @@ source code comments with developer manual sections.
 
 """
 
-from nuitka import Options, SyntaxErrors, Utils
+from nuitka import Options
 from nuitka.nodes.AssignNodes import (
     StatementAssignmentVariable,
-    StatementDelVariable
+    StatementReleaseVariable
 )
 from nuitka.nodes.BuiltinRefNodes import ExpressionBuiltinExceptionRef
 from nuitka.nodes.ComparisonNodes import (
@@ -45,11 +45,13 @@ from nuitka.nodes.StatementNodes import (
     StatementRestoreFrameException,
     StatementsSequence
 )
-from nuitka.nodes.TryNodes import StatementTryExcept
+from nuitka.nodes.TryExceptNodes import StatementTryExcept
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTargetTempVariableRef,
     ExpressionTempVariableRef
 )
+from nuitka.tree import SyntaxErrors
+from nuitka.utils import Utils
 
 from .Helpers import (
     buildNode,
@@ -112,11 +114,11 @@ def makeTryExceptNoRaise(provider, temp_scope, tried, handling, no_raise,
         ),
         StatementConditional(
             condition  = ExpressionComparisonIs(
-                left = ExpressionTempVariableRef(
+                left       = ExpressionTempVariableRef(
                     variable   = tmp_handler_indicator_variable,
                     source_ref = source_ref
                 ),
-                right = ExpressionConstantRef(
+                right      = ExpressionConstantRef(
                     constant   = True,
                     source_ref = source_ref
                 ),
@@ -128,11 +130,8 @@ def makeTryExceptNoRaise(provider, temp_scope, tried, handling, no_raise,
         )
     )
 
-    final = StatementDelVariable(
-        variable_ref = ExpressionTargetTempVariableRef(
-            variable   = tmp_handler_indicator_variable,
-            source_ref = source_ref.atInternal()
-        ),
+    final = StatementReleaseVariable(
+        variable   = tmp_handler_indicator_variable,
         tolerant   = False,
         source_ref = source_ref.atInternal()
     ),
@@ -171,7 +170,7 @@ def makeReraiseExceptionStatement(source_ref):
                 source_ref      = source_ref
             ),
         ),
-        source_ref  = source_ref
+        source_ref = source_ref
     )
 
 
@@ -191,11 +190,11 @@ def makeTryExceptSingleHandlerNode(tried, exception_name, handler_body,
 
     statements.append(
         StatementConditional(
-            condition = ExpressionComparisonExceptionMatch(
-                left      = ExpressionCaughtExceptionTypeRef(
-                    source_ref  = source_ref
+            condition  = ExpressionComparisonExceptionMatch(
+                left       = ExpressionCaughtExceptionTypeRef(
+                    source_ref = source_ref
                 ),
-                right     = ExpressionBuiltinExceptionRef(
+                right      = ExpressionBuiltinExceptionRef(
                     exception_name = exception_name,
                     source_ref     = source_ref
                 ),
@@ -237,8 +236,8 @@ def buildTryExceptionNode(provider, node, source_ref):
     # for the "del" as done for Python3. Also catches always work a tuple of
     # exception types and hides away that they may be built or not.
 
-    # Many variables, due to the re-formulation that is going on here, which
-    # just has the complexity, pylint: disable=R0914
+    # Many variables and branches, due to the re-formulation that is going on
+    # here, which just has the complexity, pylint: disable=R0912,R0914
 
     tried = buildStatementsNode(
         provider   = provider,
@@ -286,6 +285,8 @@ def buildTryExceptionNode(provider, node, source_ref):
                 source_ref = source_ref,
             )
 
+            # We didn't allow None, therefore it cannot be None, and
+            # the unpack is safe: pylint: disable=W0633
             kind, detail = target_info
 
             assert kind == "Name", kind
@@ -332,7 +333,7 @@ def buildTryExceptionNode(provider, node, source_ref):
         if exception_types is None:
             if handler is not node.handlers[-1]:
                 SyntaxErrors.raiseSyntaxError(
-                    reason    = "default 'except:' must be last",
+                    reason     = "default 'except:' must be last",
                     source_ref = source_ref.atLineNumber(
                         handler.lineno-1
                           if Options.isFullCompat() else
@@ -349,7 +350,7 @@ def buildTryExceptionNode(provider, node, source_ref):
 
     # Reraise by default
     exception_handling = makeReraiseExceptionStatement(
-        source_ref  = source_ref
+        source_ref = source_ref
     )
 
     for exception_type, handler in reversed(handlers):
@@ -360,9 +361,9 @@ def buildTryExceptionNode(provider, node, source_ref):
             exception_handling = StatementsSequence(
                 statements = (
                     StatementConditional(
-                        condition = ExpressionComparisonExceptionMatch(
+                        condition  = ExpressionComparisonExceptionMatch(
                             left       = ExpressionCaughtExceptionTypeRef(
-                                source_ref  = exception_type.source_ref
+                                source_ref = exception_type.source_ref
                             ),
                             right      = exception_type,
                             source_ref = exception_type.source_ref
@@ -408,8 +409,8 @@ def buildTryExceptionNode(provider, node, source_ref):
             exception_handling = StatementsSequence(
                 statements = prelude + (
                     makeTryFinallyStatement(
-                        tried = exception_handling,
-                        final = StatementRestoreFrameException(
+                        tried      = exception_handling,
+                        final      = StatementRestoreFrameException(
                             source_ref = source_ref.atInternal()
                         ),
                         source_ref = source_ref
