@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -15,12 +15,16 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
+from __future__ import print_function
 
-from __builtin__ import len as _len
+try:
+    from __builtin__ import len as _len
+except ImportError:
+    from builtins import len as _len
 
 def len(x):
-    print x
+    print("Private built-in called with argument", repr(x))
 
     return _len(x)
 
-print len(range(9))
+print("Calling built-in len", len(range(9)))
