@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -31,7 +31,7 @@ def functionEval2():
 
     g = {}
 
-    r = eval( "1+3", g )
+    r = eval("1+3", g)
 
     return r, g.keys()
 
@@ -40,8 +40,8 @@ print "eval in a function with globals provided", functionEval2()
 def functionEval3():
     result = []
 
-    for x in eval( "(1,2)"):
-        result.append( x )
+    for x in eval("(1,2)"):
+        result.append(x)
 
     return result
 
@@ -71,22 +71,22 @@ print "exec in function without and with locals() provided:", functionExec1(), f
 
 tmp_filename = tempfile.gettempdir() + "/execfile.py"
 
-f = open( tmp_filename, "w" )
-f.write( "e=7\nf=8\n" )
+f = open(tmp_filename, 'w')
+f.write("e=7\nf=8\n")
 f.close()
 
-execfile( tmp_filename )
+execfile(tmp_filename)
 
 print "execfile with defaults f,g=", e, f
 
 global_vars = { 'e' : '0', 'f' : 0 }
-local_vars = dict( global_vars )
+local_vars = dict(global_vars)
 
-execfile( tmp_filename, global_vars )
+execfile(tmp_filename, global_vars)
 
 print "execfile with globals dict:", global_vars.keys()
 
-execfile( tmp_filename, global_vars, local_vars )
+execfile(tmp_filename, global_vars, local_vars)
 
 print "execfile with globals and locals dict:", local_vars
 
@@ -95,10 +95,10 @@ def functionExecfile():
     f = 0
 
     global_vars = { 'e' : '0', 'f' : 0 }
-    local_vars = dict( global_vars )
+    local_vars = dict(global_vars)
 
     print "execfile with globals and locals dict in a function:",
-    x = execfile( tmp_filename, global_vars, local_vars )
+    x = execfile(tmp_filename, global_vars, local_vars)
     print x,
     print global_vars.keys(), local_vars, e, f
 
@@ -111,55 +111,67 @@ class classExecfile:
     print "execfile in a class:",
     # TODO: Won't work yet, Issue#5
     # print execfile( tmp_filename ),
-    execfile( tmp_filename )
-    print e, f
+    execfile(tmp_filename)
+    print "execfile changed local values:", e, f
 
 
-def functionExecNones():
+f = 7
+
+def functionExecNonesTuple():
     f = 0
 
-    exec( "f=1", None, None )
-
-    print "Exec with None as tuple args did update locals:", f
+    exec("f=1", None, None)
+    print "Exec with None as optimizable tuple args did update locals:", f
 
+def functionExecNonesSyntax():
+    f = 0
     exec "f=2" in None, None
+    print "Exec with None as optimizable normal args did update locals:", f
 
-    print "Exec with None as normal args did update locals:", f
+functionExecNonesTuple()
+functionExecNonesSyntax()
 
-functionExecNones()
+print "Global value is untouched", f
 
 def functionEvalNones2():
     f = 11
 
-    code = "f"
+    code = 'f'
     g = None
     l = None
 
-    f1 = eval ( code, l, g )
+    f1 = eval (code, l, g)
 
     print "Eval with None arguments from variables did access locals:", f1
 
 
 functionEvalNones2()
 
-def functionExecNones2():
+def functionExecNonesTuple2():
     f = 0
 
     code = "f=1"
     g = None
     l = None
 
-    exec ( code, l, g )
+    exec(code, l, g)
 
     print "Exec with None as tuple args from variable did update locals:", f
 
+def functionExecNonesSyntax2():
+    f = 0
+
     code = "f=2"
+    g = None
+    l = None
 
     exec code in l, g
 
     print "Exec with None as normal args did update locals:", f
 
-functionExecNones2()
+
+functionExecNonesTuple2()
+functionExecNonesSyntax2()
 
 print "Exec with a future division definition and one without:"
 
@@ -190,7 +202,7 @@ from __future__ import print_function
 x = 2
 print( "Exec local x is", x )
 """
-    print "Function global x is", x
+    print "Function global x referenced as local x in exec is", x
 
     exec """
 from __future__ import print_function
@@ -202,9 +214,9 @@ print( "Re-exec local x", x )
 from __future__ import print_function
 global x
 x = 3
-print( "Exec global x is", x )
+print( "Exec global x is inside exec", x )
 """
-    print "Exec level global x is", x
+    print "Global x referenced as global x in exec is", x
 
     exec """
 from __future__ import print_function
@@ -249,10 +261,9 @@ functionWithClosureProvidedByExec()
 x = 2
 
 def functionWithExecAffectingClosure():
-
     x = 4
 
-    code = "d=3"
+    code = "d=3;x=5"
     space = locals()
 
     exec code in space
@@ -272,12 +283,12 @@ def generatorFunctionWithExec():
 
     yield y
 
-print "Exec in a generator function", tuple( generatorFunctionWithExec() )
+print "Exec in a generator function", tuple(generatorFunctionWithExec())
 
 def evalInContractions():
 
-    r1 = list( eval( str( s ) ) for s in range( 3 ) )
-    r2 = [ eval( str( s ) ) for s in range( 4 ) ]
+    r1 = list( eval(str(s)) for s in range(3) )
+    r2 = [ eval(str(s)) for s in range(4) ]
 
     return r1, r2
 
@@ -298,4 +309,13 @@ def makeAddPair(a, b):
 
 print "Exec adds functions declares in explicit locals() given.", execDefinesFunctionToLocalsExplicity()
 
-os.unlink( tmp_filename )
+os.unlink(tmp_filename)
+
+
+def execWithShortTuple():
+    try:
+        exec("print hey",)
+    except Exception as e:
+        return "gives exception: " + repr(e)
+
+print "Exec with too short tuple argument:", execWithShortTuple()
