@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -63,4 +63,15 @@ def localImportFailure():
 
     print "but path was", path
 
-print "From import that fails in the middle", localImportFailure()
+print "From import that fails in the middle",
+localImportFailure()
+
+def nonPackageImportFailure():
+    try:
+        # Not allowed without being a package, should raise ValueError
+        from . import whatever
+    except Exception as e:
+        print type(e), repr(e)
+
+print "Package import fails in non-package:",
+nonPackageImportFailure()
