@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -22,7 +22,7 @@ With newline and stuff."""
 import os, sys
 
 print "doc:", __doc__
-print "filename:", __file__
+print "filename:", os.path.basename(__file__)
 print "builtins:", __builtins__
 print "debug", __debug__
 print "debug in builtins", __builtins__.__debug__
@@ -35,17 +35,20 @@ except NameError:
 
 def checkFromFunction():
     frame = sys._getframe(1)
-    locals = frame.f_locals
 
     def displayDict(d):
-        d = dict(d)
         if "__loader__" in d:
-            d[ "__loader__" ] = "<loader removed>"
+            d = dict(d)
+            d["__loader__"] = "<__loader__ removed>"
 
-        return repr( d )
+        if "__file__" in d:
+            d = dict(d)
+            d["__file__"] = "<__file__ removed>"
 
-    print "Globals", displayDict( frame.f_globals )
-    print "Locals", displayDict( frame.f_locals )
+        return repr(d)
+
+    print "Globals", displayDict(frame.f_globals)
+    print "Locals", displayDict(frame.f_locals)
     print "Is identical", frame.f_locals is frame.f_globals
 
 checkFromFunction()
