@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -17,5 +17,5 @@
 #
 from __future__ import print_function
 
-print( "hallo welt", end = "," )
-print( "this is the end" )
+print("hallo welt", end = ',' )
+print("this is the end")
