@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -76,8 +76,41 @@ def simpleFunction2():
     except TypeError:
         pass
 
+# Broken iterator class.
+class Broken:
+    def __iter__(self):
+        return self
+    def __next__(self):
+        return 1
+    def __getattr__(self, attr):
+
+        1/0
+
+def simpleFunction3():
+    def g():
+        yield from Broken()
+
+    try:
+        gi = g()
+        next(gi)
+    except Exception:
+        pass
+
+def simpleFunction4():
+    def g():
+        yield from Broken()
+
+    try:
+        gi = g()
+        next(gi)
+        gi.throw(AttributeError)
+    except Exception:
+        pass
+
+
+
 # These need stderr to be wrapped.
-tests_stderr = ()
+tests_stderr = (3, 4)
 
 # Disabled tests
 tests_skipped = {}
@@ -89,4 +122,4 @@ result = executeReferenceChecked(
     tests_stderr  = tests_stderr
 )
 
-sys.exit(0 if result else 1)
\ No newline at end of file
+sys.exit(0 if result else 1)
