@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -20,7 +20,7 @@ class W1(object):
         self.__hidden = 5
 
 class W2(object):
-    __slots__=['__hidden']
+    __slots__=["__hidden"]
     def __init__(self):
         self.__hidden = 5
 
@@ -29,7 +29,7 @@ class _W1(object):
         self.__hidden = 5
 
 class _W2(object):
-    __slots__=['__hidden']
+    __slots__=["__hidden"]
     def __init__(self):
         self.__hidden = 5
 
@@ -38,7 +38,7 @@ class a_W1(object):
         self.__hidden = 5
 
 class a_W2(object):
-    __slots__=['__hidden']
+    __slots__=["__hidden"]
     def __init__(self):
         self.__hidden = 5
 
@@ -47,7 +47,7 @@ class W1_(object):
         self.__hidden = 5
 
 class W2_(object):
-    __slots__=['__hidden']
+    __slots__=["__hidden"]
     def __init__(self):
         self.__hidden = 5
 
@@ -57,4 +57,4 @@ for w in (W1, W2, _W1, _W2, a_W1, a_W2, W1_, W2_):
         print(dir(w))
         a = w()
     except AttributeError:
-        print( 'bug in %s' % w )
+        print( "bug in %s" % w )
