@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -34,56 +34,56 @@ def tryScope1(x):
             finally:
                 print("Deep Nested finally is executed")
     except:
-        print("Exception occured")
+        print("Exception occurred")
     else:
-        print("No exception occured")
+        print("No exception occurred")
 
 tryScope1(1)
-print("*" * 20)
+print('*' * 20)
 tryScope1([1])
 
 def tryScope2(x, someExceptionClass):
     try:
         x += 1
     except someExceptionClass as e:
-        print("Exception class from argument occured:", someExceptionClass, repr(e))
+        print("Exception class from argument occurred:", someExceptionClass, repr(e))
     else:
-        print("No exception occured")
+        print("No exception occurred")
 
 def tryScope3(x):
     if x:
         try:
             x += 1
         except TypeError:
-            print("TypeError occured")
+            print("TypeError occurred")
     else:
         print("Not taken")
 
 
-print("*" * 20)
+print('*' * 20)
 
 tryScope2(1, TypeError)
-tryScope2([ 1 ], TypeError)
+tryScope2([1], TypeError)
 
-print("*" * 20)
+print('*' * 20)
 
 tryScope3(1)
 tryScope3([1])
 tryScope3([])
 
-print("*" * 20)
+print('*' * 20)
 
 def tryScope4(x):
     try:
         x += 1
     except:
-        print("exception occured")
+        print("exception occurred")
     else:
-        print("no exception occured")
+        print("no exception occurred")
     finally:
         print("finally obeyed")
 
-tryScope4(1 )
+tryScope4(1)
 tryScope4([1])
 
 def tryScope5():
