@@ -1,4 +1,4 @@
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Softwar where
@@ -18,6 +18,10 @@
 #     limitations under the License.
 #
 
-print( (1).imag )
-print( int.__name__ )
-print( (1).__class__ )
+print((1).imag)
+print(int.__name__)
+print((1).__class__)
+
+print(getattr(1, "real"))
+print(getattr(1, "real", None))
+print(hasattr(1, "real"))
