@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Softwar where
@@ -19,8 +19,9 @@
 #     limitations under the License.
 #
 
-import os, sys, subprocess, tempfile, shutil
+import os, sys
 
+# The test runner needs "lxml" itself.
 try:
     import lxml.etree
 except ImportError:
@@ -32,7 +33,7 @@ sys.path.insert(
     0,
     os.path.normpath(
         os.path.join(
-            os.path.dirname( os.path.abspath( __file__ ) ),
+            os.path.dirname(os.path.abspath(__file__)),
             ".."
         )
     )
@@ -41,35 +42,30 @@ from test_common import (
     my_print,
     setup,
     convertUsing2to3,
+    createSearchMode,
     hasModule,
     check_output
 )
 
 python_version = setup()
 
+# The Python version used by Nuitka needs "lxml" too.
 if not hasModule("lxml.etree"):
     print("Warning, no 'lxml' module installed, cannot run XML based tests.")
     sys.exit(0)
 
-search_mode = len( sys.argv ) > 1 and sys.argv[1] == "search"
+search_mode = createSearchMode()
 
-start_at = sys.argv[2] if len( sys.argv ) > 2 else None
-
-if start_at:
-    active = False
-else:
-    active = True
-
-def getKind( node ):
+def getKind(node):
     result = node.attrib[ "kind" ]
 
-    result = result.replace( "Statements", "" )
-    result = result.replace( "Statement", "" )
-    result = result.replace( "Expression", "" )
+    result = result.replace("Statements", "")
+    result = result.replace("Statement", "")
+    result = result.replace("Expression", "")
 
     return result
 
-def getRole( node, role ):
+def getRole(node, role):
     for child in node:
         if child.tag == "role" and child.attrib["name"] == role:
             return child
@@ -90,7 +86,8 @@ def checkSequence(statements):
         if kind == "PrintValue":
             print_arg, = getRole(statement, "value")
 
-            if getKind(print_arg) != "ConstantRef":
+            if getKind(print_arg) not in ("ConstantRef",
+                                          "ModuleFileAttributeRef"):
                 sys.exit(
                     "%s: Error, print of non-constant %s." % (
                         getSourceRef(statement),
@@ -107,7 +104,7 @@ def checkSequence(statements):
         if kind == "Only":
             only_expression = getRole(statement, "expression")[0]
 
-            if getKind( only_expression ) == "Call":
+            if getKind(only_expression) == "CallNoKeywords":
                 called_expression = getRole(only_expression, "called")[0]
 
                 if getKind(called_expression) == "BuiltinRef":
@@ -120,12 +117,14 @@ def checkSequence(statements):
             continue
 
         if kind == "AssignmentVariable":
-            assign_source, = getRole( statement, "source" )
+            assign_source, = getRole(statement, "source")
 
             source_kind = getKind(assign_source)
 
-            if source_kind not in("ConstantRef", "ImportModuleHard"):
-                sys.exit( "Error, assignment from of non-constant %s." % source_kind )
+            if source_kind not in("ConstantRef",
+                                  "ImportModuleHard",
+                                  "ModuleFileAttributeRef"):
+                sys.exit("Error, assignment from of non-constant %s." % source_kind)
             continue
 
         print(lxml.etree.tostring(statement, pretty_print = True))
@@ -133,56 +132,69 @@ def checkSequence(statements):
         sys.exit("Error, non-print statement of unknown kind '%s'." % kind)
 
 
-for filename in sorted(os.listdir( ".")):
-    if not filename.endswith(".py" ) or filename.startswith("run_"):
+for filename in sorted(os.listdir('.')):
+    if not filename.endswith(".py") or filename.startswith("run_"):
         continue
 
-    # Skip tests that require Python 2.7 at least.
-    if filename.endswith("27.py") and python_version.startswith("2.6"):
-        continue
-
-    path = filename
-
-    if not active and start_at in (filename, path):
-        active = True
-
+    active = search_mode.consider(
+        dirname  = None,
+        filename = filename
+    )
 
     extra_flags = ["expect_success"]
 
     if active:
         # Apply 2to3 conversion if necessary.
-        assert type( python_version ) is bytes
+        if python_version.startswith('3'):
+            filename, changed = convertUsing2to3(filename)
+        else:
+            changed = False
 
-        if python_version.startswith("3"):
-            path = convertUsing2to3( path )
-
-        my_print( "Consider", path, end = " " )
+        my_print("Consider", filename, end = ' ')
 
         command = [
-            os.environ[ "PYTHON" ],
-            os.path.join( "..", "..", "bin", "nuitka" ),
+            os.environ["PYTHON"],
+            os.path.abspath(os.path.join("..", "..", "bin", "nuitka")),
             "--dump-xml",
             "--module",
-            path
+            filename
         ]
 
+
+        if search_mode.isCoverage():
+            # To avoid re-execution, which is not acceptable to coverage.
+            if "PYTHONHASHSEED" not in os.environ:
+                os.environ["PYTHONHASHSEED"] = '0'
+
+            command.insert(2, "--must-not-re-execute")
+
+            command = command[0:1] + [
+                "-m",
+                "coverage",
+                "run",
+                "--rcfile", os.devnull,
+                "-a",
+            ] + command[1:]
+
         result = check_output(
             command
         )
 
         # Parse the result into XML and check it.
-        root = lxml.etree.fromstring( result )
-        module_body  = root[0]
-        module_statements_sequence = module_body[ 0 ]
+        root = lxml.etree.fromstring(result)
+        module_body = root[0]
+        module_statements_sequence = module_body[0]
 
-        assert len( module_statements_sequence ) == 1
-        module_statements = iter( module_statements_sequence ).next()
+        assert len(module_statements_sequence) == 1
+        module_statements = next(iter(module_statements_sequence))
 
-        checkSequence( module_statements )
+        checkSequence(module_statements)
 
-        if python_version.startswith("3"):
-            os.unlink( path )
+        if changed:
+            os.unlink(filename)
 
         my_print("OK.")
     else:
         my_print("Skipping", filename)
+
+search_mode.finish()
