@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Softwar where
@@ -26,7 +26,7 @@ sys.path.insert(
     0,
     os.path.normpath(
         os.path.join(
-            os.path.dirname( os.path.abspath( __file__ ) ),
+            os.path.dirname(os.path.abspath(__file__)),
             ".."
         )
     )
@@ -35,30 +35,19 @@ from test_common import (
     my_print,
     setup,
     convertUsing2to3,
+    createSearchMode,
     compareWithCPython,
     getTempDir
 )
 
 python_version = setup()
 
-search_mode = len( sys.argv ) > 1 and sys.argv[1] == "search"
+search_mode = createSearchMode()
 
-start_at = sys.argv[2] if len( sys.argv ) > 2 else None
-
-if start_at:
-    active = False
-else:
-    active = True
-
-for filename in sorted( os.listdir( "." ) ):
-    if not os.path.isdir( filename ) or filename.endswith( ".build" ):
+for filename in sorted(os.listdir(".")):
+    if not os.path.isdir(filename) or filename.endswith(".build"):
         continue
 
-    path = os.path.relpath( filename )
-
-    if not active and start_at in ( filename, path ):
-        active = True
-
     extra_flags = [
         "expect_success",
         "remove_output",
@@ -68,16 +57,21 @@ for filename in sorted( os.listdir( "." ) ):
 
     # The use of "__main__" in the test package gives a warning.
     if filename == "sub_package":
-        extra_flags.append( "ignore_warnings" )
+        extra_flags.append("ignore_warnings")
+
+    active = search_mode.consider(
+        dirname  = None,
+        filename = filename
+    )
 
     if active:
-        my_print( "Consider output of recursively compiled program:", path )
+        my_print("Consider output of recursively compiled program:", filename)
 
-        for filename_main in os.listdir( filename ):
+        for filename_main in os.listdir(filename):
             if not os.path.isdir(os.path.join(filename,filename_main)):
                 continue
 
-            if filename_main not in ( "..", "." ):
+            if filename_main not in ("..", "."):
                 break
         else:
             sys.exit(
@@ -93,10 +87,13 @@ Error, no package in dir '%s' found, incomplete test case.""" % filename
 
 
         compareWithCPython(
-            path        = os.path.join( filename, filename_main ),
+            dirname     = filename,
+            filename    = filename_main,
             extra_flags = extra_flags,
             search_mode = search_mode,
             needs_2to3  = False
         )
     else:
-        my_print( "Skipping", filename )
+        my_print("Skipping", filename)
+
+search_mode.finish()
\ No newline at end of file
