@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -15,15 +15,18 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
+from __future__ import print_function
+
+import os
 
 try:
     from SyntaxErroring import x
 except Exception as e:
-    print "Importing with syntax error gave", type(e), e
+    print("Importing with syntax error gave:", type(e), e)
 
 try:
     from IndentationErroring import x
 except Exception as e:
-    print "Importing with indentation error gave", type(e), e
-
+    print("Importing with indentation error gave:", type(e), e)
 
+print("Finished.")
