@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2014, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -32,43 +32,38 @@ sys.path.insert(
 from test_common import (
     my_print,
     setup,
-    compareWithCPython
+    compareWithCPython,
+    createSearchMode,
+    decideFilenameVersionSkip
 )
 
-python_version = setup()
+python_version = setup(needs_io_encoding = True)
 
-search_mode = len( sys.argv ) > 1 and sys.argv[1] == "search"
-
-start_at = sys.argv[2] if len( sys.argv ) > 2 else None
-
-if start_at:
-    active = False
-else:
-    active = True
+search_mode = createSearchMode()
 
 for filename in sorted( os.listdir( "." ) ):
-    if not filename.endswith( ".py" ) or filename == "run_all.py":
+    if not filename.endswith( ".py" ):
         continue
 
-    path = filename
+    if not decideFilenameVersionSkip(filename):
+        continue
 
-    if not active and start_at in ( filename, path ):
-        active = True
-
-    # Some syntax errors are for Python3 only.
-    if filename == "Importing2.py" and python_version < "3":
-        extra_flags = [ "remove_output" ]
-    elif filename == "GeneratorReturn2.py" and python_version >= "3.3":
-        extra_flags = [ "remove_output" ]
-    else:
-        extra_flags = [ "expect_failure",  "remove_output" ]
+    active = search_mode.consider(
+        dirname  = None,
+        filename = filename
+    )
 
     if active:
+        extra_flags = ["expect_failure", "remove_output"]
+
         compareWithCPython(
-            path        = path,
+            dirname     = None,
+            filename    = filename,
             extra_flags = extra_flags,
             search_mode = search_mode,
             needs_2to3  = False
         )
     else:
-        my_print( "Skipping", filename )
+        my_print("Skipping", filename)
+
+search_mode.finish()
\ No newline at end of file
